function [KsiTTg,PTTg] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,X,K1,K2,A,H,R,MT1,FT1,BT1,betavec,betaTT,M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel)
% AdaptiveLearningCondPredictionKalmanSmoother: Computes the smoothed estimates and covariances of the state variables for period T
%                                               taking the conditioning data into account over the prediction sample for the adaptive
%                                               learning version of the DSGE model. The function allows for a nonzero K2 matrix and
%                                               initial conditions for the conditioning assumptions.
%
% USAGE:
%
%       [KsiTTg,PTTg] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,X,K1,K2,A,H,R,MT1,FT1,BT1,betavec,betaTT,...
%                                                                    M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel)
%
% REQUIRED INPUT:  KsitT (matrix) of dimension rxg with smoothed values of the states at time
%                        T-g+1 until T.
%
%                  PtT (matrix) of dimension rxrxg with smoothed values of the state covariance at
%                        times T-g+1 until T.
%
%                  wtT (matrix) of dimension nxg with smooth values of the measurement errors
%                        at time T-g+1 until T.
%
%                  Z (matrix) of dimension mgxg with the conditioning assumptions.
%
%                  U (matrix) of dimension mgxg with initial conditions for the conditioning.
%
%                  X (matrix) of dimension kxg with ecogenous variables.
%
%                  K1 (matrix) of dimension nxm for mapping the observed variables into
%                        the conditioning data.
%
%                  K2 (matrix) of dimension nxm for mapping lags of the observed variables into
%                        the conditioning data. Matrix may be 3-dimensional where the 3rd
%                        allows for a lag-varying K2.
%
%                  A (matrix) of dimension kxn with the parameters on the exogenous variables in
%                        the measurement equation.
%
%                  H (matrix) of dimension rxn with parameters on the state variabels in the
%                        measurement equation.
%
%                  R (matrix) of dimension nxn with the covariances of the measurement errors
%                        in the measurement equation.
%
%                  MT1 (matrix) of dimension rxk with coefficients on the deterministic variables
%                         in the state equation used in the last period of the historical sample.
%
%                  FT1 (matrix) of dimension rxr with coefficients on the lagged state variables
%                         in the state equation used in the last period of the historical sample.
%
%                  BT1 (matrix) of dimension rxq with coefficients on the structural shocks in the
%                         state equation used in the last period of the historical sample.
%
%                  betavec  of dimension gstarx1 with the initial values for the beta-vector of the
%                         belief equations.
%
%                  betaTT (vector) of dimension gstarx1 with the updated belief coefficients computed
%                         at the end of period T.
%
%                  M (matrix) of dimension gstar x gstar with autoregressive coefficients for the
%                         belief coefficients.
%
%                  Hstar (matrix) of dimension r x r with the coefficient values on the contemporaneosu state variables.
%
%                  Hstarlag (matrix) of dimension r x r with the coefficient values on the lagged state variables.
%
%                  Hstar1f (matrix) of dimension r x f with the coefficient values on the expectated future values
%                         of the forward looking variables.
%
%                  D (matrix) of dimension r x q with the coefficient values on the structural shocks.
%
%                  Gtilde (matrix) of dimension gf x k+(r*m) which links vec of qlag (g x f) to a stacking
%                         of the deterministic variables and m lags of the state variables.
%
%                  Kgf (matrix) with the gf x gf commutation matrix such that vec(A') = Kgf*vec(A) when
%                         A is a g x f matrix.
%
%                  m (integer) with the maximum lag for the state variables in the belief equations.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
% REQUIRED OUTPUT: KsiTTg (vector) of dimension rx1 with the smooth estimates of the state variables
%                         at T conditional on the data and the conditioning assumptions.
%
%                  PTTg (matrix) of dimension rxr with the covariances for the smooth estimates of the
%                         state variables at T conditional on the data and the conditioning assumptions.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 18, 2024.
%                        This version: April 19, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-04-2024 Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
KsiTTg = [];
PTTg = [];
%
% some important dimensional parameters
%
k = size(X,1);
[r,n] = size(H);
q = size(BT1,2);
[mg,g] = size(Z);
%
if length(size(H))==2;
   if DSGEModel.FixedBeliefsForProj==1;
      %
      % we have fixed belief coefficients
      %
      FStar = [FT1 zeros(r,n);zeros(n,r+n)];
      QStar = [BT1*BT1' zeros(r,n);zeros(n,r) R];
      HStar = [H;eye(n)];
      if k>0;
         MStar = [MT1;zeros(n,k)];
      end;
      if g>1;
         if length(size(K2))==2;
            KStar = [K1;kron(ones(g-1,1),K2)];
         else;
            KStar = K1;
            for j=1:g-1;
               if j<=size(K2,3);
                  KStar = [KStar;K2(:,:,j)];
               else;
                  KStar = [KStar;zeros(size(K1))];
               end;
            end;
         end;
         FTilde = [FStar zeros(r+n,(g-1)*(r+n));eye((g-1)*(r+n)) zeros((g-1)*(r+n),r+n)];
         QTilde = [QStar zeros(r+n,(g-1)*(r+n));zeros((g-1)*(r+n),g*(r+n))];
         if k>0;
            MTilde = [MStar;zeros((g-1)*(r+n),k)];
         end;
      else;
         KStar = K1;
         FTilde = FStar;
         QTilde = QStar;
         if k>0;
            MTilde = Mstar;
         end;
      end;
      HTilde = kron(eye(g),HStar)*KStar;
      %
      % parameters on exogenous variables
      %
      if k>0;
         Xmat = zeros(n*g,g);
         for i=1:g;
            for j=1:g;
               Xmat(1+((j-1)*n):j*n,i) = A'*X(:,g+i-j);
            end;
         end;
         ATilde = [KStar;eye(mg)];
         XStar = ATilde'*[Xmat;U];
      else;
         XStar = U;
      end;
      %
      % make sure that missing obs for measurement errors are zero
      %
      wtT(isnan(wtT)) = 0;
      %
      % set up the initial values
      %
      KsiTTStar = zeros(g*(r+n),1);
      PTTTilde = zeros(g*(r+n),g*(r+n));
      for i=1:g;
         KsiTTStar(1+((i-1)*(r+n)):i*(r+n),1) = [KsitT(:,g-i+1);wtT(:,g-i+1)];
         PTTTilde(1+((i-1)*(r+n)):i*(r+n),1+((i-1)*(r+n)):i*(r+n)) = [PtT(:,:,g-i+1) zeros(r,n);zeros(n,r) R];
      end;
      %
      % dealing with off-diagonal blocks
      %
      for i=1:g;
         Fpower = FStar;
         for j=i+1:g;
            PMat = Fpower*PTTTilde(1+((j-i-1)*(r+n)):(j-i)*(r+n),1+((j-i-1)*(r+n)):(j-i)*(r+n));
            PTTTilde(1+((i-1)*(r+n)):i*(r+n),1+((j-1)*(r+n)):j*(r+n)) = PMat;
            PTTTilde(1+((j-1)*(r+n)):j*(r+n),1+((i-1)*(r+n)):i*(r+n)) = PMat';
            Fpower = Fpower*FStar;
         end;
      end;
      %
      % initialize forecast, update, and smoother matrices
      %
      KsiStarTT1 = zeros(g*(r+n),g);
      KsiStarTT = KsiStarTT1;
      KsiStarTTh = KsiStarTT1;
      PTildeTT1 = zeros(g*(r+n),g*(r+n),g);
      PTildeTT = PTildeTT1;
      PTildeTTh = PTildeTT1;
      Zpred = zeros(mg,g);
      %
      % update the initial conditions
      %
      if k>0;
         KsiStarTT1(:,1) = (MTilde*X(:,g))+(FTilde*KsiTTStar);
      else;
         KsiStarTT1(:,1) = FTilde*KsiTTStar;
      end;
      PTildeTT1(:,:,1) = (FTilde*PTTTilde*FTilde')+QTilde;
      Zpred(:,1) = XStar(:,1)+(HTilde'*KsiStarTT1(:,1));
      %
      % run the Kalman forecast and update parts
      %
      for t=2:g;
         pos = ~isnan(Z(:,t-1));
         n1 = sum(pos);
         if n1>0;
            InvMSEZ = eye(n1)/(HTilde(:,pos)'*PTildeTT1(:,:,t-1)*HTilde(:,pos));
            ZError = Z(pos,t-1)-Zpred(pos,t-1);
            KsiStarTT(:,t-1) = KsiStarTT1(:,t-1)+(PTildeTT1(:,:,t-1)*HTilde(:,pos)*InvMSEZ*ZError);
            PTildeTT(:,:,t-1) = PTildeTT1(:,:,t-1)-(PTildeTT1(:,:,t-1)*HTilde(:,pos)*InvMSEZ*HTilde(:,pos)'*PTildeTT1(:,:,t-1));
            %
            % calculate the Kalman gain
            %
            KalmanGain = FTilde*PTildeTT1(:,:,t-1)*HTilde(:,pos)*InvMSEZ;
            %
            % the state vector
            %
            if k>0;
               KsiStarTT1(:,t) = (MTilde*X(:,t+g-1))+(FTilde*KsiStarTT1(:,t-1))+(KalmanGain*ZError);
            else;
               KsiStarTT1(:,t) = (FTilde*KsiStarTT1(:,t-1))+(KalmanGain*ZError);
            end;
            FTildeDev = FTilde-(KalmanGain*HTilde(:,pos)');
            PTildeTT1(:,:,t) = (FTildeDev*PTildeTT1(:,:,t-1)*FTildeDev')+QTilde;
            Zpred(:,t) = XStar(:,t)+(HTilde'*KsiStarTT1(:,t));
         else;
            KsiStarTT(:,t-1) = KsiStarTT1(:,t-1);
            PTildeTT(:,:,t-1) = PTildeTT1(:,:,t-1);
            if k>0;
               KsiStarTT1(:,t) = (MTilde*X(:,t+g-1))+(FTilde*KsiStarTT1(:,t-1));
            else;
               KsiStarTT1(:,t) = FTilde*KsiStarTT1(:,t-1);
            end;
            PTildeTT1(:,:,t) = (FTilde*PTildeTT1(:,:,t-1)*FTilde')+QTilde;
            Zpred(:,t) = XStar(:,t)+(HTilde'*KsiStarTT1(:,t));
         end;
      end;
   else;
      %
      % belief coefficients are projected forward
      %
      HStar = [H;eye(n)];
      if g>1;
         if length(size(K2))==2;
            KStar = [K1;kron(ones(g-1,1),K2)];
         else;
            KStar = K1;
            for j=1:g-1;
               if j<=size(K2,3);
                  KStar = [KStar;K2(:,:,j)];
               else;
                  KStar = [KStar;zeros(size(K1))];
               end;
            end;
         end;
      else;
         KStar = K1;
      end;
      HTilde = kron(eye(g),HStar)*KStar;
      %
      % parameters on exogenous variables
      %
      if k>0;
         Xmat = zeros(n*g,g);
         for i=1:g;
            for j=1:g;
               Xmat(1+((j-1)*n):j*n,i) = A'*X(:,g+i-j);
            end;
         end;
         ATilde = [KStar;eye(mg)];
         XStar = ATilde'*[Xmat;U];
      else;
         XStar = U;
      end;
      %
      % make sure that missing obs for measurement errors are zero
      %
      wtT(isnan(wtT)) = 0;
      %
      % set up the initial values
      %
      KsiTTStar = zeros(g*(r+n),1);
      PTTTilde = zeros(g*(r+n),g*(r+n));
      for i=1:g;
         KsiTTStar(1+((i-1)*(r+n)):i*(r+n),1) = [KsitT(:,g-i+1);wtT(:,g-i+1)];
         PTTTilde(1+((i-1)*(r+n)):i*(r+n),1+((i-1)*(r+n)):i*(r+n)) = [PtT(:,:,g-i+1) zeros(r,n);zeros(n,r) R];
      end;
      %
      % define the 3D matrices that take time variation of the beliefs into account
      %
      FStar = zeros(r+n,r+n,g);
      QStar = zeros(r+n,r+n,g);
      if k>0;
         MStar = zeros(r+n,k,g);
         MTilde = zeros(g*(r+n),k,g);
      end;
      FTilde = zeros(g*(r+n),g*(r+n),g);
      QTilde = zeros(g*(r+n),g*(r+n),g);
      for i=1:g;
         FStar(:,:,i) = [FT1 zeros(r,n);zeros(n,r+n)];
         QStar(:,:,i) = [BT1*BT1' zeros(r,n);zeros(n,r) R];
         if k>0;
            MStar(:,:,i) = [MT1;zeros(n,k)];
            MTilde(1:(r+n),:,i) = MStar;
         end;
         if g>1;
            FTilde(:,:,i) = [FStar zeros(r+n,(g-1)*(r+n));eye((g-1)*(r+n)) zeros((g-1)*(r+n),r+n)];
            QTilde(1:(r+n),1:(r+n),i) = QStar;
         else;
            FTilde = FStar;
            QTilde = QSTar;
         end;
         %
         % compute new solution matrices
         %
         if i<g;
            if i>1;
               betaTT = betavec+(M*(betaTT-betavec));
            end;
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MT1,FT1,BT1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTT);
            else;
               [MT1,FT1,BT1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         end;
      end;
      %
      % dealing with off-diagonal blocks
      %
      for i=1:g;
         Fpower = FStar(:,:,i);
         for j=i+1:g;
            PMat = Fpower*PTTTilde(1+((j-i-1)*(r+n)):(j-i)*(r+n),1+((j-i-1)*(r+n)):(j-i)*(r+n));
            PTTTilde(1+((i-1)*(r+n)):i*(r+n),1+((j-1)*(r+n)):j*(r+n)) = PMat;
            PTTTilde(1+((j-1)*(r+n)):j*(r+n),1+((i-1)*(r+n)):i*(r+n)) = PMat';
            Fpower = FStar(:,:,j)*Fpower;
         end;
      end;
      %
      % initialize forecast, update, and smoother matrices
      %
      KsiStarTT1 = zeros(g*(r+n),g);
      KsiStarTT = KsiStarTT1;
      KsiStarTTh = KsiStarTT1;
      PTildeTT1 = zeros(g*(r+n),g*(r+n),g);
      PTildeTT = PTildeTT1;
      PTildeTTh = PTildeTT1;
      Zpred = zeros(mg,g);
      %
      % update the initial conditions
      %
      if k>0;
         KsiStarTT1(:,1) = (MTilde(:,:,1)*X(:,g))+(FTilde(:,:,1)*KsiTTStar);
      else;
         KsiStarTT1(:,1) = FTilde(:,:,1)*KsiTTStar;
      end;
      PTildeTT1(:,:,1) = (FTilde(:,:,1)*PTTTilde*FTilde(:,:,1)')+QTilde(:,:,1);
      Zpred(:,1) = XStar(:,1)+(HTilde'*KsiStarTT1(:,1));
      %
      % run the Kalman forecast and update parts
      %
      for t=2:g;
         pos = ~isnan(Z(:,t-1));
         n1 = sum(pos);
         if n1>0;
            InvMSEZ = eye(n1)/(HTilde(:,pos)'*PTildeTT1(:,:,t-1)*HTilde(:,pos));
            ZError = Z(pos,t-1)-Zpred(pos,t-1);
            KsiStarTT(:,t-1) = KsiStarTT1(:,t-1)+(PTildeTT1(:,:,t-1)*HTilde(:,pos)*InvMSEZ*ZError);
            PTildeTT(:,:,t-1) = PTildeTT1(:,:,t-1)-(PTildeTT1(:,:,t-1)*HTilde(:,pos)*InvMSEZ*HTilde(:,pos)'*PTildeTT1(:,:,t-1));
            %
            % calculate the Kalman gain
            %
            KalmanGain = FTilde(:,:,t)*PTildeTT1(:,:,t-1)*HTilde(:,pos)*InvMSEZ;
            %
            % the state vector
            %
            if k>0;
               KsiStarTT1(:,t) = (MTilde(:,:,t)*X(:,t+g-1))+(FTilde(:,:,t)*KsiStarTT1(:,t-1))+(KalmanGain*ZError);
            else;
               KsiStarTT1(:,t) = (FTilde(:,:,t)*KsiStarTT1(:,t-1))+(KalmanGain*ZError);
            end;
            FTildeDev = FTilde(:,:,t)-(KalmanGain*HTilde(:,pos)');
            PTildeTT1(:,:,t) = (FTildeDev*PTildeTT1(:,:,t-1)*FTildeDev')+QTilde(:,:,t);
            Zpred(:,t) = XStar(:,t)+(HTilde'*KsiStarTT1(:,t));
         else;
            KsiStarTT(:,t-1) = KsiStarTT1(:,t-1);
            PTildeTT(:,:,t-1) = PTildeTT1(:,:,t-1);
            if k>0;
               KsiStarTT1(:,t) = (MTilde(:,:,t)*X(:,t+g-1))+(FTilde(:,:,t)*KsiStarTT1(:,t-1));
            else;
               KsiStarTT1(:,t) = FTilde(:,:,t)*KsiStarTT1(:,t-1);
            end;
            PTildeTT1(:,:,t) = (FTilde(:,:,t)*PTildeTT1(:,:,t-1)*FTilde(:,:,t)')+QTilde(:,:,t);
            Zpred(:,t) = XStar(:,t)+(HTilde'*KsiStarTT1(:,t));
         end;
      end;
   end;
   %
   % the disturbance smother is the same for fixed beliefs and updated beliefs and fixed H
   %
   pos = ~isnan(Z(:,g));
   n1 = sum(pos);
   if n1>0;
      InvMSEZ = eye(n1)/(HTilde(:,pos)'*PTildeTT1(:,:,g)*HTilde(:,pos));
      ZError = Z(pos,g)-Zpred(pos,g);
      rg1g = HTilde(:,pos)*InvMSEZ*ZError;
      Ng1g = HTilde(:,pos)*InvMSEZ*HTilde(:,pos)';
   else;
      rg1g = 0;
      Ng1g = 0;
   end;
else;
   if DSGEModel.FixedBeliefsForProj==1;
      %
      % we have fixed belief coefficients
      %
      FStar = [FT1 zeros(r,n);zeros(n,r+n)];
      QStar = [BT1*BT1' zeros(r,n);zeros(n,r) R];
      HStar = zeros(r+n,n,size(H,3));
      for t=1:size(H,3);
         HStar(:,:,t) = [H(:,:,t);eye(n)];
      end;
      if k>0;
         MStar = [MT1;zeros(n,k)];
      end;
      if g>1;
         if length(size(K2))==2;
            KStar = [K1;kron(ones(g-1,1),K2)];
         else;
            KStar = K1;
            for j=1:g-1;
               if j<=size(K2,3);
                  KStar = [KStar;K2(:,:,j)];
               else;
                  KStar = [KStar;zeros(size(K1))];
               end;
            end;
         end;
         FTilde = [FStar zeros(r+n,(g-1)*(r+n));eye((g-1)*(r+n)) zeros((g-1)*(r+n),r+n)];
         QTilde = [QStar zeros(r+n,(g-1)*(r+n));zeros((g-1)*(r+n),g*(r+n))];
         if k>0;
            MTilde = [MStar;zeros((g-1)*(r+n),k)];
         end;
      else;
         KStar = K1;
         FTilde = FStar;
         QTilde = QStar;
         if k>0;
            MTilde = Mstar;
         end;
      end;
      HTilde = zeros((n+r)*g,size(KStar,2),g);
      for i=1:g;
         HHelpTilde = zeros((n+r)*g,n*g);
         for j=1:g;
            HHelpTilde(1+((n+r)*(j-1)):(n+r)*j,1+(n*(j-1)):n*j) = HStar(:,:,g+(i-1)-(j-1));
         end;
         HTilde(:,:,i) = HHelpTilde*KStar;
      end;
      %
      % parameters on exogenous variables
      %
      if k>0;
         Xmat = zeros(n*g,g);
         for i=1:g;
            for j=1:g;
               Xmat(1+((j-1)*n):j*n,i) = A'*X(:,g+i-j);
            end;
         end;
         ATilde = [KStar;eye(mg)];
         XStar = ATilde'*[Xmat;U];
      else;
         XStar = U;
      end;
      %
      % make sure that missing obs for measurement errors are zero
      %
      wtT(isnan(wtT)) = 0;
      %
      % set up the initial values
      %
      KsiTTStar = zeros(g*(r+n),1);
      PTTTilde = zeros(g*(r+n),g*(r+n));
      for i=1:g;
         KsiTTStar(1+((i-1)*(r+n)):i*(r+n),1) = [KsitT(:,g-i+1);wtT(:,g-i+1)];
         PTTTilde(1+((i-1)*(r+n)):i*(r+n),1+((i-1)*(r+n)):i*(r+n)) = [PtT(:,:,g-i+1) zeros(r,n);zeros(n,r) R];
      end;
      %
      % dealing with off-diagonal blocks
      %
      for i=1:g;
         Fpower = FStar;
         for j=i+1:g;
            PMat = Fpower*PTTTilde(1+((j-i-1)*(r+n)):(j-i)*(r+n),1+((j-i-1)*(r+n)):(j-i)*(r+n));
            PTTTilde(1+((i-1)*(r+n)):i*(r+n),1+((j-1)*(r+n)):j*(r+n)) = PMat;
            PTTTilde(1+((j-1)*(r+n)):j*(r+n),1+((i-1)*(r+n)):i*(r+n)) = PMat';
            Fpower = Fpower*FStar;
         end;
      end;
      %
      % initialize forecast, update, and smoother matrices
      %
      KsiStarTT1 = zeros(g*(r+n),g);
      KsiStarTT = KsiStarTT1;
      KsiStarTTh = KsiStarTT1;
      PTildeTT1 = zeros(g*(r+n),g*(r+n),g);
      PTildeTT = PTildeTT1;
      PTildeTTh = PTildeTT1;
      Zpred = zeros(mg,g);
      %
      % update the initial conditions
      %
      if k>0;
         KsiStarTT1(:,1) = (MTilde*X(:,g))+(FTilde*KsiTTStar);
      else;
         KsiStarTT1(:,1) = FTilde*KsiTTStar;
      end;
      PTildeTT1(:,:,1) = (FTilde*PTTTilde*FTilde')+QTilde;
      Zpred(:,1) = XStar(:,1)+(HTilde(:,:,1)'*KsiStarTT1(:,1));
      %
      % run the Kalman forecast and update parts
      %
      for t=2:g;
         pos = ~isnan(Z(:,t-1));
         n1 = sum(pos);
         if n1>0;
            InvMSEZ = eye(n1)/(HTilde(:,pos,t-1)'*PTildeTT1(:,:,t-1)*HTilde(:,pos,t-1));
            ZError = Z(pos,t-1)-Zpred(pos,t-1);
            KsiStarTT(:,t-1) = KsiStarTT1(:,t-1)+(PTildeTT1(:,:,t-1)*HTilde(:,pos,t-1)*InvMSEZ*ZError);
            PTildeTT(:,:,t-1) = PTildeTT1(:,:,t-1)-(PTildeTT1(:,:,t-1)*HTilde(:,pos,t-1)*InvMSEZ*HTilde(:,pos,t-1)'*PTildeTT1(:,:,t-1));
            %
            % calculate the Kalman gain
            %
            KalmanGain = FTilde*PTildeTT1(:,:,t-1)*HTilde(:,pos,t-1)*InvMSEZ;
            %
            % the state vector
            %
            if k>0;
               KsiStarTT1(:,t) = (MTilde*X(:,t+g-1))+(FTilde*KsiStarTT1(:,t-1))+(KalmanGain*ZError);
            else;
               KsiStarTT1(:,t) = (FTilde*KsiStarTT1(:,t-1))+(KalmanGain*ZError);
            end;
            FTildeDev = FTilde-(KalmanGain*HTilde(:,pos,t-1)');
            PTildeTT1(:,:,t) = (FTildeDev*PTildeTT1(:,:,t-1)*FTildeDev')+QTilde;
            Zpred(:,t) = XStar(:,t)+(HTilde(:,:,t)'*KsiStarTT1(:,t));
         else;
            KsiStarTT(:,t-1) = KsiStarTT1(:,t-1);
            PTildeTT(:,:,t-1) = PTildeTT1(:,:,t-1);
            if k>0;
               KsiStarTT1(:,t) = (MTilde*X(:,t+g-1))+(FTilde*KsiStarTT1(:,t-1));
            else;
               KsiStarTT1(:,t) = FTilde*KsiStarTT1(:,t-1);
            end;
            PTildeTT1(:,:,t) = (FTilde*PTildeTT1(:,:,t-1)*FTilde')+QTilde;
            Zpred(:,t) = XStar(:,t)+(HTilde(:,:,t)'*KsiStarTT1(:,t));
         end;
      end;
   else;
      %
      % belief coefficients are projected forward
      %
      HStar = zeros(r+n,n,size(H,3));
      for t=1:size(H,3);
         HStar(:,:,t) = [H(:,:,t);eye(n)];
      end;
      if g>1;
         if length(size(K2))==2;
            KStar = [K1;kron(ones(g-1,1),K2)];
         else;
            KStar = K1;
            for j=1:g-1;
               if j<=size(K2,3);
                  KStar = [KStar;K2(:,:,j)];
               else;
                  KStar = [KStar;zeros(size(K1))];
               end;
            end;
         end;
      else;
         KStar = K1;
      end;
      HTilde = zeros((n+r)*g,size(KStar,2),g);
      for i=1:g;
         HHelpTilde = zeros((n+r)*g,n*g);
         for j=1:g;
            HHelpTilde(1+((n+r)*(j-1)):(n+r)*j,1+(n*(j-1)):n*j) = HStar(:,:,g+(i-1)-(j-1));
         end;
         HTilde(:,:,i) = HHelpTilde*KStar;
      end;
      %
      % parameters on exogenous variables
      %
      if k>0;
         Xmat = zeros(n*g,g);
         for i=1:g;
            for j=1:g;
               Xmat(1+((j-1)*n):j*n,i) = A'*X(:,g+i-j);
            end;
         end;
         ATilde = [KStar;eye(mg)];
         XStar = ATilde'*[Xmat;U];
      else;
         XStar = U;
      end;
      %
      % make sure that missing obs for measurement errors are zero
      %
      wtT(isnan(wtT)) = 0;
      %
      % set up the initial values
      %
      KsiTTStar = zeros(g*(r+n),1);
      PTTTilde = zeros(g*(r+n),g*(r+n));
      for i=1:g;
         KsiTTStar(1+((i-1)*(r+n)):i*(r+n),1) = [KsitT(:,g-i+1);wtT(:,g-i+1)];
         PTTTilde(1+((i-1)*(r+n)):i*(r+n),1+((i-1)*(r+n)):i*(r+n)) = [PtT(:,:,g-i+1) zeros(r,n);zeros(n,r) R];
      end;
      %
      % define the 3D matrices that take time variation of the beliefs into account
      %
      FStar = zeros(r+n,r+n,g);
      QStar = zeros(r+n,r+n,g);
      if k>0;
         MStar = zeros(r+n,k,g);
         MTilde = zeros(g*(r+n),k,g);
      end;
      FTilde = zeros(g*(r+n),g*(r+n),g);
      QTilde = zeros(g*(r+n),g*(r+n),g);
      for i=1:g;
         FStar(:,:,i) = [FT1 zeros(r,n);zeros(n,r+n)];
         QStar(:,:,i) = [BT1*BT1' zeros(r,n);zeros(n,r) R];
         if k>0;
            MStar(:,:,i) = [MT1;zeros(n,k)];
            MTilde(1:(r+n),:,i) = MStar;
         end;
         if g>1;
            FTilde(:,:,i) = [FStar zeros(r+n,(g-1)*(r+n));eye((g-1)*(r+n)) zeros((g-1)*(r+n),r+n)];
            QTilde(1:(r+n),1:(r+n),i) = QStar;
         else;
            FTilde = FStar;
            QTilde = QSTar;
         end;
         %
         % compute new solution matrices
         %
         if i<g;
            if i>1;
               betaTT = betavec+(M*(betaTT-betavec));
            end;
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MT1,FT1,BT1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTT);
            else;
               [MT1,FT1,BT1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         end;
      end;
      %
      % dealing with off-diagonal blocks
      %
      for i=1:g;
         Fpower = FStar(:,:,i);
         for j=i+1:g;
            PMat = Fpower*PTTTilde(1+((j-i-1)*(r+n)):(j-i)*(r+n),1+((j-i-1)*(r+n)):(j-i)*(r+n));
            PTTTilde(1+((i-1)*(r+n)):i*(r+n),1+((j-1)*(r+n)):j*(r+n)) = PMat;
            PTTTilde(1+((j-1)*(r+n)):j*(r+n),1+((i-1)*(r+n)):i*(r+n)) = PMat';
            Fpower = FStar(:,:,j)*Fpower;
         end;
      end;
      %
      % initialize forecast, update, and smoother matrices
      %
      KsiStarTT1 = zeros(g*(r+n),g);
      KsiStarTT = KsiStarTT1;
      KsiStarTTh = KsiStarTT1;
      PTildeTT1 = zeros(g*(r+n),g*(r+n),g);
      PTildeTT = PTildeTT1;
      PTildeTTh = PTildeTT1;
      Zpred = zeros(mg,g);
      %
      % update the initial conditions
      %
      if k>0;
         KsiStarTT1(:,1) = (MTilde(:,:,1)*X(:,g))+(FTilde(:,:,1)*KsiTTStar);
      else;
         KsiStarTT1(:,1) = FTilde(:,:,1)*KsiTTStar;
      end;
      PTildeTT1(:,:,1) = (FTilde(:,:,1)*PTTTilde*FTilde(:,:,1)')+QTilde(:,:,1);
      Zpred(:,1) = XStar(:,1)+(HTilde(:,:,1)'*KsiStarTT1(:,1));
      %
      % run the Kalman forecast and update parts
      %
      for t=2:g;
         pos = ~isnan(Z(:,t-1));
         n1 = sum(pos);
         if n1>0;
            InvMSEZ = eye(n1)/(HTilde(:,pos,t-1)'*PTildeTT1(:,:,t-1)*HTilde(:,pos,t-1));
            ZError = Z(pos,t-1)-Zpred(pos,t-1);
            KsiStarTT(:,t-1) = KsiStarTT1(:,t-1)+(PTildeTT1(:,:,t-1)*HTilde(:,pos,t-1)*InvMSEZ*ZError);
            PTildeTT(:,:,t-1) = PTildeTT1(:,:,t-1)-(PTildeTT1(:,:,t-1)*HTilde(:,pos,t-1)*InvMSEZ*HTilde(:,pos,t-1)'*PTildeTT1(:,:,t-1));
            %
            % calculate the Kalman gain
            %
            KalmanGain = FTilde(:,:,t)*PTildeTT1(:,:,t-1)*HTilde(:,pos,t-1)*InvMSEZ;
            %
            % the state vector
            %
            if k>0;
               KsiStarTT1(:,t) = (MTilde(:,:,t)*X(:,t+g-1))+(FTilde(:,:,t)*KsiStarTT1(:,t-1))+(KalmanGain*ZError);
            else;
               KsiStarTT1(:,t) = (FTilde(:,:,t)*KsiStarTT1(:,t-1))+(KalmanGain*ZError);
            end;
            FTildeDev = FTilde(:,:,t)-(KalmanGain*HTilde(:,pos,t-1)');
            PTildeTT1(:,:,t) = (FTildeDev*PTildeTT1(:,:,t-1)*FTildeDev')+QTilde(:,:,t);
            Zpred(:,t) = XStar(:,t)+(HTilde(:,:,t)'*KsiStarTT1(:,t));
         else;
            KsiStarTT(:,t-1) = KsiStarTT1(:,t-1);
            PTildeTT(:,:,t-1) = PTildeTT1(:,:,t-1);
            if k>0;
               KsiStarTT1(:,t) = (MTilde(:,:,t)*X(:,t+g-1))+(FTilde(:,:,t)*KsiStarTT1(:,t-1));
            else;
               KsiStarTT1(:,t) = FTilde(:,:,t)*KsiStarTT1(:,t-1);
            end;
            PTildeTT1(:,:,t) = (FTilde(:,:,t)*PTildeTT1(:,:,t-1)*FTilde(:,:,t)')+QTilde(:,:,t);
            Zpred(:,t) = XStar(:,t)+(HTilde(:,:,t)'*KsiStarTT1(:,t));
         end;
      end;
   end;
   pos = ~isnan(Z(:,g));
   n1 = sum(pos);
   if n1>0;
      InvMSEZ = eye(n1)/(HTilde(:,pos,g)'*PTildeTT1(:,:,g)*HTilde(:,pos,g));
      ZError = Z(pos,g)-Zpred(pos,g);
      rg1g = HTilde(:,pos,g)*InvMSEZ*ZError;
      Ng1g = HTilde(:,pos,g)*InvMSEZ*HTilde(:,pos,g)';
   else;
      rg1g = 0;
      Ng1g = 0;
   end;
end;
%
KsiStarg1g = KsiStarTT1(:,g)+(PTildeTT1(:,:,g)*rg1g);
PStarg1g = PTildeTT1(:,:,g)-(PTildeTT1(:,:,g)*Ng1g*PTildeTT1(:,:,g));
KsiTTg = KsiStarg1g(1+((g-1)*(r+n)):r+((g-1)*(r+n)));
PTTg = PStarg1g(1+((g-1)*(r+n)):r+((g-1)*(r+n)),1+((g-1)*(r+n)):r+((g-1)*(r+n)));

%
% end of AdaptiveLearningCondPredictionKalmanSmoother.m
%
