function [KsiTTg,PTTg] = AdaptiveLearningCondPredictionSmoother(KsiTT1,PTT1,YhatT,YT,XPred,Z,K1,A,H,R,MT1,FT1,BT1,betavec,betaTT,M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel)
% AdaptiveLearningCondPredictionSmoother: Computes the smoothed estimates and covariances of the state variables for period T
%                                         taking the conditioning data into account over the prediction sample for the adaptive
%                                         learning version of the DSGE model.
%
% USAGE:
%
%       [KsiTTg,PTTg] = AdaptiveLearningCondPredictionSmoother(KsiTT1,PTT1,YhatT,YT,XPred,Z,K1,A,H,R,MT1,FT1,BT1,betavec,...
%                                                              betaTT,M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel)
%
% REQUIRED INPUT:  KsiTT1 (vector) of dimension rx1 with the forecast estimates of the state variables at period
%                         T using data until period T-1.
%
%                  PTT1 (matrix) of dimension rxr with the forecast estimates of the state variable covariances
%                         at period T using data until period T-1.
%
%                  KsiTT (vector) of dimension rx1 with the smooth estimates of the state variables at period
%                         T using data until period T.
%
%                  PTT (matrix) of dimension rxr with smooth estimates of the state variable covariance at
%                         period T using data until period T.
%
%                  YhatT (vector) of dimension nx1 with forecast estimates of the observed variables at T
%                         using data until period T-1.
%
%                  YT (vector) of dimension nx1 with the observed variables for period T.
%
%                  XPred (matrix) of dimension kxg with data on the exoegnous variables over the conditioning
%                         horizon.
%
%                  Z (matrix) of dimension mgxg with the conditioning assumptions.
%
%                  K1 (matrix) of dimension nxm for mapping the observed variables into the conditioning
%                         assumptions.
%
%                  A (matrix) of dimension kxn with the parameters on the exogenous variables in
%                         the measurement equation.
%
%                  H (matrix) of dimension rxn (xg) with parameters on the state variabels in the
%                         measurement equation.
%
%                  R (matrix) of dimension nxn with the covariances of the measurement errors
%                         in the measurement equation.
%
%                  MT1 (matrix) of dimension rxk with coefficients on the deterministic variables
%                         in the state equation used in the last period of the historical sample.
%
%                  FT1 (matrix) of dimension rxr with coefficients on the lagged state variables
%                         in the state equation used in the last period of the historical sample.
%
%                  BT1 (matrix) of dimension rxq with coefficients on the structural shocks in the
%                         state equation used in the last period of the historical sample.
%
%                  betavec  of dimension gstarx1 with the initial values for the beta-vector of the
%                         belief equations.
%
%                  betaTT (vector) of dimension gstarx1 with the updated belief coefficients computed
%                         at the end of period T.
%
%                  M (matrix) of dimension gstar x gstar with autoregressive coefficients for the
%                         belief coefficients.
%
%                  Hstar (matrix) of dimension r x r with the coefficient values on the contemporaneosu state variables.
%
%                  Hstarlag (matrix) of dimension r x r with the coefficient values on the lagged state variables.
%
%                  Hstar1f (matrix) of dimension r x f with the coefficient values on the expectated future values
%                         of the forward looking variables.
%
%                  D (matrix) of dimension r x q with the coefficient values on the structural shocks.
%
%                  Gtilde (matrix) of dimension gf x k+(r*m) which links vec of qlag (g x f) to a stacking
%                         of the deterministic variables and m lags of the state variables.
%
%                  Kgf (matrix) with the gf x gf commutation matrix such that vec(A') = Kgf*vec(A) when
%                         A is a g x f matrix.
%
%                  m (integer) with the maximum lag for the state variables in the belief equations.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
% REQUIRED OUTPUT: KsiTTg (vector) of dimension rx1 with the smooth estimates of the state variables
%                         at T conditional on the data and the conditioning assumptions.
%
%                  PTTg (matrix) of dimension rxr with the covariances for the smooth estimates of the
%                         state variables at T conditional on the data and the conditioning assumptions.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 16, 2024.
%                        This version: April 18, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 17-04-2024: Updated the documentation.
%
% * 18-04-2024: Added the input variable m with the maximum lag of state variables in the belief equations.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
KsiTTg = [];
PTTg = [];
%
% initialize
%
KsiTTg = [];
PTTg = [];
%
% determine parameters
%
n = length(YT);
r = length(KsiTT1);
k = size(XPred,1);
Q = BT1*BT1';
q = size(BT1,2);
[mg,g] = size(Z);
KsiLast = FT1*KsiTT1;
PLast = (FT1*PTT1*FT1')+Q;
Im = eye(mg);
%
% check if the standard Kalman filtering routines should be applied or the
% square root filtering routines
%
zError = zeros(mg,g);
InvMSEZ = zeros(mg,mg,g);
if k>0;
   Ak = K1'*A'*XPred;
else;
   Ak = zeros(mg,g);
end;
%
if length(size(H))==2;
   if DSGEModel.FixedBeliefsForProj==1;
      %
      % we have fixed belief coefficients
      %
      LZ = zeros(r,r,g);
      Hstar = H*K1;
      HstarT = Hstar';
      Rstar = K1'*R*K1;
      %
      % run the filter over the conditioning horizon
      %
      for t=1:g;
         pos = ~isnan(Z(:,t));
         n1 = sum(pos);
         if n1>0;
            PLastH = PLast*Hstar(:,pos);
            InvMSEZ(pos,pos,t) = Im(pos,pos)/((HstarT(pos,:)*PLastH)+Rstar(pos,pos));
            zPred = Ak(:,t)+(HstarT*KsiLast);
            zError(pos,t) = Z(pos,t)-zPred(pos,1);
            KalmanGain = FT1*PLastH*InvMSEZ(pos,pos,t);
            KsiLast = (MT1*XPred(:,t))+(FT1*KsiLast)+(KalmanGain*zError(pos,t));
            LZ(:,:,t) = FT1-(KalmanGain*HstarT(pos,:));
            PLast = (LZ(:,:,t)*PLast*LZ(:,:,t)')+(KalmanGain*Rstar(pos,pos)*KalmanGain')+Q;
         else;
            KsiLast = (MT1*XPred(:,t))+(FT1*KsiLast);
            LZ(:,:,t) = FT1;
            PLast = (LZ(:,:,t)*PLast*LZ(:,:,t)')+Q;
         end;
      end;
   else;
      %
      % belief coefficients are projected forward
      %
      LZ = zeros(r,r,g);
      Hstar = H*K1;
      HstarT = Hstar';
      Rstar = K1'*R*K1;
      %
      % run the filter over the conditioning horizon
      %
      for t=1:g;
         pos = ~isnan(Z(:,t));
         n1 = sum(pos);
         if n1>0;
            PLastH = PLast*Hstar(:,pos);
            InvMSEZ(pos,pos,t) = Im(pos,pos)/((HstarT(pos,:)*PLastH)+Rstar(pos,pos));
            zPred = Ak(:,t)+(HstarT*KsiLast);
            zError(pos,t) = Z(pos,t)-zPred(pos,1);
            KalmanGain = FT1*PLastH*InvMSEZ(pos,pos,t);
            KsiLast = (MT1*XPred(:,t))+(FT1*KsiLast)+(KalmanGain*zError(pos,t));
            LZ(:,:,t) = FT1-(KalmanGain*HstarT(pos,:));
            PLast = (LZ(:,:,t)*PLast*LZ(:,:,t)')+(KalmanGain*Rstar(pos,pos)*KalmanGain')+Q;
         else;
            KsiLast = (MT1*XPred(:,t))+(FT1*KsiLast);
            LZ(:,:,t) = FT1;
            PLast = (LZ(:,:,t)*PLast*LZ(:,:,t)')+Q;
         end;
         %
         % update the solution
         %
         if t<g;
            if t>1;
               betaTT = betavec+(M*(betaTT-betavec));
            end;
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MT1,FT1,BT1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTT);
            else;
               [MT1,FT1,BT1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         end;
      end;
   end;
   %
   % now run the disturbance smoother
   %
   rt = zeros(r,1);
   Nt = zeros(r,r);
   for t=g:-1:1;
      pos = ~isnan(Z(:,t));
      n1 = sum(pos);
      if n1>0;
         rt = (Hstar(:,pos)*InvMSEZ(pos,pos,t)*zError(pos,t))+(LZ(:,:,t)'*rt);
         Nt = (Hstar(:,pos)*InvMSEZ(pos,pos,t)*HstarT(pos,:))+(LZ(:,:,t)'*Nt*LZ(:,:,t));
      else;
         rt = LZ(:,:,t)'*rt;
         Nt = LZ(:,:,t)'*Nt*LZ(:,:,t);
      end;
   end;
else;
   if DSGEModel.FixedBeliefsForProj==1;
      %
      % we have fixed belief coefficients
      %
      LZ = zeros(r,r,g);
      Rstar = K1'*R*K1;
      %
      % run the filter over the conditioning horizon
      %
      for t=1:g;
         pos = ~isnan(Z(:,t));
         n1 = sum(pos);
         if n1>0;
            Hstar = H(:,:,t)*K1;
            HstarT = Hstar';
            PLastH = PLast*Hstar(:,pos);
            InvMSEZ(pos,pos,t) = Im(pos,pos)/((HstarT(pos,:)*PLastH)+Rstar(pos,pos));
            zPred = Ak(:,t)+(HstarT*KsiLast);
            zError(pos,t) = Z(pos,t)-zPred(pos,1);
            KalmanGain = FT1*PLastH*InvMSEZ(pos,pos,t);
            KsiLast = (MT1*XPred(:,t))+(FT1*KsiLast)+(KalmanGain*zError(pos,t));
            LZ(:,:,t) = FT1-(KalmanGain*HstarT(pos,:));
            PLast = (LZ(:,:,t)*PLast*LZ(:,:,t)')+(KalmanGain*Rstar(pos,pos)*KalmanGain')+Q;
         else;
            KsiLast = (MT1*XPred(:,t))+(FT1*KsiLast);
            LZ(:,:,t) = FT1;
            PLast = (LZ(:,:,t)*PLast*LZ(:,:,t)')+Q;
         end;
      end;
   else;
      %
      % belief coefficients are projected forward
      %
      LZ = zeros(r,r,g);
      Rstar = K1'*R*K1;
      %
      % run the filter over the conditioning horizon
      %
      for t=1:g;
         pos = ~isnan(Z(:,t));
         n1 = sum(pos);
         if n1>0;
            Hstar = H(:,:,t)*K1;
            HstarT = Hstar';
            PLastH = PLast*Hstar(:,pos);
            InvMSEZ(pos,pos,t) = Im(pos,pos)/((HstarT(pos,:)*PLastH)+Rstar(pos,pos));
            zPred = Ak(:,t)+(HstarT*KsiLast);
            zError(pos,t) = Z(pos,t)-zPred(pos,1);
            KalmanGain = FT1*PLastH*InvMSEZ(pos,pos,t);
            KsiLast = (MT1*XPred(:,t))+(FT1*KsiLast)+(KalmanGain*zError(pos,t));
            LZ(:,:,t) = FT1-(KalmanGain*HstarT(pos,:));
            PLast = (LZ(:,:,t)*PLast*LZ(:,:,t)')+(KalmanGain*Rstar(pos,pos)*KalmanGain')+Q;
         else;
            KsiLast = (MT1*XPred(:,t))+(FT1*KsiLast);
            LZ(:,:,t) = FT1;
            PLast = (LZ(:,:,t)*PLast*LZ(:,:,t)')+Q;
         end;
         %
         % update the solution
         %
         if t<g;
            if t>1;
               betaTT = betavec+(M*(betaTT-betavec));
            end;
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MT1,FT1,BT1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTT);
            else;
               [MT1,FT1,BT1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         end;
      end;
   end;
   %
   % now run the disturbance smoother
   %
   rt = zeros(r,1);
   Nt = zeros(r,r);
   for t=g:-1:1;
      pos = ~isnan(Z(:,t));
      n1 = sum(pos);
      if n1>0;
         Hstar = H(:,:,t)*K1;
         HstarT = Hstar';
         rt = (Hstar(:,pos)*InvMSEZ(pos,pos,t)*zError(pos,t))+(LZ(:,:,t)'*rt);
         Nt = (Hstar(:,pos)*InvMSEZ(pos,pos,t)*HstarT(pos,:))+(LZ(:,:,t)'*Nt*LZ(:,:,t));
      else;
         rt = LZ(:,:,t)'*rt;
         Nt = LZ(:,:,t)'*Nt*LZ(:,:,t);
      end;
   end;
end;
%
% compute r(T|T+g) and N(T|T+g), using r(T+1|T+g) and N(T+1|T+g), respectively,
% as input for the distrubance smoother
%
pos = ~isnan(YT);
n1 = sum(pos);
if length(size(H))==2;
   if n1>0;
      InvMSEY = eye(n1)/((H(:,pos)'*PTT1*H(:,pos))+R(pos,pos));
      LT = FT1-(FT1*PTT1*H(:,pos)*InvMSEY*H(:,pos)');
      rt = (H(:,pos)*InvMSEY*(YT(pos,1)-YhatT(pos,1)))+(LT'*rt);
      Nt = (H(:,pos)*InvMSEY*H(:,pos)')+(LT'*Nt*LT);
   else;
      rt = FT1'*rt;
      Nt = FT1'*Nt*FT1;
   end;
else;
   
end;
%
% now compute the smoothed values for
%
KsiTTg = KsiTT1+(PTT1*rt);
PTTg = PTT1-(PTT1*Nt*PTT1);

%
% end of AdaptiveLearningCondPredictionSmoother.m
%
