function [CorrData,TransStatus,InitialStatus,status] = CalculateSteadyStateCorrelations(theta,thetaPositions,ModelParameters,BeliefSys,FirstPeriod,LastPeriod,DSGEModel,CurrINI)
% CalculateSteadyStateCorrelations: Computes the steady-state observed and state variable autocorrelations for the
%                                   adaptive learning version of the DSGE model.
%
% USAGE:
%
%       [CorrData,TransStatus,InitialStatus,status] = CalculateSteadyStateCorrelations(theta,thetaPositions,ModelParameters,...
%                                                                        FirstPeriod,LastPeriod,BeliefSys,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector) with values of the original DSGE model parameters as well as the estimated
%                        parameters of the adaptive learning system.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function needs the
%                        former field to know which parameter name corresponds to position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                        DSGE model along with the adaptive learning system.
%
%                  FirstPeriod (integer), with the first period in the sample when the training sample size
%                        for the Kalman filter is not taken into account.
%
%                  LastPeriod (integer), with the last period in the sample.
%
%                  BeliefSys (structure) with fields "m" an integer denoting the maximum number of lags
%                        of the state variables in the belief equations; "Name" a string with the name of the
%                        belief system; and G a cell array with f matrices, one for each forward looking
%                        variable of dimension (k+r*m) x g_j where g_j is the number of explanatory
%                        variables in the belief equation for forward looking variable j. If empty, the
%                        function will compute the structure itself.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: CorrData (structure) with main fields "Obs", "State" and "Data". These fields have sub-fields
%                        "Mean", "Cov", "Corr", "Skew" and "Kurt". The main fields "Obs" and "State" refer to the
%                        results based on the adaptive learning DSGE model, while the field "Data" refers to
%                        the results based on the data. Notice that the autocovariance and autocorrelation
%                        results using the data are calculated by de-meaning using the mean from the adaptive
%                        learning model. This should make the results across the two approaches easier to compare.
%
% OPTIONAL OUTPUT: TransStatus (boolean) which takes the value 1 if the rewrite of the structural form of
%                        the DSGE model was successful and 0 otherwise.
%
%                  InitialStatus (boolean) which takes the value 1 if the initial values of the belief
%                        coefficients could be computed without error and 0 otherwise.
%
%                  status (integer) Indicates whether the solution of the model is unique or not.
%                                 mcode is 1 when unique, 0 if not computed, and various other values when
%                                 there are problems with the solution. Specifically:
%
%                                      mcode = 2:      Roots not correctly computed by real_schur
%                                      mcode = 3:      Too many big roots
%                                      mcode = 35:     Too many big roots, and q(:,right) is singular
%                                      mcode = 4:      Too few big roots
%                                      mcode = 45:     Too few big roots, and q(:,right) is singular
%                                      mcode = 5:      q(:,right) is singular
%                                      mcode = 61:     Too many exact shiftrights
%                                      mcode = 62:     Too many numeric shiftrights
%                                      mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                      cannot be calculated.
%                                      mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                      numbers.
%                                      mcode = -1:     No stable solution.
%                                      mcode = -2:     Too many large generalized eigenvalues.
%                                      mcode = -3:     Too few large generalized eigenvalues.
%                                      else            Return code not properly specified
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: August 28, 2024.
%                        This version: August 29, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 29-08-2024: The code provides the same type of output for the observed and the state
%               variables.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
CorrData = [];
if nargout>1;
   TransStatus = 1;
end;
if nargout>2;
   InitialStatus = 1;
end;
%
%
if nargout<=3;
   %
   % Make sure that the ModelParameter structure takes the correct values.
   % This is not necessary to do when initial values are given to the function.
   % In that case, more substantial checking is required!
   %
   ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
   %
   % copy files to the tmp folder
   %
   [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
   [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.SystemPriorFile)==1;
      [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   end;
   %
   % check if we should copy model solver file
   %
   if DSGEModel.ModelSolver==4;
      if FileExist(DSGEModel.ModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
      end;
      if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
      end;
   end;
   %
   % Update any parameteters that need updating
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
   end;
else;
   status = 1;
end;
%
% check if we should also copy the belief system file
%
if DSGEModel.BeliefSystem==2;
   [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if nargout>3;
      status = 0;
   end;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% See if we can solve the model
%
if nargout<=3;
   if DSGEModel.ModelSolver==1;
      [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   elseif DSGEModel.ModelSolver==2;
      [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==3;
      [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==4;
      [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
   end;
else;
   if MatlabNumber>=8.0;
      try;
         if DSGEModel.ModelSolver==1;
            [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
         elseif DSGEModel.ModelSolver==2;
            [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==3;
            [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==4;
            [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      catch ME;
         if exist('mcode')>0;
            status = mcode;
         else;
            status = 0;
         end;
         if DSGEModel.ModelSolver==1;
            ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==2;
            ErrorStr = ['There was an error running the Klein solver. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==3;
            ErrorStr = ['There was an error running Gensys. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==4;
            ErrorStr = ['There was an error running own DSGE model solver. YADA received the following message: ' YADALastError(ME)];
         end;
         About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
         return;
      end;
   else;
      try;
         if DSGEModel.ModelSolver==1;
            [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
         elseif DSGEModel.ModelSolver==2;
            [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==3;
            [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==4;
            [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      catch;
         if exist('mcode')>0;
            status = mcode;
         else;
            status = 0;
         end;
         if DSGEModel.ModelSolver==1;
            ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==2;
            ErrorStr = ['There was an error running the Klein solver. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==3;
            ErrorStr = ['There was an error running Gensys. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==4;
            ErrorStr = ['There was an error running own DSGE model solver. YADA received the following message: ' YADALastError];
         end;
         About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
         return;
      end;
   end;
end;
if nargout>3;
   status = mcode;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
   return;
end;
%
% Convert to the state equation matrices
%
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
[B0Alt,KeepVar] = RemoveRedundantColumns(B0);
ModelParameters.YADA.F = F;
ModelParameters.YADA.B0 = B0Alt;
%
% obtain the measurement equation matrices
%
if MatlabNumber>=8.0;
   try;
      [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
   catch ME;
      %
      % display an error message
      %
      ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError(ME)];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
else;
   try;
      [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
   catch;
      %
      % display an error message
      %
      ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
if length(size(H))==3;
   if size(H,3)<size(DSGEModel.Y,2);
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations (T = ' int2str(size(DSGEModel.Y,2)) '). YADA has therefore aborted from the calculations.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
%
% get the structural form matrices
%
[Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
%
% Determine the belief system matrices
%
if DSGEModel.BeliefSystem==1;
   BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
else;
   BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
end;
%
% Compute the transformation of the structural from that we use when solving the model
%
r = length(DSGEModel.StateVariablePositions);
f = length(DSGEModel.ForwardVariablePositions);
SMatrix = eye(r);
Sperp = SMatrix(:,setdiff((1:r),DSGEModel.ForwardVariablePositions));
SMatrix = SMatrix(:,DSGEModel.ForwardVariablePositions);
%
% call the function: 
%
[tstatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
if nargout>1;
   TransStatus = tstatus;
   if TransStatus==0;
      return;
   end;
end;
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
%
% Setup all the basic data matrix for the deterministic variables
% used for initializing Sigmax
%
if DSGEModel.k>0
   x = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
else;
   x = [];
end;
%
% Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
%
try;
   [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,BeliefSys.m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue);
catch;
   InitialStatus = 0;
   return;
end;
%
% simplify integers
%
m = BeliefSys.m;
k = DSGEModel.k;
n = DSGEModel.n;
%
% call the joint Kalman filter for state variables, belief coefficients and
% solution matrices
%
ConstPos = 0;
if k>0;
   ConstPos = LocateStr(DSGEModel.XVariableNames,'const');
   if ConstPos==0;
      %
      % we will now check if X has a unit vector
      %
      X = DSGEModel.X(:,FirstPeriod:LastPeriod);
      T = size(X,2);
      for j=1:k;
         ConstTest = (DSGEModel.X(j,FirstPeriod:LastPeriod)==ones(1,T));
         if sum(ConstTest)==T;
            ConstPos = j;
            break;
         end;
      end;
   end;
   xbar = zeros(k,1);
   xbar(ConstPos) = 1;
else;
   xbar = [];
end;
%
% compute initial values for the solution matrices
%
if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
   [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
else;
   [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
%
% Determine the autocovariance horizon
%
if strcmp(DSGEModel.DataFrequency,'q')==1;
   SampleFreq = 4;
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   SampleFreq = 12;
else;
   SampleFreq = 1;
end;
Y = DSGEModel.Y(:,FirstPeriod-1+KalmanFirstObservation:LastPeriod);
if k>0;
   X = DSGEModel.X(:,FirstPeriod-1+KalmanFirstObservation:LastPeriod);
end;
T = size(Y,2);
AutoCovHorizon = max(min([SampleFreq*3 T-20 18]),0);
%
if k>0;
   if length(size(H))==2;
      CorrData.State.Mean = (eye(r)-Ft1)\(Mt1*xbar);
      CorrData.Obs.Mean = ((1/T)*sum(A'*X,2))+(H'*CorrData.State.Mean);
   else;
      CorrData.State.Mean = (eye(r)-Ft1)\(Mt1*xbar);
      Ht = H(:,:,FirstPeriod-1+KalmanFirstObservation:LastPeriod);
      CorrData.Obs.Mean = zeros(n,1);
      for t=1:T;
         CorrData.Obs.Mean = CorrData.Obs.Mean+((1/T)*Ht(:,:,t)'*CorrData.State.Mean);
      end;
      CorrData.Obs.Mean = ((1/T)*sum(A'*X,2))+CorrData.Obs.Mean;
   end;
else;
   CorrData.Obs.Mean = zeros(n,1);
   CorrData.State.Mean = zeros(r,1);
end;
%
% calculate the unconditional variance of the state variables
%
[Sigma,stat] = DoublingAlgorithmLyapunov(Ft1,Bt1*Bt1',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
%
% the state variables
%
AutoCov = Sigma;
CorrVec = sqrt(diag(AutoCov));
CorrData.State.Std = CorrVec;
CorrData.State.Cov = AutoCov;
CorrData.State.Corr = zeros((r*(r+1))/2,(2*AutoCovHorizon)+1);
CorrData.State.Corr(:,AutoCovHorizon+1) = vech(AutoCov./(CorrVec*CorrVec'));
CorrData.AutoCovHorizon = (-AutoCovHorizon:1:AutoCovHorizon);
CorrData.State.Skew = zeros(r,1);
CorrData.State.Kurt = 3*ones(r,1);
FPower = Ft1;
for i=1:AutoCovHorizon;
   Corr = (FPower*Sigma)./(CorrVec*CorrVec');
   CorrData.State.Corr(:,AutoCovHorizon+1+i) = vech(Corr');
   CorrData.State.Corr(:,AutoCovHorizon+1-i) = vech(Corr);
   FPower = FPower*Ft1;
end;
%
% the observed variables
%
CorrData.Obs.Corr = zeros((n*(n+1))/2,(2*AutoCovHorizon)+1);
%
% population skewness is zero for the model
%
CorrData.Obs.Skew = zeros(n,1);
CorrData.Obs.Kurt = 3*ones(n,1);
if length(size(H))==2;
   AutoCov = (H'*Sigma*H)+R;
   CorrVec = sqrt(diag(AutoCov));
   CorrData.Obs.Std = CorrVec;
   CorrData.Obs.Cov = AutoCov;
   CorrData.Obs.Corr(:,AutoCovHorizon+1) = vech(AutoCov./(CorrVec*CorrVec'));
   FPower = Ft1;
   for i=1:AutoCovHorizon;
      Corr = (H'*FPower*Sigma*H)./(CorrVec*CorrVec');
      CorrData.Obs.Corr(:,AutoCovHorizon+1+i) = vech(Corr');
      CorrData.Obs.Corr(:,AutoCovHorizon+1-i) = vech(Corr);
      FPower = FPower*Ft1;
   end;
else;
   AutoCov = zeros(n,n);
   Sample = FirstPeriod-1+KalmanFirstObservation:LastPeriod;
   for t=Sample;
      AutoCov = AutoCov+((1/length(Sample))*(H(:,:,t)'*Sigma*H(:,:,t))+R);
   end;
   CorrVec = sqrt(diag(AutoCov));
   CorrData.Obs.Std = CorrVec;
   CorrData.Obs.Cov = AutoCov;
   CorrData.Obs.Corr(:,AutoCovHorizon+1) = vech(AutoCov./(CorrVec*CorrVec'));
   FPower = Ft1;
   for i=1:AutoCovHorizon;
      AutoCov = zeros(n,n);
      for t=FirstPeriod-1+DSGEModel.KalmanFirstObservation:LastPeriod-i;
         AutoCov = AutoCov+((1/(length(Sample)-i))*(H(:,:,t+i)'*FPower*Sigma*H(:,:,t)));
      end;
      Corr = AutoCov./(CorrVec*CorrVec');
      CorrData.Obs.Corr(:,AutoCovHorizon+1+i) = vech(Corr');
      CorrData.Obs.Corr(:,AutoCovHorizon+1-i) = vech(Corr);
      FPower = FPower*Ft1;
   end;
end;
%
% Do the same with the actual data
%
if DSGEModel.YNaNs==0;
   if k>0;
      CorrData.Data.Mean = (Y*X')/(X*X');
      Y = Y-(CorrData.Data.Mean*X);
   else;
      CorrData.Data.Mean = (1/T)*sum(Y,2);
      Y = Y-(CorrData.Data.Mean*ones(1,T));
   end;
   %
   AutoCov = (1/T)*Y*Y';
   CorrVec = sqrt(diag(AutoCov));
   CorrData.Data.Cov = AutoCov;
   CorrData.Data.Std = CorrVec;
   CorrData.Data.Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
   CorrData.Data.Corr(:,AutoCovHorizon+1) = vech(AutoCov./(CorrVec*CorrVec'));
   CorrData.Data.Skew = ((1/T)*sum(Y.*Y.*Y,2))./(CorrVec.*CorrVec.*CorrVec);
   CorrData.Data.Kurt = ((1/T)*sum(Y.*Y.*Y.*Y,2))./(CorrVec.*CorrVec.*CorrVec.*CorrVec);
   %
   Yold = Y;
   if k>0;
      Xold = X;
   end;
   TFixed = T;
   for i=1:AutoCovHorizon;
      Y = Yold(:,1+i:size(Yold,2));
      YL = Yold(:,1:size(Yold,2)-i);
      T = size(Y,2);
      if k>0;
         X = Xold(:,1+i:size(Yold,2));
         XL = Xold(:,1:size(Yold,2)-i);
         NewMeanY = (Y*X')/(X*X');
         Y = Y-(NewMeanY*X);
         YL = YL-(NewMeanY*XL);
      else;
         Y = Y-(CorrData.Data.Mean*ones(1,T));
         YL = YL-(CorrData.Data.Mean*ones(1,T));
      end;
      %
      AutoCov = (1/TFixed)*Y*YL';
      Corr = AutoCov./(CorrVec*CorrVec');
      CorrData.Data.Corr(:,AutoCovHorizon+1+i) = vech(Corr');
      CorrData.Data.Corr(:,AutoCovHorizon+1-i) = vech(Corr);
   end;
   Y = Yold;
   if k>0;
      X = Xold;
   end;
else;
   %
   % deal with NaNs
   %
   pos = (~isnan(Y));
   NaNY = isnan(Y);
   Y(NaNY) = 0;
   Tvec = T./sum(pos,2);
   if k==0;
      CorrData.Data.Mean = diag(Tvec)*((Y*X')/(X*X'));
      %
      % de-mean Y using estimate of the mean from the data rather than A'.
      %
      Y = Y-(CorrData.Data.Mean*X);
   else;
      CorrData.Data.Mean = Tvec.*((1/T)*sum(Y,2));
      Y = Y-(CorrData.Data.Mean*ones(1,T));
   end;
   Yold = Y;
   if k>0;
      Xold = X;
   end;
   Y(NaNY) = 0;
   %
   TMat = ones(n,n)./TMatrix(sum(pos,2));
   AutoCov = TMat.*(Y*Y');
   CorrVec = sqrt(diag(AutoCov));
   CorrData.Data.Cov = AutoCov;
   CorrData.Data.Std = CorrVec;
   CorrData.Data.Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
   CorrData.Data.Corr(:,AutoCovHorizon+1) = vech(AutoCov./(CorrVec*CorrVec'));
   CorrData.Data.Skew = ((Tvec/T).*sum(Y.*Y.*Y,2))./(CorrVec.*CorrVec.*CorrVec);
   CorrData.Data.Kurt = ((Tvec/T).*sum(Y.*Y.*Y.*Y,2))./(CorrVec.*CorrVec.*CorrVec.*CorrVec);
   %
   TFixed = T;
   for i=1:AutoCovHorizon;
      Y = Yold(:,1+i:size(Yold,2));
      YL = Yold(:,1:size(Yold,2)-i);
      T = size(Y,2);
      %
      pos = (~isnan(Y));
      NaNY = isnan(Y);
      NaNYL = isnan(YL);
      Y(NaNY) = 0;
      YL(NaNYL) = 0;
      Tvec = T./sum(pos,2);
      %
      if k>0;
         X = Xold(:,1+i:size(Yold,2));
         XL = Xold(:,1:size(Yold,2)-i);
         NewMeanY = diag(Tvec)*((Y*X')/(X*X'));
         Y = Y-(NewMeanY*X);
         YL = YL-(NewMeanY*XL);
      else;
         Y = Y-(CorrData.Data.Mean*ones(1,T));
         YL = YL-(CorrData.Data.Mean*ones(1,T));
      end;
      Y(NaNY) = 0;
      YL(NaNYL) = 0;
      %
      AutoCov = TMat.*(Y*YL');
      Corr = AutoCov./(CorrVec*CorrVec');
      CorrData.Data.Corr(:,AutoCovHorizon+1+i) = vech(Corr');
      CorrData.Data.Corr(:,AutoCovHorizon+1-i) = vech(Corr);
   end;
end;

%
% end of CalculateSteadyStateCorrelations.m
%
