function [CorrData,TransStatus,InitialStatus,status] = SteadyStateCorrelationSimulationTheta(theta,thetaPositions,ModelParameters,NumPaths,EstType,BeliefSys,FirstPeriod,LastPeriod,DSGEModel,CurrINI)
% SteadyStateCorrelationSimulationTheta: Estimates observed and state variable moments for a DSGE model with adaptive learning.
%                                        The function relies on simulated data and, hence, provides simulation based estimates
%                                        of the sample moments rather than of the population equivalents.
%
% USAGE:
%
%       [CorrData,TransStatus,InitialStatus,status] = SteadyStateCorrelationSimulationTheta(theta,thetaPositions,ModelParameters,...
%                                                            NumPaths,EstType,BeliefSys,FirstPeriod,LastPeriod,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector) with values of the original DSGE model parameters as well as the estimated
%                        parameters of the adaptive learning system.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function needs the
%                        former field to know which parameter name corresponds to position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                        DSGE model along with the adaptive learning system.
%
%                  NumPaths (integer) with the number of data paths to simulate per parameter value.
%
%                  EstType (string vector) which determines the type of parameter values that are used. This is
%                       "Posterior Mode", "DSGE Posterior Mode" or "Initial Values".
%
%                  FirstPeriod (integer), with the first period in the sample when the training sample size
%                        for the Kalman filter is not taken into account.
%
%                  LastPeriod (integer), with the last period in the sample.
%
%                  BeliefSys (structure) with fields "m" an integer denoting the maximum number of lags
%                        of the state variables in the belief equations; "Name" a string with the name of the
%                        belief system; and G a cell array with f matrices, one for each forward looking
%                        variable of dimension (k+r*m) x g_j where g_j is the number of explanatory
%                        variables in the belief equation for forward looking variable j. If empty, the
%                        function will compute the structure itself.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: CorrData (structure) with main fields "Obs", "State" and "Data". These fields have sub-fields
%                        "Mean", "Cov", "Corr", "Skew" and "Kurt". The main fields "Obs" and "State" refer to the
%                        results based on the adaptive learning DSGE model, while the field "Data" refers to
%                        the results based on the data. Notice that the autocovariance and autocorrelation
%                        results using the data are calculated by de-meaning using the mean from the adaptive
%                        learning model. This should make the results across the two approaches easier to compare.
%
% OPTIONAL OUTPUT: TransStatus (boolean) which takes the value 1 if the rewrite of the structural form of
%                        the DSGE model was successful and 0 otherwise.
%
%                  InitialStatus (boolean) which takes the value 1 if the initial values of the belief
%                        coefficients could be computed without error and 0 otherwise.
%
%                  status (integer) Indicates whether the solution of the model is unique or not.
%                                 mcode is 1 when unique, 0 if not computed, and various other values when
%                                 there are problems with the solution. Specifically:
%
%                                      mcode = 2:      Roots not correctly computed by real_schur
%                                      mcode = 3:      Too many big roots
%                                      mcode = 35:     Too many big roots, and q(:,right) is singular
%                                      mcode = 4:      Too few big roots
%                                      mcode = 45:     Too few big roots, and q(:,right) is singular
%                                      mcode = 5:      q(:,right) is singular
%                                      mcode = 61:     Too many exact shiftrights
%                                      mcode = 62:     Too many numeric shiftrights
%                                      mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                      cannot be calculated.
%                                      mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                      numbers.
%                                      mcode = -1:     No stable solution.
%                                      mcode = -2:     Too many large generalized eigenvalues.
%                                      mcode = -3:     Too few large generalized eigenvalues.
%                                      else            Return code not properly specified
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: August 29, 2024.
%                        This version: August 30, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 30-08-2024: Finished the function where quantiles are computed.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
CorrData = [];
if nargout>1;
   TransStatus = 1;
end;
if nargout>2;
   InitialStatus = 1;
end;
%
%
if nargout<=3;
   %
   % Make sure that the ModelParameter structure takes the correct values.
   % This is not necessary to do when initial values are given to the function.
   % In that case, more substantial checking is required!
   %
   ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
   %
   % copy files to the tmp folder
   %
   [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
   [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.SystemPriorFile)==1;
      [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   end;
   %
   % check if we should copy model solver file
   %
   if DSGEModel.ModelSolver==4;
      if FileExist(DSGEModel.ModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
      end;
      if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
      end;
   end;
   %
   % Update any parameteters that need updating
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
   end;
else;
   status = 1;
end;
%
% check if we should also copy the belief system file
%
if DSGEModel.BeliefSystem==2;
   [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if nargout>3;
      status = 0;
   end;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% now we try to solve the model
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch ME;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   if nargout>3;
      status = mcode;
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
   return;
end;
if length(size(H))==3;
   if size(H,3)<LastPeriod;
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations (T = ' int2str(DSGEModel.LastPeriod) '). YADA has therefore aborted from the observed variable correlations.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
%
% make sure that only non-zero shocks are kept
%
[B0Alt,KeepShocks] = RemoveRedundantColumns(B0);
ModelParameters.YADA.F = F;
ModelParameters.YADA.B0 = B0Alt;
%
% get the structural form matrices
%
[Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
%
% Determine the belief system matrices
%
if DSGEModel.BeliefSystem==1;
   BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
else;
   BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
end;
%
% Compute the transformation of the structural from that we use when solving the model
%
r = length(DSGEModel.StateVariablePositions);
f = length(DSGEModel.ForwardVariablePositions);
SMatrix = eye(r);
Sperp = SMatrix(:,setdiff((1:r),DSGEModel.ForwardVariablePositions));
SMatrix = SMatrix(:,DSGEModel.ForwardVariablePositions);
%
% call the function: 
%
[tstatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
if nargout>1;
   TransStatus = tstatus;
   if TransStatus==0;
      return;
   end;
end;
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
%
% Setup all the basic data matrix for the deterministic variables
% used for initializing Sigmax
%
if DSGEModel.k>0
   x = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
else;
   x = [];
end;
%
% Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
%
try;
   [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,BeliefSys.m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue);
catch;
   InitialStatus = 0;
   return;
end;
%
% simplify integers
%
m = BeliefSys.m;
k = DSGEModel.k;
n = DSGEModel.n;
%
% call the joint Kalman filter for state variables, belief coefficients and
% solution matrices
%
ConstPos = 0;
if k>0;
   ConstPos = LocateStr(DSGEModel.XVariableNames,'const');
   if ConstPos==0;
      %
      % we will now check if X has a unit vector
      %
      X = DSGEModel.X(:,FirstPeriod:LastPeriod);
      T = size(X,2);
      for j=1:k;
         ConstTest = (DSGEModel.X(j,FirstPeriod:LastPeriod)==ones(1,T));
         if sum(ConstTest)==T;
            ConstPos = j;
            break;
         end;
      end;
   end;
   xbar = zeros(k,1);
   xbar(ConstPos) = 1;
else;
   xbar = [];
end;
%
% compute initial values for the solution matrices
%
if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
   [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
else;
   [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
q = size(Bt1,2);
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% initial values for the mean and covariance matrix of the distribution of the state variables
%
if isempty(DSGEModel.UnitRootStates)==1;
   [P0,stat] = DoublingAlgorithmLyapunov(Ft1,Bt1*Bt1',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
else;
   StationaryPos = GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates);
   P0 = DSGEModel.StateCovConstValue*eye(r);
   F1 = Ft1(StationaryPos,StationaryPos);
   Q1 = Bt1(StationaryPos,:)*Bt1(StationaryPos,:)';
   [Pstat,stat] = DoublingAlgorithmLyapunov(F1,Q1,DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
   P0(StationaryPos,StationaryPos) = Pstat;
end;
%
Ksi0 = DSGEModel.InitialStateVector;
[S,L] = SchurDecomposition(P0);
%
% deal with measurement errors
%
if rank(R)>0;
   [Rm,Lm] = SchurDecomposition(R);
   Rmeasure = Rm*sqrt(Lm);
   NumMeasure = size(Lm,1);
else;
   Rmeasure = [];
   wSim = 0;
   NumMeasure = 0;
end;
%
% set up Y and X
%
Y = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
if isempty(DSGEModel.X)==0;
   X = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
   XSim = DSGEModel.X(:,FirstPeriod:LastPeriod);
else;
   X = [];
   XSim = [];
end;
T = size(Y,2);
YTmp = Y;
XTmp = X;
%
% set up the autocovariance horizon
%
if strcmp(DSGEModel.DataFrequency,'q')==1;
   SampleFreq = 4;
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   SampleFreq = 12;
else;
   SampleFreq = 1;
end;
AutoCovHorizon = max(min([SampleFreq*3 T-20 18]),0);
SVEs.AutoCovHorizon = (-AutoCovHorizon:1:AutoCovHorizon);
%
% compute correlation data for the observed variables
%
if DSGEModel.YNaNs==0;
   if k>0;
      SVEs.Data.Mean = (Y*X')/(X*X');
      Y = Y-(SVEs.Data.Mean*X);
   else;
      SVEs.Data.Mean = (1/T)*sum(Y,2);
      Y = Y-(SVEs.Data.Mean*ones(1,T));
   end;
   %
   AutoCov = (1/T)*Y*Y';
   CorrVec = sqrt(diag(AutoCov));
   SVEs.Data.Cov = AutoCov;
   SVEs.Data.Std = CorrVec;
   SVEs.Data.Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
   SVEs.Data.Corr(:,AutoCovHorizon+1) = vech(AutoCov./(CorrVec*CorrVec'));
   SVEs.Data.Skew = ((1/T)*sum(Y.*Y.*Y,2))./(CorrVec.*CorrVec.*CorrVec);
   SVEs.Data.Kurt = ((1/T)*sum(Y.*Y.*Y.*Y,2))./(CorrVec.*CorrVec.*CorrVec.*CorrVec);
   %
   Yold = Y;
   if k>0;
      Xold = X;
   end;
   TFixed = T;
   for i=1:AutoCovHorizon;
      Y = Yold(:,1+i:size(Yold,2));
      YL = Yold(:,1:size(Yold,2)-i);
      T = size(Y,2);
      if k>0;
         X = Xold(:,1+i:size(Yold,2));
         XL = Xold(:,1:size(Yold,2)-i);
         NewMeanY = (Y*X')/(X*X');
         Y = Y-(NewMeanY*X);
         YL = YL-(NewMeanY*XL);
      else;
         Y = Y-(SVEs.Data.Mean*ones(1,T));
         YL = YL-(SVEs.Data.Mean*ones(1,T));
      end;
      %
      AutoCov = (1/TFixed)*Y*YL';
      Corr = AutoCov./(CorrVec*CorrVec');
      SVEs.Data.Corr(:,AutoCovHorizon+1+i) = vech(Corr');
      SVEs.Data.Corr(:,AutoCovHorizon+1-i) = vech(Corr);
   end;
   Y = Yold;
   if k>0;
      X = Xold;
   end;
else;
   %
   % deal with NaNs
   %
   pos = (~isnan(Y));
   NaNY = isnan(Y);
   Y(NaNY) = 0;
   Tvec = T./sum(pos,2);
   if k==0;
      SVEs.Data.Mean = diag(Tvec)*((Y*X')/(X*X'));
      %
      % de-mean Y using estimate of the mean from the data rather than A'.
      %
      Y = Y-(SVEs.Data.Mean*X);
   else;
      SVEs.Data.Mean = Tvec.*((1/T)*sum(Y,2));
      Y = Y-(SVEs.Data.Mean*ones(1,T));
   end;
   Yold = Y;
   if k>0;
      Xold = X;
   end;
   Y(NaNY) = 0;
   %
   TMat = ones(n,n)./TMatrix(sum(pos,2));
   AutoCov = TMat.*(Y*Y');
   CorrVec = sqrt(diag(AutoCov));
   SVEs.Data.Cov = AutoCov;
   SVEs.Data.Std = CorrVec;
   SVEs.Data.Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
   SVEs.Data.Corr(:,AutoCovHorizon+1) = vech(AutoCov./(CorrVec*CorrVec'));
   SVEs.Data.Skew = ((Tvec/T).*sum(Y.*Y.*Y,2))./(CorrVec.*CorrVec.*CorrVec);
   SVEs.Data.Kurt = ((Tvec/T).*sum(Y.*Y.*Y.*Y,2))./(CorrVec.*CorrVec.*CorrVec.*CorrVec);
   %
   TFixed = T;
   for i=1:AutoCovHorizon;
      Y = Yold(:,1+i:size(Yold,2));
      YL = Yold(:,1:size(Yold,2)-i);
      T = size(Y,2);
      %
      pos = (~isnan(Y));
      NaNY = isnan(Y);
      NaNYL = isnan(YL);
      Y(NaNY) = 0;
      YL(NaNYL) = 0;
      Tvec = T./sum(pos,2);
      %
      if k>0;
         X = Xold(:,1+i:size(Yold,2));
         XL = Xold(:,1:size(Yold,2)-i);
         NewMeanY = diag(Tvec)*((Y*X')/(X*X'));
         Y = Y-(NewMeanY*X);
         YL = YL-(NewMeanY*XL);
      else;
         Y = Y-(SVEs.Data.Mean*ones(1,T));
         YL = YL-(SVEs.Data.Mean*ones(1,T));
      end;
      Y(NaNY) = 0;
      YL(NaNYL) = 0;
      %
      AutoCov = TMat.*(Y*YL');
      Corr = AutoCov./(CorrVec*CorrVec');
      SVEs.Data.Corr(:,AutoCovHorizon+1+i) = vech(Corr');
      SVEs.Data.Corr(:,AutoCovHorizon+1-i) = vech(Corr);
   end;
end;
Y = Yold;
X = Xold;
%
% initialize the random number generator
%
if DSGEModel.RandomNumberValue==1;
   randn('state',0);
   rand('state',0);
else;
   randn('state',sum(100*clock));
   rand('state',sum(100*clock));
end;
%
% reset number of observations
%
T = LastPeriod-FirstPeriod+1;
Tsum = T-KalmanFirstObservation+1;
%
% begin the loop
%
abort = '';
MeanEstimationTime = 0;
SVE = [];
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' num2str(NumPaths,'%0.0f') ' simulation paths'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Mean estimation time:';
   %
   ProgressStructure.name = ['Autocorrelation Distribution - ' EstType];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA simulates ' int2str(NumPaths) ' samples with data from the model to compute sample means, autocorrelations, and skewness of the observed and state variables from the simulated data.'];
   WaitHandle = WaitDLG(txt,'information',['Autocorrelation Distribution - ' EstType],600,400,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
%
% start the simulation of data
%
if length(size(H))==2;
   for p=1:NumPaths;
      %
      % fix the progress dialog is necessary
      %
      tic;
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            StopSimulation = 1;
            break;
         else;
            ProgressDLG([p/NumPaths MeanEstimationTime]);
         end;
      end;
      %
      % draw the economic shocks
      %
      etaSim = randn(q,T);
      %
      % draw the measurement errors
      %
      if isempty(Rmeasure)==0;
         wSim = Rmeasure*randn(NumMeasure,T);
      end;
      %
      % draw an initial state
      %
      StateSim = zeros(r,T+1);
      StateSim(:,1) = Ksi0+(S*sqrt(L)*randn(size(L,2),1));
      %
      % generate the state matrix
      %
      if k>0;
         for t=1:T;
            StateSim(:,t+1) = (Mt1*XSim(:,t))+(Ft1*StateSim(:,t))+(Bt1*etaSim(:,t));
         end;
      else;
         for t=1:T;
            StateSim(:,t+1) = (Ft1*StateSim(:,t))+(Bt1*etaSim(:,t));
         end;
      end;
      drawnow;
      %
      % remove the initial value of the state vector
      %
      StateSim = StateSim(:,2:T+1);
      %
      % create the observed variables
      %
      if k>0;
         YSim = (A'*XSim)+(H'*StateSim)+wSim;
      else;
         YSim = (H'*StateSim)+wSim;
      end;
      %
      % compute the sample moments of the simulated state variables
      %
      YTmp = StateSim(:,KalmanFirstObservation:T);
      if k>0;
         XTmp = XSim(:,KalmanFirstObservation:T);
         MeanY = (YTmp*XTmp')/(XTmp*XTmp');
         YTmp = YTmp-(MeanY*XTmp);
      else;
         MeanY = (1/Tsum)*sum(YTmp,2);
         YTmp = YTmp-MeanY;
      end;
      SVE.State.CorrData(p).MeanY = MeanY;
      SVE.State.CorrData(p).StdY = sqrt(diag((1/Tsum)*YTmp*YTmp'));
      %
      % compute simulated skewness
      %
      SVE.State.CorrData(p).Skew = ((1/Tsum)*sum(YTmp.*YTmp.*YTmp,2))./(SVE.State.CorrData(p).StdY.*SVE.State.CorrData(p).StdY.*SVE.State.CorrData(p).StdY);
      %
      % and kurtosis
      %
      SVE.State.CorrData(p).Kurt = ((1/Tsum)*sum(YTmp.*YTmp.*YTmp.*YTmp,2))./(SVE.State.CorrData(p).StdY.*SVE.State.CorrData(p).StdY.*SVE.State.CorrData(p).StdY.*SVE.State.CorrData(p).StdY);
      %
      % and now for the autocovariances
      %
      SVE.State.CorrData(p).CorrY = zeros((r*(r+1))/2,1+(2*AutoCovHorizon));
      SVE.State.CorrData(p).CorrY(:,AutoCovHorizon+1) = vech(((1/Tsum)*YTmp*YTmp')./(SVE.State.CorrData(p).StdY*SVE.State.CorrData(p).StdY'));
      for i=1:AutoCovHorizon;
         Yt = YTmp(:,1+i:Tsum);
         YtL = YTmp(:,1:Tsum-i);
         if k>0;
            Xt = XTmp(:,1+i:Tsum);
            XtL = XTmp(:,1:Tsum-i);
            NewMeanYt = (Yt*Xt')/(Xt*Xt');
            Yt = Yt-(NewMeanYt*Xt);
            YtL = YtL-(NewMeanYt*XtL);
         else;
            Yt = Yt-MeanY;
            YtL = YtL-MeanY;
         end;
         Corr = ((1/Tsum)*Yt*YtL')./(SVE.State.CorrData(p).StdY*SVE.State.CorrData(p).StdY');
         SVE.State.CorrData(p).CorrY(:,AutoCovHorizon+1+i) = vech(Corr');
         SVE.State.CorrData(p).CorrY(:,AutoCovHorizon+1-i) = vech(Corr);         
      end;
      %
      % compute the sample moments based on the simulated observed data
      %
      YTmp = YSim(:,KalmanFirstObservation:T);
      if DSGEModel.YNaNs==0;
         if k>0;
            XTmp = XSim(:,KalmanFirstObservation:T);
            MeanY = (YTmp*XTmp')/(XTmp*XTmp');
            YTmp = YTmp-(MeanY*XTmp);
         else;
            MeanY = (1/Tsum)*sum(YTmp,2);
            YTmp = YTmp-MeanY;
         end;
         SVE.CorrData(p).MeanY = MeanY;
         SVE.CorrData(p).StdY = sqrt(diag((1/Tsum)*YTmp*YTmp'));
         %
         % compute simulated skewness
         %
         SVE.CorrData(p).Skew = ((1/Tsum)*sum(YTmp.*YTmp.*YTmp,2))./(SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY);
         %
         % and kurtosis
         %
         SVE.CorrData(p).Kurt = ((1/Tsum)*sum(YTmp.*YTmp.*YTmp.*YTmp,2))./(SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY);
         %
         % and now for the autocovariances
         %
         SVE.CorrData(p).CorrY = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
         SVE.CorrData(p).CorrY(:,AutoCovHorizon+1) = vech(((1/Tsum)*YTmp*YTmp')./(SVE.CorrData(p).StdY*SVE.CorrData(p).StdY'));
         for i=1:AutoCovHorizon;
            Yt = YTmp(:,1+i:Tsum);
            YtL = YTmp(:,1:Tsum-i);
            if k>0;
               Xt = XTmp(:,1+i:Tsum);
               XtL = XTmp(:,1:Tsum-i);
               NewMeanYt = (Yt*Xt')/(Xt*Xt');
               Yt = Yt-(NewMeanYt*Xt);
               YtL = YtL-(NewMeanYt*XtL);
            else;
               Yt = Yt-MeanY;
               YtL = YtL-MeanY;
            end;
            Corr = ((1/Tsum)*Yt*YtL')./(SVE.CorrData(p).StdY*SVE.CorrData(p).StdY');
            SVE.CorrData(p).CorrY(:,AutoCovHorizon+1+i) = vech(Corr');
            SVE.CorrData(p).CorrY(:,AutoCovHorizon+1-i) = vech(Corr);         
         end;
      else;
         pos = (~isnan(Y));
         NaNY = isnan(Y);
         YTmp(NaNY) = 0;
         Tvec = T./sum(pos,2);
         if k>0;
            XTmp = XSim(:,KalmanFirstObservation:T);
            MeanY = diag(Tvec)*(YTmp*XTmp')/(XTmp*XTmp');
            YTmp = YTmp-(MeanY*XTmp);
         else;
            MeanY = Tvec.*((1/Tsum)*sum(YTmp,2));
            YTmp = YTmp-MeanY;
         end;
         YTmp(NaNY) = 0;
         SVE.CorrData(p).MeanY = MeanY;
         SVE.CorrData(p).StdY = sqrt(diag(TMat.*(YTmp*YTmp')));
         %
         % compute simulated skewness
         %
         SVE.CorrData(p).Skew = ((Tvec/Tsum).*sum(YTmp.*YTmp.*YTmp,2))./(SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY);
         %
         % and kurtosis
         %
         SVE.CorrData(p).Kurt = ((Tvec/Tsum).*sum(YTmp.*YTmp.*YTmp.*YTmp,2))./(SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY);
         %
         % and now for the autocovariances
         %
         SVE.CorrData(p).CorrY = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
         SVE.CorrData(p).CorrY(:,AutoCovHorizon+1) = vech(((1/Tsum)*YTmp*YTmp')./(SVE.CorrData(p).StdY*SVE.CorrData(p).StdY'));
         for i=1:AutoCovHorizon;
            Yt = YTmp(:,1+i:Tsum);
            YtL = YTmp(:,1:Tsum-i);
            NaNYt = isnan(Y(:,1+i:Tsum));
            NaNYtL = isnan(Y(:,1:Tsum-i));
            if k>0;
               Xt = XTmp(:,1+i:Tsum);
               XtL = XTmp(:,1:Tsum-i);
               NewMeanYt = diag(Tvec)*((Yt*Xt')/(Xt*Xt'));
               Yt = Yt-(NewMeanYt*Xt);
               YtL = YtL-(NewMeanYt*XtL);
            else;
               Yt = Yt-MeanY;
               YtL = YtL-MeanY;
            end;
            Yt(NaNYt) = 0;
            YtL(NaNYtL) = 0;
            Corr = (TMat.*(Yt*YtL'))./(SVE.CorrData(p).StdY*SVE.CorrData(p).StdY');
            SVE.CorrData(p).CorrY(:,AutoCovHorizon+1+i) = vech(Corr');
            SVE.CorrData(p).CorrY(:,AutoCovHorizon+1-i) = vech(Corr);         
         end;
      end;
      MeanEstimationTime = (((p-1)/p)*MeanEstimationTime)+((1/p)*toc);
      drawnow;
   end;
else;
   for p=1:NumPaths;
      %
      % fix the progress dialog is necessary
      %
      tic;
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            StopSimulation = 1;
            break;
         else;
            ProgressDLG([p/NumPaths MeanEstimationTime]);
         end;
      end;
      %
      % draw the economic shocks
      %
      etaSim = randn(q,T);
      %
      % draw the measurement errors
      %
      if isempty(Rmeasure)==0;
         wSim = Rmeasure*randn(NumMeasure,T);
      end;
      %
      % draw an initial state
      %
      StateSim = zeros(r,T+1);
      StateSim(:,1) = Ksi0+(S*sqrt(L)*randn(size(L,2),1));
      %
      % generate the state matrix
      %
      if k>0;
         for t=1:T;
            StateSim(:,t+1) = (Mt1*XSim(:,t))+(Ft1*StateSim(:,t))+(Bt1*etaSim(:,t));
         end;
      else;
         for t=1:T;
            StateSim(:,t+1) = (Ft1*StateSim(:,t))+(Bt1*etaSim(:,t));
         end;
      end;
      drawnow;
      %
      % remove the initial value of the state vector
      %
      StateSim = StateSim(:,2:T+1);
      %
      % create the observed variables
      %
      if k>0;
         for t=FirstPeriod:LastPeriod;
            YSim(:,t-FirstPeriod+1) = (A'*XSim(:,t-FirstPeriod+1))+(H(:,:,t)'*StateSim(:,t-FirstPeriod+1))+wSim(:,t-FirstPeriod+1);
         end;
      else;
         for t=FirstPeriod:LastPeriod;
            YSim(:,t-FirstPeriod+1) = (H(:,:,t)'*StateSim(:,t-FirstPeriod+1))+wSim(:,t-FirstPeriod+1);
         end;
      end;
      %
      % compute the sample moments of the simulated state variables
      %
      YTmp = StateSim(:,KalmanFirstObservation:T);
      if k>0;
         XTmp = XSim(:,KalmanFirstObservation:T);
         MeanY = (YTmp*XTmp')/(XTmp*XTmp');
         YTmp = YTmp-(MeanY*XTmp);
      else;
         MeanY = (1/Tsum)*sum(YTmp,2);
         YTmp = YTmp-MeanY;
      end;
      SVE.State.CorrData(p).MeanY = MeanY;
      SVE.State.CorrData(p).StdY = sqrt(diag((1/Tsum)*YTmp*YTmp'));
      %
      % compute simulated skewness
      %
      SVE.State.CorrData(p).Skew = ((1/Tsum)*sum(YTmp.*YTmp.*YTmp,2))./(SVE.State.CorrData(p).StdY.*SVE.State.CorrData(p).StdY.*SVE.State.CorrData(p).StdY);
      %
      % and kurtosis
      %
      SVE.State.CorrData(p).Kurt = ((1/Tsum)*sum(YTmp.*YTmp.*YTmp.*YTmp,2))./(SVE.State.CorrData(p).StdY.*SVE.State.CorrData(p).StdY.*SVE.State.CorrData(p).StdY.*SVE.State.CorrData(p).StdY);
      %
      % and now for the autocovariances
      %
      SVE.State.CorrData(p).CorrY = zeros((r*(r+1))/2,1+(2*AutoCovHorizon));
      SVE.State.CorrData(p).CorrY(:,AutoCovHorizon+1) = vech(((1/Tsum)*YTmp*YTmp')./(SVE.State.CorrData(p).StdY*SVE.State.CorrData(p).StdY'));
      for i=1:AutoCovHorizon;
         Yt = YTmp(:,1+i:Tsum);
         YtL = YTmp(:,1:Tsum-i);
         if k>0;
            Xt = XTmp(:,1+i:Tsum);
            XtL = XTmp(:,1:Tsum-i);
            NewMeanYt = (Yt*Xt')/(Xt*Xt');
            Yt = Yt-(NewMeanYt*Xt);
            YtL = YtL-(NewMeanYt*XtL);
         else;
            Yt = Yt-MeanY;
            YtL = YtL-MeanY;
         end;
         Corr = ((1/Tsum)*Yt*YtL')./(SVE.State.CorrData(p).StdY*SVE.State.CorrData(p).StdY');
         SVE.State.CorrData(p).CorrY(:,AutoCovHorizon+1+i) = vech(Corr');
         SVE.State.CorrData(p).CorrY(:,AutoCovHorizon+1-i) = vech(Corr);         
      end;
      %
      % compute the sample moments based on the simulated observed data
      %
      YTmp = YSim(:,KalmanFirstObservation:T);
      if DSGEModel.YNaNs==0;
         if k>0;
            XTmp = XSim(:,KalmanFirstObservation:T);
            MeanY = (YTmp*XTmp')/(XTmp*XTmp');
            YTmp = YTmp-(MeanY*XTmp);
         else;
            MeanY = (1/Tsum)*sum(YTmp,2);
            YTmp = YTmp-MeanY;
         end;
         SVE.CorrData(p).MeanY = MeanY;
         SVE.CorrData(p).StdY = sqrt(diag((1/Tsum)*YTmp*YTmp'));
         %
         % compute simulated skewness
         %
         SVE.CorrData(p).Skew = ((1/Tsum)*sum(YTmp.*YTmp.*YTmp,2))./(SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY);
         %
         % and kurtosis
         %
         SVE.CorrData(p).Kurt = ((1/Tsum)*sum(YTmp.*YTmp.*YTmp.*YTmp,2))./(SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY);
         %
         % and now for the autocovariances
         %
         SVE.CorrData(p).CorrY = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
         SVE.CorrData(p).CorrY(:,AutoCovHorizon+1) = vech(((1/Tsum)*YTmp*YTmp')./(SVE.CorrData(p).StdY*SVE.CorrData(p).StdY'));
         for i=1:AutoCovHorizon;
            Yt = YTmp(:,1+i:Tsum);
            YtL = YTmp(:,1:Tsum-i);
            if k>0;
               Xt = XTmp(:,1+i:Tsum);
               XtL = XTmp(:,1:Tsum-i);
               NewMeanYt = (Yt*Xt')/(Xt*Xt');
               Yt = Yt-(NewMeanYt*Xt);
               YtL = YtL-(NewMeanYt*XtL);
            else;
               Yt = Yt-MeanY;
               YtL = YtL-MeanY;
            end;
            Corr = ((1/Tsum)*Yt*YtL')./(SVE.CorrData(p).StdY*SVE.CorrData(p).StdY');
            SVE.CorrData(p).CorrY(:,AutoCovHorizon+1+i) = vech(Corr');
            SVE.CorrData(p).CorrY(:,AutoCovHorizon+1-i) = vech(Corr);         
         end;
      else;
         pos = (~isnan(Y));
         NaNY = isnan(Y);
         YTmp(NaNY) = 0;
         Tvec = T./sum(pos,2);
         if k>0;
            XTmp = XSim(:,KalmanFirstObservation:T);
            MeanY = diag(Tvec)*(YTmp*XTmp')/(XTmp*XTmp');
            YTmp = YTmp-(MeanY*XTmp);
         else;
            MeanY = Tvec.*((1/Tsum)*sum(YTmp,2));
            YTmp = YTmp-MeanY;
         end;
         YTmp(NaNY) = 0;
         SVE.CorrData(p).MeanY = MeanY;
         SVE.CorrData(p).StdY = sqrt(diag(TMat.*(YTmp*YTmp')));
         %
         % compute simulated skewness
         %
         SVE.CorrData(p).Skew = ((Tvec/Tsum).*sum(YTmp.*YTmp.*YTmp,2))./(SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY);
         %
         % and kurtosis
         %
         SVE.CorrData(p).Kurt = ((Tvec/Tsum).*sum(YTmp.*YTmp.*YTmp.*YTmp,2))./(SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY.*SVE.CorrData(p).StdY);
         %
         % and now for the autocovariances
         %
         SVE.CorrData(p).CorrY = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
         SVE.CorrData(p).CorrY(:,AutoCovHorizon+1) = vech(((1/Tsum)*YTmp*YTmp')./(SVE.CorrData(p).StdY*SVE.CorrData(p).StdY'));
         for i=1:AutoCovHorizon;
            Yt = YTmp(:,1+i:Tsum);
            YtL = YTmp(:,1:Tsum-i);
            NaNYt = isnan(Y(:,1+i:Tsum));
            NaNYtL = isnan(Y(:,1:Tsum-i));
            if k>0;
               Xt = XTmp(:,1+i:Tsum);
               XtL = XTmp(:,1:Tsum-i);
               NewMeanYt = diag(Tvec)*((Yt*Xt')/(Xt*Xt'));
               Yt = Yt-(NewMeanYt*Xt);
               YtL = YtL-(NewMeanYt*XtL);
            else;
               Yt = Yt-MeanY;
               YtL = YtL-MeanY;
            end;
            Yt(NaNYt) = 0;
            YtL(NaNYtL) = 0;
            Corr = (TMat.*(Yt*YtL'))./(SVE.CorrData(p).StdY*SVE.CorrData(p).StdY');
            SVE.CorrData(p).CorrY(:,AutoCovHorizon+1+i) = vech(Corr');
            SVE.CorrData(p).CorrY(:,AutoCovHorizon+1-i) = vech(Corr);         
         end;
      end;
      MeanEstimationTime = (((p-1)/p)*MeanEstimationTime)+((1/p)*toc);
      drawnow;
   end;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   if IsHandle(WaitHandle)==1;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
end;
%
% now we need to set up the distribution results
%
if strcmp(lower(abort),'cancel')==0;
   NumQuants = length(DSGEModel.Percentiles);
   %
   % initialize model based mean, standard deviation, and autocorrelations
   % for the observed variables
   %
   if k>0;
      SVEs.Mean = zeros(n,k);
   else;
      SVEs.Mean = zeros(n,1);
   end;
   SVEs.Std = zeros(n,1);
   SVEs.Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
   SVEs.Skew = zeros(n,1);
   SVEs.Kurt = zeros(n,1);
   for i=1:NumQuants;
      if k>0;
         SVEs.Quantiles(i).Mean = zeros(n,k);
      else;
         SVEs.Quantiles(i).Mean = zeros(n,1);
      end;
      SVEs.Quantiles(i).Std = zeros(n,1);
      SVEs.Quantiles(i).Corr = zeros((n*(n+1))/2,1+(2*AutoCovHorizon));
      SVEs.Quantiles(i).Skew = zeros(n,1);
      SVEs.Quantiles(i).Kurt = zeros(n,1);
      SVEs.Quantiles(i).percent = DSGEModel.Percentiles(i);
   end;
   %
   % setup a wait dialog
   %
   txt = ['Please wait while YADA computes the distributions for the model based sample means, autocorrelations, skewness, and kurtosis of the observed variables. Current variable: ' StringTrim(DSGEModel.VariableNames(1,:)) '.'];
   WaitHandle = WaitDLG(txt,'information',['Autocorrelation Distribution - ' EstType],600,400,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   %
   for i=1:n;
      %
      % setup a wait dialog
      %
      txt = ['Please wait while YADA computes the distributions for the model based sample means, autocorrelations, skewness, and kurtosis of the observed variables. Current variable: ' StringTrim(DSGEModel.VariableNames(i,:)) '.'];
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      % set up a matrix for the data
      %
      if k>0;
         TempMean = zeros(NumPaths,k);
      else;
         TempMean = zeros(NumPaths,1);
      end;
      TempStd = zeros(NumPaths,1);
      TempSkew = zeros(NumPaths,1);
      TempKurt = zeros(NumPaths,1);
      for p=1:NumPaths;
         TempMean(p,:) = SVE.CorrData(p).MeanY(i,:);
         TempStd(p,1) = SVE.CorrData(p).StdY(i);
         TempSkew(p,1) = SVE.CorrData(p).Skew(i);
         TempKurt(p,1) = SVE.CorrData(p).Kurt(i);
      end;
      drawnow;
      %
      % we can now calculate the mean
      %
      SVEs.Mean(i,:) = (1/NumPaths)*sum(TempMean,1);
      SVEs.Std(i,1) = (1/NumPaths)*sum(TempStd,1);
      SVEs.Skew(i,1) = (1/NumPaths)*sum(TempSkew,1);
      SVEs.Kurt(i,1) = (1/NumPaths)*sum(TempKurt,1);
      %
      % now we can sort these and create the quantiles from the sorted
      %
      for j=1:size(TempMean,2)
         TempMeanj = sort(TempMean(:,j));
         if j==1;
            TempStd = sort(TempStd);
            TempSkew = sort(TempSkew);
            TempKurt = sort(TempKurt);
         end;
         if DSGEModel.ConfidenceRegionMethod==1;
            for l=1:NumQuants;
               if DSGEModel.Percentiles(l)<50;
                  QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumPaths);
               else;
                  QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumPaths);
               end;
               SVEs.Quantiles(l).Mean(i,j) = TempMeanj(QuantVal,1);
               if j==1;
                  SVEs.Quantiles(l).Std(i,1) = TempStd(QuantVal,1);
                  SVEs.Quantiles(l).Skew(i,1) = TempSkew(QuantVal,1);
                  SVEs.Quantiles(l).Kurt(i,1) = TempKurt(QuantVal,1);
               end;
            end;
         else;
            for l=1:floor(NumQuants/2);
               ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumPaths/100);
               if j==1;
                  BoundsStd = HighestProbabilityDensityBand(TempStd,ConfDraws);
                  SVEs.Quantiles(l).Std(i,1) = BoundsStd(1);
                  SVEs.Quantiles(NumQuants-l+1).Std(i,1) = BoundsStd(2);
                  BoundsSkew = HighestProbabilityDensityBand(TempSkew,ConfDraws);
                  SVEs.Quantiles(l).Skew(i,1) = BoundsSkew(1);
                  SVEs.Quantiles(NumQuants-l+1).Skew(i,1) = BoundsSkew(2);
                  BoundsKurt = HighestProbabilityDensityBand(TempKurt,ConfDraws);
                  SVEs.Quantiles(l).Kurt(i,1) = BoundsKurt(1);
                  SVEs.Quantiles(NumQuants-l+1).Kurt(i,1) = BoundsKurt(2);
               end;
               Bounds = HighestProbabilityDensityBand(TempMeanj,ConfDraws);
               SVEs.Quantiles(l).Mean(i,j) = Bounds(1);
               SVEs.Quantiles(NumQuants-l+1).Mean(i,j) = Bounds(2);
            end;
         end;
         drawnow;
      end;
   end;
   %
   % now we need to deal with the autocorrelations
   %
   SVEs.AutoCovHorizon = (-AutoCovHorizon:1:AutoCovHorizon);
   for j=1:(2*AutoCovHorizon)+1;
      %
      % setup a wait dialog
      %
      txt = ['Please wait while YADA computes the distributions for the model based sample means, autocorrelations, skewness, and kurtosis of the observed variables. Current autocorrelation horizon: ' int2str(j-(AutoCovHorizon+1)) '.'];
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      TempCorr = zeros(NumPaths,(n*(n+1))/2);
      for p=1:NumPaths;
         TempCorr(p,:) = SVE.CorrData(p).CorrY(:,j)';
      end;
      drawnow;
      %
      % we can now calculate the mean
      %
      SVEs.Corr(:,j) = (1/NumPaths)*sum(TempCorr,1)';
      %
      % sort the data and determine the quantile values
      %
      TempCorr = sort(TempCorr);
      if DSGEModel.ConfidenceRegionMethod==1;
         for l=1:NumQuants;
            if DSGEModel.Percentiles(l)<50;
               QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumPaths);
            else;
               QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumPaths);
            end;
            SVEs.Quantiles(l).Corr(:,j) = TempCorr(QuantVal,:)';
         end;
      else;
         for l=1:floor(NumQuants/2);
            ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumPaths/100);
            for t=1:n*(n+1)/2;
               Bounds = HighestProbabilityDensityBand(TempCorr(:,t),ConfDraws);
               SVEs.Quantiles(l).Corr(t,j) = Bounds(1);
               SVEs.Quantiles(NumQuants-l+1).Corr(t,j) = Bounds(2);
            end;
         end;
      end;
      drawnow;
   end;
   %
   % initialize model based mean, standard deviation, and autocorrelations
   % for the state variables
   %
   if k>0;
      SVEs.State.Mean = zeros(r,k);
   else;
      SVEs.State.Mean = zeros(r,1);
   end;
   SVEs.State.Std = zeros(r,1);
   SVEs.State.Corr = zeros((r*(r+1))/2,1+(2*AutoCovHorizon));
   SVEs.State.Skew = zeros(r,1);
   SVEs.State.Kurt = zeros(r,1);
   for i=1:NumQuants;
      if k>0;
         SVEs.State.Quantiles(i).Mean = zeros(r,k);
      else;
         SVEs.State.Quantiles(i).Mean = zeros(r,1);
      end;
      SVEs.State.Quantiles(i).Std = zeros(r,1);
      SVEs.State.Quantiles(i).Corr = zeros((r*(r+1))/2,1+(2*AutoCovHorizon));
      SVEs.State.Quantiles(i).Skew = zeros(r,1);
      SVEs.State.Quantiles(i).Kurt = zeros(r,1);
      SVEs.State.Quantiles(i).percent = DSGEModel.Percentiles(i);
   end;
   %
   % now the state variables
   %
   for i=1:r;
      %
      % setup a wait dialog
      %
      txt = ['Please wait while YADA computes the distributions for the model based sample means, autocorrelations, skewness, and kurtosis of the state variables. Current variable: ' StringTrim(DSGEModel.StateVariableNames(i,:)) '.'];
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      % set up a matrix for the data
      %
      if k>0;
         TempMean = zeros(NumPaths,k);
      else;
         TempMean = zeros(NumPaths,1);
      end;
      TempStd = zeros(NumPaths,1);
      TempSkew = zeros(NumPaths,1);
      TempKurt = zeros(NumPaths,1);
      for p=1:NumPaths;
         TempMean(p,:) = SVE.State.CorrData(p).MeanY(i,:);
         TempStd(p,1) = SVE.State.CorrData(p).StdY(i);
         TempSkew(p,1) = SVE.State.CorrData(p).Skew(i);
         TempKurt(p,1) = SVE.State.CorrData(p).Kurt(i);
      end;
      drawnow;
      %
      % we can now calculate the mean
      %
      SVEs.State.Mean(i,:) = (1/NumPaths)*sum(TempMean,1);
      SVEs.State.Std(i,1) = (1/NumPaths)*sum(TempStd,1);
      SVEs.State.Skew(i,1) = (1/NumPaths)*sum(TempSkew,1);
      SVEs.State.Kurt(i,1) = (1/NumPaths)*sum(TempKurt,1);
      %
      % now we can sort these and create the quantiles from the sorted
      %
      for j=1:size(TempMean,2)
         TempMeanj = sort(TempMean(:,j));
         if j==1;
            TempStd = sort(TempStd);
            TempSkew = sort(TempSkew);
            TempKurt = sort(TempKurt);
         end;
         if DSGEModel.ConfidenceRegionMethod==1;
            for l=1:NumQuants;
               if DSGEModel.Percentiles(l)<50;
                  QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumPaths);
               else;
                  QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumPaths);
               end;
               SVEs.State.Quantiles(l).Mean(i,j) = TempMeanj(QuantVal,1);
               if j==1;
                  SVEs.State.Quantiles(l).Std(i,1) = TempStd(QuantVal,1);
                  SVEs.State.Quantiles(l).Skew(i,1) = TempSkew(QuantVal,1);
                  SVEs.State.Quantiles(l).Kurt(i,1) = TempKurt(QuantVal,1);
               end;
            end;
         else;
            for l=1:floor(NumQuants/2);
               ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumPaths/100);
               if j==1;
                  BoundsStd = HighestProbabilityDensityBand(TempStd,ConfDraws);
                  SVEs.State.Quantiles(l).Std(i,1) = BoundsStd(1);
                  SVEs.State.Quantiles(NumQuants-l+1).Std(i,1) = BoundsStd(2);
                  BoundsSkew = HighestProbabilityDensityBand(TempSkew,ConfDraws);
                  SVEs.State.Quantiles(l).Skew(i,1) = BoundsSkew(1);
                  SVEs.State.Quantiles(NumQuants-l+1).Skew(i,1) = BoundsSkew(2);
                  BoundsKurt = HighestProbabilityDensityBand(TempKurt,ConfDraws);
                  SVEs.State.Quantiles(l).Kurt(i,1) = BoundsKurt(1);
                  SVEs.State.Quantiles(NumQuants-l+1).Kurt(i,1) = BoundsKurt(2);
               end;
               Bounds = HighestProbabilityDensityBand(TempMeanj,ConfDraws);
               SVEs.State.Quantiles(l).Mean(i,j) = Bounds(1);
               SVEs.State.Quantiles(NumQuants-l+1).Mean(i,j) = Bounds(2);
            end;
         end;
         drawnow;
      end;
   end;
   %
   % now we need to deal with the autocorrelations
   %
   SVEs.AutoCovHorizon = (-AutoCovHorizon:1:AutoCovHorizon);
   for j=1:(2*AutoCovHorizon)+1;
      %
      % setup a wait dialog
      %
      txt = ['Please wait while YADA computes the distributions for the model based sample means, autocorrelations, skewness, and kurtosis of the state variables. Current autocorrelation horizon: ' int2str(j-(AutoCovHorizon+1)) '.'];
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      TempCorr = zeros(NumPaths,(r*(r+1))/2);
      for p=1:NumPaths;
         TempCorr(p,:) = SVE.State.CorrData(p).CorrY(:,j)';
      end;
      drawnow;
      %
      % we can now calculate the mean
      %
      SVEs.State.Corr(:,j) = (1/NumPaths)*sum(TempCorr,1)';
      %
      % sort the data and determine the quantile values
      %
      TempCorr = sort(TempCorr);
      if DSGEModel.ConfidenceRegionMethod==1;
         for l=1:NumQuants;
            if DSGEModel.Percentiles(l)<50;
               QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumPaths);
            else;
               QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumPaths);
            end;
            SVEs.State.Quantiles(l).Corr(:,j) = TempCorr(QuantVal,:)';
         end;
      else;
         for l=1:floor(NumQuants/2);
            ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumPaths/100);
            for t=1:n*(n+1)/2;
               Bounds = HighestProbabilityDensityBand(TempCorr(:,t),ConfDraws);
               SVEs.State.Quantiles(l).Corr(t,j) = Bounds(1);
               SVEs.State.Quantiles(NumQuants-l+1).Corr(t,j) = Bounds(2);
            end;
         end;
      end;
      drawnow;
   end;
   %
   % close the wait dialog
   %
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
else;
   SVEs = [];
end;
%
CorrData = SVEs;

%
% end of SteadyStateCorrelationSimulationTheta.m
%
