function MFileName = YADAREBasedBeliefSystem(F,X,XVariableNames,StateVariableNames,ForwardVariableNames,OutputDirectory,ModelName)
% YADAREBasedBeliefSystem: Sets up a belief system where each forward looking variable is explained by
%                          the variables from the functional form taken from the RE solution, but without
%                          imposing the cross-equation restrictions.
%
% USAGE:
%
%       MFileName = YADAREBasedBeliefSystem(F,X,XVariableNames,StateVariableNames,ForwardVariableNames,ModelName)
%
% REQUIRED INPUT:  F (matrix) of dimension r x r with the RE-based solution on lagged state variables.
%
%                  X (matrix) of dimension k x T with the deterministic variables (if any).
%
%                  XVariableNames (string matrix) of dimension k x k1 with the names of the deterministic variables.
%
%                  StateVariableNames (string matrix) of dimension r x r1 with the names of the state variables.
%
%                  ForwardVariableNames (string matrix) of dimension f x f1 with the names of the forward
%                                       looking variables.
%
%                  OutputDirectory (string) with the full path to the directory where the m-file with the RE-based
%                                       functional form of the solution should be stored.
%
%                  ModelName (string) with the name of the DSGE model.
%
% REQUIRED OUTPUT: MFileName (string) with the name of the m-file written to disk.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: October 24, 2024.
%                        This version: October 25, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 25-10-2024: Continued working on the function.
%
% * 28-10-2024: Finished the function.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
MFileName = [OutputDirectory '\YADA' strrep(strrep(ModelName,'-','_'),' ','') 'REBeliefSystem.m'];
%m = 1;
%Name = 'RE-Solution-Based';
%
% first we check if we have a constant
%
[k,T] = size(X);
ConstPos = 0;
if k>0;
   ConstPos = LocateStr(XVariableNames,'const');
   if ConstPos==0;
      %
      % we will now check if X has a unit vector
      %
      for j=1:k;
         ConstTest = (X(j,:)==ones(1,T));
         if sum(ConstTest)==k2;
            ConstPos = j;
            break;
         end;
      end;
   end;
end;
if ConstPos>0
   NumConst = 1;
else;
   NumConst = 0;
end;
r = size(StateVariableNames,1);
f = size(ForwardVariableNames,1);
%
% write to a matlab m-file
%
NameLength = length(GetName(MFileName));
fid = fopen(MFileName,'wt');
fprintf(fid,'function BeliefSys = %s(XVariableNames,StateVariableNames,ForwardVariableNames);\n',GetName(MFileName));
fprintf(fid,'%% %s: Sets up a belief system where each forward looking variable is explained by\n',GetName(MFileName));
fprintf(fid,'%% %s  the variables from the functional form taken from the RE solution, but without\n',SpaceStr(NameLength));
fprintf(fid,'%% %s  imposing the cross-equation restrictions.\n',SpaceStr(NameLength));
fprintf(fid,'%%\n');
fprintf(fid,'%% USAGE:\n');
fprintf(fid,'%%\n');
fprintf(fid,'%%       BeliefSys = %s(XVariableNames,StateVariableNames,ForwardVariableNames)\n',GetName(MFileName));
fprintf(fid,'%%\n');
fprintf(fid,'%% REQUIRED INPUT:  XVariableNames (string matrix) of dimension k x k1 with the names of the deterministic variables.\n');
fprintf(fid,'%%\n');
fprintf(fid,'%%                  StateVariableNames (string matrix) of dimension r x r1 with the names of the state variables.\n');
fprintf(fid,'%%\n');
fprintf(fid,'%%                  ForwardVariableNames (string matrix) of dimension f x f1 with the names of the forward\n');
fprintf(fid,'%%\n');
fprintf(fid,'%% REQUIRED OUTPUT: BeliefSys (structure) with fields "m" an integer denoting the maximum number of lags of the state\n');
fprintf(fid,'%%                                       variables in the belief equations; "Name" a string with the  name of the\n');
fprintf(fid,'%%                                       belief system; and G a cell array with f matrices, one for each forward\n');
fprintf(fid,'%%                                       looking variable of dimension (k+r*m) x g_j where g_j is the number of\n');
fprintf(fid,'%%                                       explanatory variables in the belief equation for forward looking variable j.\n');
fprintf(fid,'%%\n');
fprintf(fid,'%%\n');
fprintf(fid,'%%                       Written by: Anders Warne\n');
fprintf(fid,'%%                                   New Area Wide Model Project\n');
fprintf(fid,'%%                                   Forecasting and Policy Modelling Division\n');
fprintf(fid,'%%                                   European Central Bank (ECB)\n');
fprintf(fid,'%%                                   Email: anders.warne@ecb.europa.eu\n');
fprintf(fid,'%%                                   Copyright  2006-%s European Central Bank.\n',datestr(now,10));
fprintf(fid,'%%\n');
fprintf(fid,'%%                        This version: %s %s, %s.\n',datestr(now,'mmmm'),datestr(now,'dd'),datestr(now,10));
fprintf(fid,'%%\n');
fprintf(fid,'%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n');
fprintf(fid,'%%\n');
fprintf(fid,'%% LICENSE INFORMATION:\n');
fprintf(fid,'%%\n');
fprintf(fid,'%%      YADA is free software: you can redistribute it and/or modify\n');
fprintf(fid,'%%      it under the terms of the GNU General Public License as published by\n');
fprintf(fid,'%%      the Free Software Foundation, either version 3 of the License, or\n');
fprintf(fid,'%%      (at your option) any later version.\n');
fprintf(fid,'%%\n');
fprintf(fid,'%%      This program is distributed in the hope that it will be useful,\n');
fprintf(fid,'%%      but WITHOUT ANY WARRANTY; without even the implied warranty of\n');
fprintf(fid,'%%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n');
fprintf(fid,'%%      GNU General Public License for more details.\n');
fprintf(fid,'%%\n');
fprintf(fid,'%%      You should have received a copy of the GNU General Public License\n');
fprintf(fid,'%%      along with this program.  If not, see <http://www.gnu.org/licenses/>.\n');
fprintf(fid,'%%\n');
fprintf(fid,'%%      YADA is released under the GNU General Public License, Version 3,\n');
fprintf(fid,'%%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of\n');
fprintf(fid,'%%      the program was last modified by the ECB on the "This version" date\n');
fprintf(fid,'%%      above.\n');
fprintf(fid,'%%\n');
fprintf(fid,'%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n');
fprintf(fid,'%%\n\n');
fprintf(fid,'%%\n');
fprintf(fid,'%% initialize\n');
fprintf(fid,'%%\n');
fprintf(fid,'BeliefSys.m = 1;\n');
fprintf(fid,'BeliefSys.Name = ''RE-Solution-Based'';\n');
fprintf(fid,'BeliefSys.G = [];\n');
fprintf(fid,'%%\n');
fprintf(fid,'%% determine the number of forward looking variables\n');
fprintf(fid,'%%\n');
fprintf(fid,'f = size(ForwardVariableNames,1);\n');
fprintf(fid,'G = cell(f,1);\n');
fprintf(fid,'for j=1:f;\n');
for j=1:f;
   %
   % first check which row in F should be examined
   %
   CurrForward = StringTrim(ForwardVariableNames(j,:));
   FPos = MatchStrRow(StateVariableNames,CurrForward);
   %
   % now determine all nonzero elements of row FPos in F
   %
   LagPos = zeros(1,r);
   for i=1:r;
      if abs(F(FPos,i))>=1.0e-08;
         LagPos(i) = i; 
      end;
   end;
   LagPos = LagPos(LagPos>0);
   fprintf(fid,'   if j==%0.0f;\n',j);
   fprintf(fid,'      Ghelp = zeros(%0.0f,%0.0f);\n',k+r,NumConst+length(LagPos));
   if ConstPos>0;
      fprintf(fid,'      %%\n');
      fprintf(fid,'      %% add constant term\n');
      fprintf(fid,'      %%\n');
      fprintf(fid,'      Ghelp(%0.0f,1) = 1;\n',ConstPos);
   end;
   fprintf(fid,'      %%\n');
   fprintf(fid,'      %% current forward looking variable: %s\n',CurrForward);
   for i=1:length(LagPos);
      fprintf(fid,'      %%\n');
      fprintf(fid,'      %% PLM explanatory variable: %s\n',StringTrim(StateVariableNames(LagPos(i),:)));
      fprintf(fid,'      %%\n');
      fprintf(fid,'      Ghelp(%0.0f,%0.0f) = 1;\n',k+LagPos(i),NumConst+i);
   end;
   fprintf(fid,'      G{%0.0f,1} = Ghelp;\n',j);
   fprintf(fid,'   end;\n');
end;
fprintf(fid,'end;\n');
fprintf(fid,'%%\n');
fprintf(fid,'%% complete the output structure\n');
fprintf(fid,'%%\n');
fprintf(fid,'BeliefSys.G = G;\n\n');
fprintf(fid,'%%\n');
fprintf(fid,'%% end of %s\n',GetFilename(MFileName));
fprintf(fid,'%%\n');
fclose(fid);

%
% end of YADAREBasedBeliefSystem.m
%
