function DoneCalc = AdaptiveLearningDSGECondPredictiveMomentsWZ(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,h,TotalDraws,CurrChain,IsPosterior,Weights,SmpStr,ComputePITMoment)
% AdaptiveLearningDSGECondPredictiveMomentsWZ: Computes the conditional predictive mean and covariance matrix under the Waggoner-Zha approach for
%                                              a DSGE model with adaptive learning.
%
% USAGE:
%
%       DoneCalc = AdaptiveLearningDSGECondPredictiveMomentsWZ(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,...
%                                                            X,Z,U,h,TotalDraws,CurrChain,IsPosterior,Weights,SmpStr,ComputePITMoment)
%
% REQUIRED INPUT:  thetaPostSample (matrix) of dimension NumDrawsxNumParam with draws from the posterior or prior
%                                  distribution of the original DSGE model parameters.
%
%                  thetaMode (vector) with the posterior mode or initial value of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                  needs the former field to know which parameter name corresponds to
%                                  position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                  DSGE model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  X (matrix) of dimension kxh with the observations of the exogenous variables for
%                                  the prediction paths.
%
%                  Z (matrix) of dimension mxg with the out-of-sample data we shall condition the
%                                  predictions on.
%
%                  U (matrix) of dimension mxg with initial values for the mapping between the observed
%                                  data and the conditioning data.
%
%                  h (integer) with the prediction horizon.
%
%                  TotalDraws (integer) with the total number of parameter draws that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                  if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                                 likelihood tempering posterior sampler.
%
%                  SmpStr (string) which reflects the first and last period in the prediction sample.
%
%                  ComputePITMoment (boolean) that takes the value 1 if moments useable for the PITs should
%                                 be computed and 0 otherwise.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: August 7, 2024.
%                        This version: August 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 08-08-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DoneCalc = 0;
[NumDraws,NumParam] = size(thetaPostSample);
k = DSGEModel.k;
n = DSGEModel.n;
[qm,g] = size(Z);
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% check if we should copy the belief system file
%
if DSGEModel.BeliefSystem==2;
   [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
   end;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% try to solve the model for the posterior mode estimates
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch ME;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],500,600,CurrINI);
   return;
end;
if length(size(H))==3;
   if size(H,3)<DSGEModel.LastPeriod+h;
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations plus forecasts (T+h = ' int2str(DSGEModel.LastPeriod+h) '). YADA has therefore aborted from the conditional predictions.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
%
% Determine the belief system matrices
%
if DSGEModel.BeliefSystem==1;
   BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
else;
   BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
end;
%
% Compute the transformation of the structural from that we use when solving the model
%
r = length(DSGEModel.StateVariablePositions);
f = length(DSGEModel.ForwardVariablePositions);
m = BeliefSys.m;
q = size(B0,2);
SMatrix = eye(r);
Sperp = SMatrix(:,setdiff((1:r),DSGEModel.ForwardVariablePositions));
SMatrix = SMatrix(:,DSGEModel.ForwardVariablePositions);
%
% get the structural form matrices
%
[Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
%
% Determine the first observation to use when computing log-likelihood
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
FirstPeriod = DSGEModel.FirstPeriod;
LastPeriod = DSGEModel.LastPeriod;
%
% Setup all the basic data matrix for the deterministic variables
% used for initializing Sigmax
%
if k>0
   x = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
else;
   x = [];
end;
T = LastPeriod-(FirstPeriod+KalmanFirstObservation-1)+1;
%
% Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
%
[betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue);
ConstPos = 0;
if k>0;
   ConstPos = LocateStr(DSGEModel.XVariableNames,'const');
   if ConstPos==0;
      %
      % we will now check if X has a unit vector
      %
      Xhist = DSGEModel.X(:,FirstPeriod:LastPeriod);
      T = size(Xhist,2);
      for j=1:k;
         ConstTest = (Xhist==ones(1,T));
         if sum(ConstTest)==T;
            ConstPos = j;
            break;
         end;
      end;
   end;
   xbar = zeros(k,1);
   xbar(ConstPos) = 1;
else;
   xbar = [];
end;
%
% call the transformation function
%
[TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
%
% compute initial values for the solution matrices
%
if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
   [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
else;
   [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
%
% check if the adaptive learning parameters allows for updating of the beliefs
%
IsALUpdate = (CheckAdaptiveLearningFixedBeliefs(ModelParameters)==0);
%
% determine initial conditions for the forecasts
%
if MatlabNumber>=8.0;
   try;
      if IsALUpdate==1;
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         else;
            [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      else;
         [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
      end;
   catch ME;
      txt = ['YADA caught the following error message from the Kalman filter routine under adaptive learning: ' YADALastError(ME)];
      About(txt,'error','Error - Kalman Filter',200,500,CurrINI);
      return;
   end;
else;
   try;
      if IsALUpdate==1;
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         else;
            [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      else;
         [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
      end;
   catch;
      txt = ['YADA caught the following error message from the Kalman filter routine under adaptive learning: ' YADALastError];
      About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
      return;
   end;
end;
if bstatus==0;
   txt = 'The Kalman filter estimates of the belief coefficients, beta, produced Nan''s or imaginary numbers. YADA was therefore unable to continue with the conditional predictive distributions.';
   About(txt,'information','Conditional Predictive Distributions',300,600,CurrINI);
   return;
end;
%
% create the mapping matrices for conditioning variables
%
K1 = DSGEModel.K1(:,DSGEModel.ConditionalVariables);
if length(size(DSGEModel.K2))==2;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables);
   K2length = 2;
else;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables,:);
   K2length = 3;
end;
SCond = (isequal(U,zeros(size(U)))&isequal(K2,zeros(size(K2))));
if (SCond==1)&(K2length==3);
   K2 = K2(:,:,1);
   K2length = 2;
end;
%
% check if the user wants to take the conditioning infortmation into account
% when determining the mean and covariance matrix for the states at T.
%
if (DSGEModel.KsiUseCondData==1)&(SCond==1);
   if (k>0);
      XPred = X(:,1:g);
   else;
      XPred = [];
   end;
   if length(size(H))==2;
      [KsiTTz,PTTz] = AdaptiveLearningCondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),Yhat(:,T),DSGEModel.Y(:,LastPeriod),XPred,Z,K1,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
   else;
      [KsiTTz,PTTz] = AdaptiveLearningCondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),Yhat(:,T),DSGEModel.Y(:,LastPeriod),XPred,Z,K1,A,H(:,:,LastPeriod+1:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
   end;
   KsiTTz = Ksitt(:,T);
   PTTz = Ptt(:,:,T);
elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
   %
   % we need to run the Kalman smoother
   %
   [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
   PtT = PtT(:,:,T-g+1:T);
   KsitT = KsitT(:,T-g+1:T);
   if isempty(wtT)==1;
      wtT = zeros(n,g);
   else;
      wtT = wtT(:,T-g+1:T);
   end;
   if length(size(H))==2;
      if k>0;
         if g>1;
            Xtra = DSGEModel.X(:,LastPeriod-g+2:LastPeriod);
         else;
            Xtra = [];
         end;
         [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
      else;
         [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
      end;
   else;
      if k>0;
         if g>1;
            Xtra = DSGEModel.X(:,LastPeriod-g+2:LastPeriod);
         else;
            Xtra = [];
         end;
         [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H(:,:,LastPeriod-g+2:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
      else;
         [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H(:,:,LastPeriod-g+2:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
      end;
   end;
else;
   KsiTTz = Ksitt(:,T);
   PTTz = Ptt(:,:,T);
end;
PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
%
% Now we need to determine KsiTT for just the last period
%
KsiTT = Ksitt(:,T);
%
% the last estimates
%
MT1 = Mtilde(:,:,T);
FT1 = Ftilde(:,:,T);
BT1 = Btilde(:,:,T);
%
% initialize solution matrices for possible later use
%
if DSGEModel.FixedBeliefsForProj==1;
   MTh = MT1;
   FTh = FT1;
   BTh = BT1;
else;
   MTh = zeros(r,k,h);
   FTh = zeros(r,r,h);
   BTh = zeros(r,q,h);
end;
%
% check if shocks can be used
%
if length(size(H))==2;
   if rank(K1'*H'*BT1)<qm;
      txt = ['The conditioning assumptions you have selected cannot be used due to a singularity in the solution algorithm. Estimation of the conditional predictive distribution' lower(NameStr) ' has therfore been aborted.'];
      About(txt,'error','Error - Conditional Shocks',150,600,CurrINI);
      PredPaths = [];
      return;
   end;
else;
   BadEtaMat = 0;
   for i=1:g;
      if rank(K1'*H(:,:,LastPeriod+i)'*BT1)<qm;
         txt = ['The conditioning assumptions you have selected cannot be used due to a singularity in the solution algorithm. Estimation of the conditional predictive distribution' lower(NameStr) ' has therfore been aborted.'];
         About(txt,'error','Error - Conditional Shocks',150,600,CurrINI);
         PredPaths = [];
         BadEtaMat = 1;
         break;
      end;
   end;
   if BadEtaMat==1;
      return;
   end;
end;
%
% stack the model to determine the shock distribution
%
Xbar = zeros(n*g,1);
XTi = zeros(r,1);
G = zeros(n*g,r);
WTg = zeros(n*g,1);
K = zeros(n*g,qm*g);
DT1 = zeros(n*g,q*g);
ZTg = zeros(qm*g,1);
UT = zeros(qm*g,1);
Fexp = eye(r);
if length(size(H))==2;
   if DSGEModel.FixedBeliefsForProj==1;
      for i=1:g;
         Fexp = FTh*Fexp;
         Ind = (1+((g-i)*n):(g-i+1)*n);
         Ind1 = (1+((g-i)*q):(g-i+1)*q);
         Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
         if k>0;
            XTi = (MTh*X(:,i))+(FTh*XTi);
            Xbar(Ind,1) = (A'*X(:,i))+(H'*XTi);
         end;
         G(Ind,:) = H'*Fexp;
         DT1(Ind,Ind1) = H'*BTh;
         K(Ind,Ind2) = K1;
         Fi = FTh;
         for j=1:i-1;
            Ind1 = Ind1+q;
            Ind3 = Ind+(j*n);
            DT1(Ind,Ind1) = H'*Fi*BTh;
            if K2length==2;
               K(Ind3,Ind2) = K2;
            else;
               if j<=size(K2,3);
                  K(Ind3,Ind2) = K2(:,:,j);
               end;
            end;
            Fi = Fi*FTh;
         end;
         ZTg(Ind2,1) = Z(:,i);
         UT(Ind2,1) = U(:,g);
      end;
   else;
      %
      % we have updated beliefs
      %
      for i=1:g;
         Fexp = FTh(:,:,i)*Fexp;
         Ind = (1+((g-i)*n):(g-i+1)*n);
         Ind1 = (1+((g-i)*q):(g-i+1)*q);
         Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
         if k>0;
            XTi = (MTh(:,:,i)*X(:,i))+(FTh(:,:,i)*XTi);
            Xbar(Ind,1) = (A'*X(:,i))+(H'*XTi);
         end;
         G(Ind,:) = H'*Fexp;
         DT1(Ind,Ind1) = H'*BTh(:,:,i);
         K(Ind,Ind2) = K1;
         Fi = FTh(:,:,i);
         for j=1:i-1;
            Ind1 = Ind1+q;
            Ind3 = Ind+(j*n);
            DT1(Ind,Ind1) = H'*Fi*BTh(:,:,i);
            if K2length==2;
               K(Ind3,Ind2) = K2;
            else;
               if j<=size(K2,3);
                  K(Ind3,Ind2) = K2(:,:,j);
               end;
            end;
            Fi = Fi*FTh(:,:,i-j);
         end;
         ZTg(Ind2,1) = Z(:,i);
         UT(Ind2,1) = U(:,g);
         %
         % require new solution matrices
         %
         if i==1;
            MTh(:,:,i+1) = LastSolution.MTT;
            FTh(:,:,i+1) = LastSolution.FTT;
            BTh(:,:,i+1) = LastSolution.BTT;
            betaTiT = LastSolution.betaTT;
         elseif (i>1)&(i<g);
            %
            % we first need to project betaTiT
            %
            betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
            else;
               [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
            MTh(:,:,i+1) = MTiT;
            FTh(:,:,i+1) = FTiT;
            BTh(:,:,i+1) = BTiT;
         end;
      end;
   end;
else;
   if DSGEModel.FixedBeliefsForProj==1;
      for i=1:g;
         Fexp = FTh*Fexp;
         Ind = (1+((g-i)*n):(g-i+1)*n);
         Ind1 = (1+((g-i)*q):(g-i+1)*q);
         Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
         if k>0;
            XTi = (MTh*X(:,i))+(FTh*XTi);
            Xbar(Ind,1) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*XTi);
         end;
         G(Ind,:) = H(:,:,LastPeriod+i)'*Fexp;
         DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*BTh;
         K(Ind,Ind2) = K1;
         Fi = FTh;
         for j=1:i-1;
            Ind1 = Ind1+q;
            Ind3 = Ind+(j*n);
            DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*Fi*BTh;
            if K2length==2;
               K(Ind3,Ind2) = K2;
            else;
               if j<=size(K2,3);
                  K(Ind3,Ind2) = K2(:,:,j);
               end;
            end;
            Fi = Fi*FTh;
         end;
         ZTg(Ind2,1) = Z(:,i);
         UT(Ind2,1) = U(:,g);
      end;
   else;
      %
      % we have updated beliefs
      %
      for i=1:g;
         Fexp = FTh(:,:,i)*Fexp;
         Ind = (1+((g-i)*n):(g-i+1)*n);
         Ind1 = (1+((g-i)*q):(g-i+1)*q);
         Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
         if k>0;
            XTi = (MTh(:,:,i)*X(:,i))+(FTh(:,:,i)*XTi);
            Xbar(Ind,1) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*XTi);
         end;
         G(Ind,:) = H(:,:,LastPeriod+i)'*Fexp;
         DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*BTh(:,:,i);
         K(Ind,Ind2) = K1;
         Fi = FTh(:,:,i);
         for j=1:i-1;
            Ind1 = Ind1+q;
            Ind3 = Ind+(j*n);
            DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*Fi*BTh(:,:,i);
            if K2length==2;
               K(Ind3,Ind2) = K2;
            else;
               if j<=size(K2,3);
                  K(Ind3,Ind2) = K2(:,:,j);
               end;
            end;
            Fi = Fi*FTh(:,:,i-j);
         end;
         ZTg(Ind2,1) = Z(:,i);
         UT(Ind2,1) = U(:,g);
         %
         % require new solution matrices
         %
         if i==1;
            MTh(:,:,i+1) = LastSolution.MTT;
            FTh(:,:,i+1) = LastSolution.FTT;
            BTh(:,:,i+1) = LastSolution.BTT;
            betaTiT = LastSolution.betaTT;
         elseif (i>1)&(i<h);
            %
            % we first need to project betaTiT
            %
            betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
            else;
               [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
            MTh(:,:,i+1) = MTiT;
            FTh(:,:,i+1) = FTiT;
            BTh(:,:,i+1) = BTiT;
         end;
      end;
   end;
end;
%
% create a vector with positions for the conditioning assumptions that should be used
%
PosVec = (1:qm*g)' .* ~isnan(ZTg);
PosVec = PosVec(PosVec>0);
%
% adjust for missing restrictions
%
K = K(:,PosVec);
ZTg = ZTg(PosVec,1);
UT = UT(PosVec,1);
%
% speed-up code suggested by Ingvar Strid
%
KpD = K'*DT1;
KpDDpK = KpD*KpD';
DpKInvKpDDpK = KpD'/KpDDpK;
%
% check the rank of K'D
%
if rank(KpD)<length(PosVec);
   txt = ['Some of the conditioning assumptions are linearly dependent. The rank of K''*D is ' int2str(rank(KpD)) ', while the number of conditioning assumptions is ' int2str(length(PosVec)) '. YADA is therefore unable to perform the conditional forecasts for the Waggoner-Zha approach.'];
   About(txt,'error','Error - Conditioning Assumptions',200,500,CurrINI);
   PredPaths = [];
   return;
end;
%
% compute the conditioning relation when state vector and measurement errors are at mean
%
kbarTg = ZTg-UT-(K'*(Xbar+(G*KsiTTz)));
%
% compute the mean of the shocks when state vector and measurement errors are at mean
%
mubarTg = DpKInvKpDDpK*kbarTg;
%
% compute the idempotent covariance matrix for the shocks
%
SigmaTg = eye(q*g)-(DpKInvKpDDpK*KpD);
SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
%
% compute the predictive mean for the fixed parameter case
%
if k>0;
   Ax = A'*X;
else;
   Ax = zeros(n,h);
   X = zeros(r,h)
end;
PredMean = zeros(n,h);
KsiT = KsiTTz;
etaShocks = zeros(q,h);
etaShocks(:,1:g) = fliplr(reshape(mubarTg,q,g));
xiStates = zeros(r,h);
ZPos = ~isnan(Z);
if length(size(H))==2;
   if DSGEModel.FixedBeliefsForProj==1;
      %
      % we have fixed belief coefficients
      %
      for i=1:h;
         %
         % update the states
         %
         if k>0;
            KsiT = (MT1*X(:,i))+(FT1*KsiT)+(BT1*etaShocks(:,i));
         else;
            KsiT = (FT1*KsiT)+(BT1*etaShocks(:,i));
         end;
         xiStates(:,i) = KsiT;
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredMean(:,i) = (A'*X(:,i))+(H'*KsiT);
         else;
            PredMean(:,i) = H'*KsiT;
         end;
      end;
   else;
      %
      % belief coefficients are projected forward
      %
      for i=1:h;
         %
         % require new solution matrices
         %
         if (i>g)&(i<h);
            %
            % we first need to project betaTiT
            %
            betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
            else;
               [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
            MTh(:,:,i+1) = MTiT;
            FTh(:,:,i+1) = FTiT;
            BTh(:,:,i+1) = BTiT;
         end;
         %
         % update the states
         %
         if k>0;
            KsiT = (MTh(:,:,i)*X(:,i))+(FTh(:,:,i)*KsiT)+(BTh(:,:,i)*etaShocks(:,i));
         else;
            KsiT = (FTh(:,:,i)*KsiT)+(BTh(:,:,i)*etaShocks(:,i));
         end;
         xiStates(:,i) = KsiT;
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredMean(:,i) = (A'*X(:,i))+(H'*KsiT);
         else;
            PredMean(:,i) = H'*KsiT;
         end;
      end;
   end;
else;
   if DSGEModel.FixedBeliefsForProj==1;
      %
      % we have fixed belief coefficients
      %
      for i=1:h;
         %
         % update the states
         %
         if k>0;
            KsiT = (MT1*X(:,i))+(FT1*KsiT)+(BT1*etaShocks(:,i));
         else;
            KsiT = (FT1*KsiT)+(BT1*etaShocks(:,i));
         end;
         xiStates(:,i) = KsiT;
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredMean(:,i) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*KsiT);
         else;
            PredMean(:,i) = H(:,:,LastPeriod+i)'*KsiT;
         end;
      end;
   else;
      %
      % belief coefficients are projected forward
      %
      for i=1:h;
         %
         % require new solution matrices
         %
         if (i>g)&(i<h);
            %
            % we first need to project betaTiT
            %
            betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
            else;
               [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
            MTh(:,:,i+1) = MTiT;
            FTh(:,:,i+1) = FTiT;
            BTh(:,:,i+1) = BTiT;
         end;
         %
         % update the states
         %
         if k>0;
            KsiT = (MTh(:,:,i)*X(:,i))+(FTh(:,:,i)*KsiT)+(BTh(:,:,i)*etaShocks(:,i));
         else;
            KsiT = (FTh(:,:,i)*KsiT)+(BTh(:,:,i)*etaShocks(:,i));
         end;
         xiStates(:,i) = KsiT;
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredMean(:,i) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*KsiT);
         else;
            PredMean(:,i) = H(:,:,LastPeriod+i)'*KsiT;
         end;
      end;
   end;
end;
%
% store mode results for use if required
%
Hmode = H;
Rmode = R;
BT1mode = BT1;
MT1mode = MT1;
FT1mode = FT1;
BThmode = BTh;
MThmode = MTh;
FThmode = FTh;
PTTzmode = PTTz;
PredMeanmode = PredMean;
Gmode = G;
DT1mode = DT1;
SigmaTgmode = SigmaTg;
DpKInvKpDDpKmode = DpKInvKpDDpK;
%
if isempty(DSGEModel.X)==0;
   Xdata = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
else;
   Xdata = [];
end;
%
% initialize
%
StateCov = zeros(n,n,h);
ShockCov = StateCov;
MeasureCov = zeros(n,n,h);
ParameterCov = StateCov;
MeanPredictY = zeros(n,h);
PITMoments = [];
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' int2str(NumDraws) ' parameter draws'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Draw Number:';
   %
   ProgressStructure.name = ['Marginal Predictive Moments' PriorHeader ' - ' int2str(h) ' Periods Ahead'];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes marginal predictive moments for the original data. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Marginal Predictive Moments ' PriorHeader ' - ' int2str(h) ' Periods Ahead'],600,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% begin the loop
%
abort = '';
StopEstimation = 0;
ZPos = ~isnan(Z);
if length(size(H))==2;
   if DSGEModel.FixedBeliefsForProj==1;
      %
      % we have fixed belief coefficients
      %
      for ParamIndex=1:NumDraws;
         if DSGEModel.ShowProgress==1;
            abort = get(WaitHandle,'UserData');
            if strcmp(abort,'cancel')==1;
               StopEstimation = 1;
               break;
            else;
               ProgressDLG([ParamIndex/NumDraws ParamIndex]);
            end;
         end;
         %
         % Solve the model for the current parameter vector
         %
         theta = thetaPostSample(ParamIndex,:)';
         [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         if status~=1;
            %
            % use posterior mode or initial values as a backup
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % get the structural form matrices
         %
         [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
         %
         % Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
         %
         [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
         %
         % call the transformation function
         %
         [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
         %
         % compute initial values for the solution matrices
         %
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
         else;
            [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
         %
         % run the Kalman filter
         %
         if IsALUpdate==1;
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            else;
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         end;
         %
         KsiTT = Ksitt(:,T);
         MT1 = Mtilde(:,:,T);
         FT1 = Ftilde(:,:,T);
         BT1 = Btilde(:,:,T);
         if rank(K1'*H'*BT1)==qm;
            %
            % get the initial conditions for the state variables
            %
            if (DSGEModel.KsiUseCondData==1)&(SCond==1);
               if (k>0);
                  XPred = X(:,1:g);
               else;
                  XPred = [];
               end;
               [KsiTTz,PTTz] = AdaptiveLearningCondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),Yhat(:,T),DSGEModel.Y(:,LastPeriod),XPred,Z,K1,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
               KsiTTz = Ksitt(:,T);
               PTTz = Ptt(:,:,T);
            elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
               %
               % we need to run the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               PtT = PtT(:,:,T-g+1:T);
               KsitT = KsitT(:,T-g+1:T);
               if isempty(wtT)==1;
                  wtT = zeros(n,g);
               else;
                  wtT = wtT(:,T-g+1:T);
               end;
               if k>0;
                  if g>1;
                     Xtra = DSGEModel.X(:,LastPeriod-g+2:LastPeriod);
                  else;
                     Xtra = [];
                  end;
                  [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
               else;
                  [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
               end;
            else;
               KsiTTz = Ksitt(:,T);
               PTTz = Ptt(:,:,T);
            end;
            PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
            if k>0;
               Ax = A'*X;
            else;
               Ax = zeros(n,h);
               X = zeros(r,h)
            end;
            %
            % compute the mean prediction
            %
            PredMean = zeros(n,h);
            KsiT = KsiTTz;
            Fexp = eye(r);
            G = zeros(n*g,r);
            DT1 = zeros(n*g,q*g);
            Xbar = zeros(n*g,1);
            XTi = zeros(r,1);
            etaShocks = zeros(q,h);
            xiStates = zeros(r,h);
            for i=1:g;
               Fexp = FT1*Fexp;
               Ind = (1+((g-i)*n):(g-i+1)*n);
               Ind1 = (1+((g-i)*q):(g-i+1)*q);
               if k>0;
                  XTi = (MT1*X(:,i))+(FT1*XTi);
                  Xbar(Ind,1) = (A'*X(:,i))+(H'*XTi);
               end;
               G(Ind,:) = H'*Fexp;
               DT1(Ind,Ind1) = H'*BT1;
               Fi = FT1;
               for j=1:i-1;
                  Ind1 = Ind1+q;
                  DT1(Ind,Ind1) = H'*Fi*BT1;
                  Fi = Fi*FT1;
               end;
            end;
            %
            % speed-up code suggested by Ingvar Strid
            %
            KpD = K'*DT1;
            KpDDpK = KpD*KpD';
            DpKInvKpDDpK = KpD'/KpDDpK;
            %
            % compute the conditioning relation when state vector and measurement errors are at mean
            %
            kbarTg = ZTg-UT-(K'*(Xbar+(G*KsiTTz)));
            %
            % compute the mean of the shocks when state vector and measurement errors are at mean
            %
            mubarTg = DpKInvKpDDpK*kbarTg;
            %
            % compute the idempotent covariance matrix for the shocks
            %
            SigmaTg = eye(q*g)-(DpKInvKpDDpK*KpD);
            SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
            %
            % compute the mean shock values
            %
            etaShocks(:,1:g) = fliplr(reshape(mubarTg,q,g));
            for i=1:h;
               %
               % update the states
               %
               if k>0;
                  KsiT = (MT1*X(:,i))+(FT1*KsiT)+(BT1*etaShocks(:,i));
               else;
                  KsiT = (FT1*KsiT)+(BT1*etaShocks(:,i));
               end;
               xiStates(:,i) = KsiT;
               %
               % compute the conditional mean predictions
               %
               if k>0;
                  PredMean(:,i) = (A'*X(:,i))+(H'*KsiT);
               else;
                  PredMean(:,i) = H'*KsiT;
               end;
            end;
            TestValue = max(max(abs((PredMean-PredMeanmode)./PredMeanmode)));
            if TestValue>=100;
               %
               % skip parameters and use the "mode" values instead
               %
               H = Hmode;
               R = Rmode;
               BT1 = BT1mode;
               MT1 = MT1mode;
               FT1 = FT1mode;
               PTTz = PTTzmode;
               PredMean = PredMeanmode;
               G = Gmode;
               DT1 = DT1mode;
               SigmaTg = SigmaTgmode;
               DpKInvKpDDpK = DpKInvKpDDpKmode;
            end;
         else;
            H = Hmode;
            R = Rmode;
            BT1 = BT1mode;
            MT1 = MT1mode;
            FT1 = FT1mode;
            PTTz = PTTzmode;
            PredMean = PredMeanmode;
            G = Gmode;
            DT1 = DT1mode;
            SigmaTg = SigmaTgmode;
            DpKInvKpDDpK = DpKInvKpDDpKmode;
         end;
         %
         % compute matrices for the predictive covariance matrix
         %
         Dbar = eye(n*g)-(DT1*DpKInvKpDDpK*K');
         CovShock = DT1*SigmaTg*DT1';
         CovState = Dbar*G*PTTz*G'*Dbar';
         if rank(R)>0;
            CovME = Dbar*kron(eye(g),R)*Dbar';
         else;
            CovME = 0;
         end;
         Bbar = zeros(r,q*g);
         Fg = eye(r);
         Q = BT1*BT1';
         if ComputePITMoment==1;
            PITMoments(ParamIndex).PredictYCov = zeros(n,n,h);
         end;
         for i=1:g;
            Ind1 = (1+((g-i)*n):(g-i+1)*n);
            Ind2 = (1+(i-1)*q):(i*q);
            StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovState(Ind1,Ind1))))+tril(CovState(Ind1,Ind1),-1)+tril(CovState(Ind1,Ind1),-1)'));
            ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovShock(Ind1,Ind1))))+tril(CovShock(Ind1,Ind1),-1)+tril(CovShock(Ind1,Ind1),-1)'));
            if rank(R)>0;
               MeasureCov(:,:,i) = MeasureCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovME(Ind1,Ind1))))+tril(CovME(Ind1,Ind1),-1)+tril(CovME(Ind1,Ind1),-1)'));
            end;
            Bbar(:,Ind2) = Fg*BT1;
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)');
            Fg = FT1*Fg;
            if ComputePITMoment==1;
               if rank(R)>0;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = CovState(Ind1,Ind1)+CovShock(Ind1,Ind1)+CovME(Ind1,Ind1);
               else;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = CovState(Ind1,Ind1)+CovShock(Ind1,Ind1);
               end;
            end;
         end;
         for i=g+1:h;
            if i==g+1;
               Kbar = ((Bbar*DT1'*K)/(K'*DT1*DT1'*K))*K';
               Gbar = Fg-(Kbar*G);
               Fig = eye(r);
               FjSum = zeros(r,r);
            end;
            FjSum = FjSum+(Fig*Q*Fig');
            Fig = FT1*Fig;
            StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*Fig*Gbar*PTTz*Gbar'*Fig'*H);
            %
            % compute the shock covariance
            %
            ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*(FjSum+(Fig*Bbar*SigmaTg*Bbar'*Fig'))*H);
            if rank(R)>0;
               MeasureCov(:,:,i) = MeasureCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*((H'*Fig*Kbar*kron(eye(g),R)*Kbar'*Fig'*H)+R));
            end;
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)');
            if ComputePITMoment==1;
               if rank(R)>0;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = (H'*Fig*Gbar*PTTz*Gbar'*Fig'*H)+(H'*(FjSum+(Fig*Bbar*SigmaTg*Bbar'*Fig'))*H)+((H'*Fig*Kbar*kron(eye(g),R)*Kbar'*Fig'*H)+R);
               else;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = (H'*Fig*Gbar*PTTz*Gbar'*Fig'*H)+(H'*(FjSum+(Fig*Bbar*SigmaTg*Bbar'*Fig'))*H);
               end;
            end;
         end;
         MeanPredictY = MeanPredictY+((Weights(ParamIndex)/NumDraws)*PredMean);
         if ComputePITMoment==1;
            PITMoments(ParamIndex).PredictY = PredMean;
         end;
         drawnow;
      end;
   else;
      %
      % we have updated beliefs
      %
      for ParamIndex=1:NumDraws;
         if DSGEModel.ShowProgress==1;
            abort = get(WaitHandle,'UserData');
            if strcmp(abort,'cancel')==1;
               StopEstimation = 1;
               break;
            else;
               ProgressDLG([ParamIndex/NumDraws ParamIndex]);
            end;
         end;
         %
         % Solve the model for the current parameter vector
         %
         theta = thetaPostSample(ParamIndex,:)';
         [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         if status~=1;
            %
            % use posterior mode or initial values as a backup
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % get the structural form matrices
         %
         [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
         %
         % Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
         %
         [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
         %
         % call the transformation function
         %
         [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
         %
         % compute initial values for the solution matrices
         %
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
         else;
            [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
         %
         % run the Kalman filter
         %
         if IsALUpdate==1;
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            else;
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         end;
         %
         KsiTT = Ksitt(:,T);
         MT1 = Mtilde(:,:,T);
         FT1 = Ftilde(:,:,T);
         BT1 = Btilde(:,:,T);
         if rank(K1'*H'*BT1)==qm;
            %
            % get the initial conditions for the state variables
            %
            if (DSGEModel.KsiUseCondData==1)&(SCond==1);
               if (k>0);
                  XPred = X(:,1:g);
               else;
                  XPred = [];
               end;
               [KsiTTz,PTTz] = AdaptiveLearningCondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),Yhat(:,T),DSGEModel.Y(:,LastPeriod),XPred,Z,K1,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
               KsiTTz = Ksitt(:,T);
               PTTz = Ptt(:,:,T);
            elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
               %
               % we need to run the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               PtT = PtT(:,:,T-g+1:T);
               KsitT = KsitT(:,T-g+1:T);
               if isempty(wtT)==1;
                  wtT = zeros(n,g);
               else;
                  wtT = wtT(:,T-g+1:T);
               end;
               if k>0;
                  if g>1;
                     Xtra = DSGEModel.X(:,LastPeriod-g+2:LastPeriod);
                  else;
                     Xtra = [];
                  end;
                  [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
               else;
                  [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
               end;
            else;
               KsiTTz = Ksitt(:,T);
               PTTz = Ptt(:,:,T);
            end;
            PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
            if k>0;
               Ax = A'*X;
            else;
               Ax = zeros(n,h);
               X = zeros(r,h)
            end;
            %
            % compute the mean prediction
            %
            MTh = zeros(r,k,h);
            FTh = zeros(r,r,h);
            BTh = zeros(r,q,h);
            MTh(:,:,1) = MT1;
            FTh(:,:,1) = FT1;
            BTh(:,:,1) = BT1;
            PredMean = zeros(n,h);
            KsiT = KsiTTz;
            Fexp = eye(r);
            Xbar = zeros(n*g,1);
            XTi = zeros(r,1);
            G = zeros(n*g,r);
            DT1 = zeros(n*g,q*g);
            etaShocks = zeros(q,h);
            xiStates = zeros(r,h);
            for i=1:g;
               %
               % require new solution matrices
               %
               if i==1;
                  MTh(:,:,i+1) = LastSolution.MTT;
                  FTh(:,:,i+1) = LastSolution.FTT;
                  BTh(:,:,i+1) = LastSolution.BTT;
                  betaTiT = LastSolution.betaTT;
               elseif (i>1)&(i<h);
                  %
                  % we first need to project betaTiT
                  %
                  betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
                  if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                     [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
                  else;
                     [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
                  end;
                  MTh(:,:,i+1) = MTiT;
                  FTh(:,:,i+1) = FTiT;
                  BTh(:,:,i+1) = BTiT;
               end;
               %
               Fexp = FTh(:,:,i)*Fexp;
               Ind = (1+((g-i)*n):(g-i+1)*n);
               Ind1 = (1+((g-i)*q):(g-i+1)*q);
               if k>0;
                  XTi = (MTh(:,:,i)*X(:,i))+(FTh(:,:,i)*XTi);
                  Xbar(Ind,1) = (A'*X(:,i))+(H'*XTi);
               end;
               G(Ind,:) = H'*Fexp;
               DT1(Ind,Ind1) = H'*BTh(:,:,i);
               Fi = FTh(:,:,i);
               for j=1:i-1;
                  Ind1 = Ind1+q;
                  DT1(Ind,Ind1) = H'*Fi*BTh(:,:,i);
                  Fi = Fi*FTh(:,:,i-j);
               end;
            end;
            %
            % speed-up code suggested by Ingvar Strid
            %
            KpD = K'*DT1;
            KpDDpK = KpD*KpD';
            DpKInvKpDDpK = KpD'/KpDDpK;
            %
            % compute the conditioning relation when state vector and measurement errors are at mean
            %
            kbarTg = ZTg-UT-(K'*(Xbar+(G*KsiTTz)));
            %
            % compute the mean of the shocks when state vector and measurement errors are at mean
            %
            mubarTg = DpKInvKpDDpK*kbarTg;
            %
            % compute the idempotent covariance matrix for the shocks
            %
            SigmaTg = eye(q*g)-(DpKInvKpDDpK*KpD);
            SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
            %
            % compute the mean shock values
            %
            etaShocks(:,1:g) = fliplr(reshape(mubarTg,q,g));
            for i=1:h;
               if (i>g)&(i<h);
                  %
                  % we first need to project betaTiT
                  %
                  betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
                  if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                     [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
                  else;
                     [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
                  end;
                  MTh(:,:,i+1) = MTiT;
                  FTh(:,:,i+1) = FTiT;
                  BTh(:,:,i+1) = BTiT;
               end;
               %
               % update the states
               %
               if k>0;
                  KsiT = (MTh(:,:,i)*X(:,i))+(FTh(:,:,i)*KsiT)+(BTh(:,:,i)*etaShocks(:,i));
               else;
                  KsiT = (FTh(:,:,i)*KsiT)+(BTh(:,:,i)*etaShocks(:,i));
               end;
               xiStates(:,i) = KsiT;
               %
               % compute the conditional mean predictions
               %
               if k>0;
                  PredMean(:,i) = (A'*X(:,i))+(H'*KsiT);
               else;
                  PredMean(:,i) = H'*KsiT;
               end;
            end;
            TestValue = max(max(abs((PredMean-PredMeanmode)./PredMeanmode)));
            if TestValue>=100;
               %
               % skip parameters and use the "mode" values instead
               %
               PTTz = PTTzmode;
               R = Rmode;
               H = Hmode;
               PredMean = PredMeanmode;
               G = Gmode;
               DT1 = DT1mode;
               BTh = BThmode;
               MTh = MThmode;
               FTh = FThmode;
               SigmaTg = SigmaTgmode;
               DpKInvKpDDpK = DpKInvKpDDpKmode;
            end;
         else;
            PTTz = PTTzmode;
            R = Rmode;
            H = Hmode;
            PredMean = PredMeanmode;
            G = Gmode;
            DT1 = DT1mode;
            BTh = BThmode;
            MTh = MThmode;
            FTh = FThmode;
            SigmaTg = SigmaTgmode;
            DpKInvKpDDpK = DpKInvKpDDpKmode;
         end;
         %
         % compute matrices for the predictive covariance matrix
         %
         Dbar = eye(n*g)-(DT1*DpKInvKpDDpK*K');
         CovShock = DT1*SigmaTg*DT1';
         CovState = Dbar*G*PTTz*G'*Dbar';
         if rank(R)>0;
            CovME = Dbar*kron(eye(g),R)*Dbar';
         else;
            CovME = 0;
         end;
         Bbar = zeros(r,q*g);
         Fg = eye(r);
         if ComputePITMoment==1;
            PITMoments(ParamIndex).PredictYCov = zeros(n,n,h);
         end;
         for i=1:g;
            Ind1 = (1+((g-i)*n):(g-i+1)*n);
            Ind2 = (1+(i-1)*q):(i*q);
            StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovState(Ind1,Ind1))))+tril(CovState(Ind1,Ind1),-1)+tril(CovState(Ind1,Ind1),-1)'));
            ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovShock(Ind1,Ind1))))+tril(CovShock(Ind1,Ind1),-1)+tril(CovShock(Ind1,Ind1),-1)'));
            if rank(R)>0;
               MeasureCov(:,:,i) = MeasureCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovME(Ind1,Ind1))))+tril(CovME(Ind1,Ind1),-1)+tril(CovME(Ind1,Ind1),-1)'));
            end;
            Bbar(:,Ind2) = Fg*BTh(:,:,i);
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)');
            Fg = FTh(:,:,i)*Fg;
            if ComputePITMoment==1;
               if rank(R)>0;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = CovState(Ind1,Ind1)+CovShock(Ind1,Ind1)+CovME(Ind1,Ind1);
               else;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = CovState(Ind1,Ind1)+CovShock(Ind1,Ind1);
               end;
            end;
         end;
         for i=g+1:h;
            if i==g+1;
               Kbar = ((Bbar*DT1'*K)/(K'*DT1*DT1'*K))*K';
               Gbar = Fg-(Kbar*G);
               Fig = eye(r);
               FjSum = zeros(r,r);
            end;
            Q = BTh(:,:,i)*BTh(:,:,i)';
            FjSum = FjSum+(Fig*Q*Fig');
            Fig = FTh(:,:,i)*Fig;
            StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*Fig*Gbar*PTTz*Gbar'*Fig'*H);
            %
            % compute the shock covariance
            %
            ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*(FjSum+(Fig*Bbar*SigmaTg*Bbar'*Fig'))*H);
            if rank(R)>0;
               MeasureCov(:,:,i) = MeasureCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*((H'*Fig*Kbar*kron(eye(g),R)*Kbar'*Fig'*H)+R));
            end;
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)');
            if ComputePITMoment==1;
               if rank(R)>0;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = (H'*Fig*Gbar*PTTz*Gbar'*Fig'*H)+(H'*(FjSum+(Fig*Bbar*SigmaTg*Bbar'*Fig'))*H)+((H'*Fig*Kbar*kron(eye(g),R)*Kbar'*Fig'*H)+R);
               else;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = (H'*Fig*Gbar*PTTz*Gbar'*Fig'*H)+(H'*(FjSum+(Fig*Bbar*SigmaTg*Bbar'*Fig'))*H);
               end;
            end;
         end;
         MeanPredictY = MeanPredictY+((Weights(ParamIndex)/NumDraws)*PredMean);
         if ComputePITMoment==1;
            PITMoments(ParamIndex).PredictY = PredMean;
         end;
         drawnow;
      end;
   end;
else;
   %
   % we have a time-varying H matrix
   %
   if DSGEModel.FixedBeliefsForProj==1;
      %
      % we have fixed belief coefficients
      %
      for ParamIndex=1:NumDraws;
         if DSGEModel.ShowProgress==1;
            abort = get(WaitHandle,'UserData');
            if strcmp(abort,'cancel')==1;
               StopEstimation = 1;
               break;
            else;
               ProgressDLG([ParamIndex/NumDraws ParamIndex]);
            end;
         end;
         %
         % Solve the model for the current parameter vector
         %
         theta = thetaPostSample(ParamIndex,:)';
         [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         if status~=1;
            %
            % use posterior mode or initial values as a backup
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % get the structural form matrices
         %
         [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
         %
         % Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
         %
         [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
         %
         % call the transformation function
         %
         [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
         %
         % compute initial values for the solution matrices
         %
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
         else;
            [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
         %
         % run the Kalman filter
         %
         if IsALUpdate==1;
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            else;
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         end;
         %
         KsiTT = Ksitt(:,T);
         MT1 = Mtilde(:,:,T);
         FT1 = Ftilde(:,:,T);
         BT1 = Btilde(:,:,T);
         if rank(K1'*H(:,:,LastPeriod+1)'*BT1)==qm;
            %
            % get the initial conditions for the state variables
            %
            if (DSGEModel.KsiUseCondData==1)&(SCond==1);
               if (k>0);
                  XPred = X(:,1:g);
               else;
                  XPred = [];
               end;
               [KsiTTz,PTTz] = AdaptiveLearningCondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),Yhat(:,T),DSGEModel.Y(:,LastPeriod),XPred,Z,K1,A,H(:,:,LastPeriod+1:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
               KsiTTz = Ksitt(:,T);
               PTTz = Ptt(:,:,T);
            elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
               %
               % we need to run the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               PtT = PtT(:,:,T-g+1:T);
               KsitT = KsitT(:,T-g+1:T);
               if isempty(wtT)==1;
                  wtT = zeros(n,g);
               else;
                  wtT = wtT(:,T-g+1:T);
               end;
               if k>0;
                  if g>1;
                     Xtra = DSGEModel.X(:,LastPeriod-g+2:LastPeriod);
                  else;
                     Xtra = [];
                  end;
                  [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H(:,:,LastPeriod-g+2:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
               else;
                  [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H(:,:,LastPeriod-g+2:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
               end;
            else;
               KsiTTz = Ksitt(:,T);
               PTTz = Ptt(:,:,T);
            end;
            PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
            if k>0;
               Ax = A'*X;
            else;
               Ax = zeros(n,h);
               X = zeros(r,h)
            end;
            %
            % compute the mean prediction
            %
            PredMean = zeros(n,h);
            KsiT = KsiTTz;
            Fexp = eye(r);
            G = zeros(n*g,r);
            DT1 = zeros(n*g,q*g);
            Xbar = zeros(n*g,1);
            XTi = zeros(r,1);
            etaShocks = zeros(q,h);
            xiStates = zeros(r,h);
            for i=1:g;
               Fexp = FT1*Fexp;
               Ind = (1+((g-i)*n):(g-i+1)*n);
               Ind1 = (1+((g-i)*q):(g-i+1)*q);
               if k>0;
                  XTi = (MT1*X(:,i))+(FT1*XTi);
                  Xbar(Ind,1) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*XTi);
               end;
               G(Ind,:) = H(:,:,LastPeriod+i)'*Fexp;
               DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*BT1;
               Fi = FT1;
               for j=1:i-1;
                  Ind1 = Ind1+q;
                  DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*Fi*BT1;
                  Fi = Fi*FT1;
               end;
            end;
            %
            % speed-up code suggested by Ingvar Strid
            %
            KpD = K'*DT1;
            KpDDpK = KpD*KpD';
            DpKInvKpDDpK = KpD'/KpDDpK;
            %
            % compute the conditioning relation when state vector and measurement errors are at mean
            %
            kbarTg = ZTg-UT-(K'*(Xbar+(G*KsiTTz)));
            %
            % compute the mean of the shocks when state vector and measurement errors are at mean
            %
            mubarTg = DpKInvKpDDpK*kbarTg;
            %
            % compute the idempotent covariance matrix for the shocks
            %
            SigmaTg = eye(q*g)-(DpKInvKpDDpK*KpD);
            SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
            %
            % compute the mean shock values
            %
            etaShocks(:,1:g) = fliplr(reshape(mubarTg,q,g));
            for i=1:h;
               %
               % update the states
               %
               if k>0;
                  KsiT = (MT1*X(:,i))+(FT1*KsiT)+(BT1*etaShocks(:,i));
               else;
                  KsiT = (FT1*KsiT)+(BT1*etaShocks(:,i));
               end;
               xiStates(:,i) = KsiT;
               %
               % compute the conditional mean predictions
               %
               if k>0;
                  PredMean(:,i) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*KsiT);
               else;
                  PredMean(:,i) = H(:,:,LastPeriod+i)'*KsiT;
               end;
            end;
            TestValue = max(max(abs((PredMean-PredMeanmode)./PredMeanmode)));
            if TestValue>=100;
               %
               % skip parameters and use the "mode" values instead
               %
               H = Hmode;
               R = Rmode;
               BT1 = BT1mode;
               MT1 = MT1mode;
               FT1 = FT1mode;
               PTTz = PTTzmode;
               PredMean = PredMeanmode;
               G = Gmode;
               DT1 = DT1mode;
               SigmaTg = SigmaTgmode;
               DpKInvKpDDpK = DpKInvKpDDpKmode;
            end;
         else;
            H = Hmode;
            R = Rmode;
            BT1 = BT1mode;
            MT1 = MT1mode;
            FT1 = FT1mode;
            PTTz = PTTzmode;
            PredMean = PredMeanmode;
            G = Gmode;
            DT1 = DT1mode;
            SigmaTg = SigmaTgmode;
            DpKInvKpDDpK = DpKInvKpDDpKmode;
         end;
         %
         % compute matrices for the predictive covariance matrix
         %
         Dbar = eye(n*g)-(DT1*DpKInvKpDDpK*K');
         CovShock = DT1*SigmaTg*DT1';
         CovState = Dbar*G*PTTz*G'*Dbar';
         if rank(R)>0;
            CovME = Dbar*kron(eye(g),R)*Dbar';
         else;
            CovME = 0;
         end;
         Bbar = zeros(r,q*g);
         Fg = eye(r);
         Q = BT1*BT1';
         if ComputePITMoment==1;
            PITMoments(ParamIndex).PredictYCov = zeros(n,n,h);
         end;
         for i=1:g;
            Ind1 = (1+((g-i)*n):(g-i+1)*n);
            Ind2 = (1+(i-1)*q):(i*q);
            StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovState(Ind1,Ind1))))+tril(CovState(Ind1,Ind1),-1)+tril(CovState(Ind1,Ind1),-1)'));
            ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovShock(Ind1,Ind1))))+tril(CovShock(Ind1,Ind1),-1)+tril(CovShock(Ind1,Ind1),-1)'));
            if rank(R)>0;
               MeasureCov(:,:,i) = MeasureCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovME(Ind1,Ind1))))+tril(CovME(Ind1,Ind1),-1)+tril(CovME(Ind1,Ind1),-1)'));
            end;
            Bbar(:,Ind2) = Fg*BT1;
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)');
            Fg = FT1*Fg;
            if ComputePITMoment==1;
               if rank(R)>0;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = CovState(Ind1,Ind1)+CovShock(Ind1,Ind1)+CovME(Ind1,Ind1);
               else;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = CovState(Ind1,Ind1)+CovShock(Ind1,Ind1);
               end;
            end;
         end;
         for i=g+1:h;
            if i==g+1;
               Kbar = ((Bbar*DT1'*K)/(K'*DT1*DT1'*K))*K';
               Gbar = Fg-(Kbar*G);
               Fig = eye(r);
               FjSum = zeros(r,r);
            end;
            FjSum = FjSum+(Fig*Q*Fig');
            Fig = FT1*Fig;
            StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*Fig*Gbar*PTTz*Gbar'*Fig'*H(:,:,LastPeriod+i));
            %
            % compute the shock covariance
            %
            ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*(FjSum+(Fig*Bbar*SigmaTg*Bbar'*Fig'))*H(:,:,LastPeriod+i));
            if rank(R)>0;
               MeasureCov(:,:,i) = MeasureCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*((H(:,:,LastPeriod+i)'*Fig*Kbar*kron(eye(g),R)*Kbar'*Fig'*H(:,:,LastPeriod+i))+R));
            end;
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)');
            if ComputePITMoment==1;
               if rank(R)>0;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = (H(:,:,LastPeriod+i)'*Fig*Gbar*PTTz*Gbar'*Fig'*H(:,:,LastPeriod+i))+(H(:,:,LastPeriod+i)'*(FjSum+(Fig*Bbar*SigmaTg*Bbar'*Fig'))*H(:,:,LastPeriod+i))+((H(:,:,LastPeriod+i)'*Fig*Kbar*kron(eye(g),R)*Kbar'*Fig'*H(:,:,LastPeriod+i))+R);
               else;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = (H(:,:,LastPeriod+i)'*Fig*Gbar*PTTz*Gbar'*Fig'*H(:,:,LastPeriod+i))+(H(:,:,LastPeriod+i)'*(FjSum+(Fig*Bbar*SigmaTg*Bbar'*Fig'))*H(:,:,LastPeriod+i));
               end;
            end;
         end;
         MeanPredictY = MeanPredictY+((Weights(ParamIndex)/NumDraws)*PredMean);
         if ComputePITMoment==1;
            PITMoments(ParamIndex).PredictY = PredMean;
         end;
         drawnow;
      end;
   else;
      %
      % we have updated beliefs
      %
      for ParamIndex=1:NumDraws;
         if DSGEModel.ShowProgress==1;
            abort = get(WaitHandle,'UserData');
            if strcmp(abort,'cancel')==1;
               StopEstimation = 1;
               break;
            else;
               ProgressDLG([ParamIndex/NumDraws ParamIndex]);
            end;
         end;
         %
         % Solve the model for the current parameter vector
         %
         theta = thetaPostSample(ParamIndex,:)';
         [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         if status~=1;
            %
            % use posterior mode or initial values as a backup
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % get the structural form matrices
         %
         [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
         %
         % Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
         %
         [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
         %
         % call the transformation function
         %
         [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
         %
         % compute initial values for the solution matrices
         %
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
         else;
            [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
         %
         % run the Kalman filter
         %
         if IsALUpdate==1;
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            else;
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         end;
         %
         KsiTT = Ksitt(:,T);
         MT1 = Mtilde(:,:,T);
         FT1 = Ftilde(:,:,T);
         BT1 = Btilde(:,:,T);
         if rank(K1'*H(:,:,LastPeriod+1)'*BT1)==qm;
            %
            % get the initial conditions for the state variables
            %
            if (DSGEModel.KsiUseCondData==1)&(SCond==1);
               if (k>0);
                  XPred = X(:,1:g);
               else;
                  XPred = [];
               end;
               [KsiTTz,PTTz] = AdaptiveLearningCondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),Yhat(:,T),DSGEModel.Y(:,LastPeriod),XPred,Z,K1,A,H(:,:,LastPeriod+1:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
               KsiTTz = Ksitt(:,T);
               PTTz = Ptt(:,:,T);
            elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
               %
               % we need to run the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               PtT = PtT(:,:,T-g+1:T);
               KsitT = KsitT(:,T-g+1:T);
               if isempty(wtT)==1;
                  wtT = zeros(n,g);
               else;
                  wtT = wtT(:,T-g+1:T);
               end;
               if k>0;
                  if g>1;
                     Xtra = DSGEModel.X(:,LastPeriod-g+2:LastPeriod);
                  else;
                     Xtra = [];
                  end;
                  [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H(:,:,LastPeriod-g+2:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
               else;
                  [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H(:,:,LastPeriod-g+2:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
               end;
            else;
               KsiTTz = Ksitt(:,T);
               PTTz = Ptt(:,:,T);
            end;
            PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
            if k>0;
               Ax = A'*X;
            else;
               Ax = zeros(n,h);
               X = zeros(r,h)
            end;
            %
            % compute the mean prediction
            %
            MTh = zeros(r,k,h);
            FTh = zeros(r,r,h);
            BTh = zeros(r,q,h);
            MTh(:,:,1) = MT1;
            FTh(:,:,1) = FT1;
            BTh(:,:,1) = BT1;
            PredMean = zeros(n,h);
            KsiT = KsiTTz;
            Fexp = eye(r);
            Xbar = zeros(n*g,1);
            XTi = zeros(r,1);
            G = zeros(n*g,r);
            DT1 = zeros(n*g,q*g);
            etaShocks = zeros(q,h);
            xiStates = zeros(r,h);
            for i=1:g;
               %
               % require new solution matrices
               %
               if i==1;
                  MTh(:,:,i+1) = LastSolution.MTT;
                  FTh(:,:,i+1) = LastSolution.FTT;
                  BTh(:,:,i+1) = LastSolution.BTT;
                  betaTiT = LastSolution.betaTT;
               elseif (i>1)&(i<h);
                  %
                  % we first need to project betaTiT
                  %
                  betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
                  if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                     [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
                  else;
                     [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
                  end;
                  MTh(:,:,i+1) = MTiT;
                  FTh(:,:,i+1) = FTiT;
                  BTh(:,:,i+1) = BTiT;
               end;
               %
               Fexp = FTh(:,:,i)*Fexp;
               Ind = (1+((g-i)*n):(g-i+1)*n);
               Ind1 = (1+((g-i)*q):(g-i+1)*q);
               if k>0;
                  XTi = (MTh(:,:,i)*X(:,i))+(FTh(:,:,i)*XTi);
                  Xbar(Ind,1) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*XTi);
               end;
               G(Ind,:) = H(:,:,LastPeriod+i)'*Fexp;
               DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*BTh(:,:,i);
               Fi = FTh(:,:,i);
               for j=1:i-1;
                  Ind1 = Ind1+q;
                  DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*Fi*BTh(:,:,i);
                  Fi = Fi*FTh(:,:,i-j);
               end;
            end;
            %
            % speed-up code suggested by Ingvar Strid
            %
            KpD = K'*DT1;
            KpDDpK = KpD*KpD';
            DpKInvKpDDpK = KpD'/KpDDpK;
            %
            % compute the conditioning relation when state vector and measurement errors are at mean
            %
            kbarTg = ZTg-UT-(K'*(Xbar+(G*KsiTTz)));
            %
            % compute the mean of the shocks when state vector and measurement errors are at mean
            %
            mubarTg = DpKInvKpDDpK*kbarTg;
            %
            % compute the idempotent covariance matrix for the shocks
            %
            SigmaTg = eye(q*g)-(DpKInvKpDDpK*KpD);
            SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
            %
            % compute the mean shock values
            %
            etaShocks(:,1:g) = fliplr(reshape(mubarTg,q,g));
            for i=1:h;
               if (i>g)&(i<h);
                  %
                  % we first need to project betaTiT
                  %
                  betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
                  if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                     [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
                  else;
                     [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
                  end;
                  MTh(:,:,i+1) = MTiT;
                  FTh(:,:,i+1) = FTiT;
                  BTh(:,:,i+1) = BTiT;
               end;
               %
               % update the states
               %
               if k>0;
                  KsiT = (MTh(:,:,i)*X(:,i))+(FTh(:,:,i)*KsiT)+(BTh(:,:,i)*etaShocks(:,i));
               else;
                  KsiT = (FTh(:,:,i)*KsiT)+(BTh(:,:,i)*etaShocks(:,i));
               end;
               xiStates(:,i) = KsiT;
               %
               % compute the conditional mean predictions
               %
               if k>0;
                  PredMean(:,i) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*KsiT);
               else;
                  PredMean(:,i) = H(:,:,LastPeriod+i)'*KsiT;
               end;
            end;
            TestValue = max(max(abs((PredMean-PredMeanmode)./PredMeanmode)));
            if TestValue>=100;
               %
               % skip parameters and use the "mode" values instead
               %
               H = Hmode;
               R = Rmode;
               BTh = BThmode;
               MTh = MThmode;
               FTh = FThmode;
               PTTz = PTTzmode;
               PredMean = PredMeanmode;
               G = Gmode;
               DT1 = DT1mode;
               SigmaTg = SigmaTgmode;
               DpKInvKpDDpK = DpKInvKpDDpKmode;
            end;
         else;
            H = Hmode;
            R = Rmode;
            BTh = BThmode;
            MTh = MThmode;
            FTh = FThmode;
            PTTz = PTTzmode;
            PredMean = PredMeanmode;
            G = Gmode;
            DT1 = DT1mode;
            SigmaTg = SigmaTgmode;
            DpKInvKpDDpK = DpKInvKpDDpKmode;
         end;
         %
         % compute matrices for the predictive covariance matrix
         %
         Dbar = eye(n*g)-(DT1*DpKInvKpDDpK*K');
         CovShock = DT1*SigmaTg*DT1';
         CovState = Dbar*G*PTTz*G'*Dbar';
         if rank(R)>0;
            CovME = Dbar*kron(eye(g),R)*Dbar';
         end;
         Bbar = zeros(r,q*g);
         Fg = eye(r);
         if ComputePITMoment==1;
            PITMoments(ParamIndex).PredictYCov = zeros(n,n,h);
         end;
         for i=1:g;
            Ind1 = (1+((g-i)*n):(g-i+1)*n);
            Ind2 = (1+(i-1)*q):(i*q);
            StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovState(Ind1,Ind1))))+tril(CovState(Ind1,Ind1),-1)+tril(CovState(Ind1,Ind1),-1)'));
            ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovShock(Ind1,Ind1))))+tril(CovShock(Ind1,Ind1),-1)+tril(CovShock(Ind1,Ind1),-1)'));
            if rank(R)>0;
               MeasureCov(:,:,i) = MeasureCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovME(Ind1,Ind1))))+tril(CovME(Ind1,Ind1),-1)+tril(CovME(Ind1,Ind1),-1)'));
            end;
            Bbar(:,Ind2) = Fg*BTh(:,:,i);
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)');
            Fg = FTh(:,:,i)*Fg;
            if ComputePITMoment==1;
               if rank(R)>0;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = CovState(Ind1,Ind1)+CovShock(Ind1,Ind1)+CovME(Ind1,Ind1);
               else;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = CovState(Ind1,Ind1)+CovShock(Ind1,Ind1);
               end;
            end;
         end;
         for i=g+1:h;
            if i==g+1;
               Kbar = ((Bbar*DT1'*K)/(K'*DT1*DT1'*K))*K';
               Gbar = Fg-(Kbar*G);
               Fig = eye(r);
               FjSum = zeros(r,r);
            end;
            Q = BTh(:,:,i)*BTh(:,:,i)';
            FjSum = FjSum+(Fig*Q*Fig');
            Fig = FTh(:,:,i)*Fig;
            StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*Fig*Gbar*PTTz*Gbar'*Fig'*H(:,:,LastPeriod+i));
            %
            % compute the shock covariance
            %
            ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*(FjSum+(Fig*Bbar*SigmaTg*Bbar'*Fig'))*H(:,:,LastPeriod+i));
            if rank(R)>0;
               MeasureCov(:,:,i) = MeasureCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*((H(:,:,LastPeriod+i)'*Fig*Kbar*kron(eye(g),R)*Kbar'*Fig'*H(:,:,LastPeriod+i))+R));
            end;
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)');
            if ComputePITMoment==1;
               if rank(R)>0;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = (H(:,:,LastPeriod+i)'*Fig*Gbar*PTTz*Gbar'*Fig'*H(:,:,LastPeriod+i))+(H(:,:,LastPeriod+i)'*(FjSum+(Fig*Bbar*SigmaTg*Bbar'*Fig'))*H(:,:,LastPeriod+i))+((H(:,:,LastPeriod+i)'*Fig*Kbar*kron(eye(g),R)*Kbar'*Fig'*H(:,:,LastPeriod+i))+R);
               else;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = (H(:,:,LastPeriod+i)'*Fig*Gbar*PTTz*Gbar'*Fig'*H(:,:,LastPeriod+i))+(H(:,:,LastPeriod+i)'*(FjSum+(Fig*Bbar*SigmaTg*Bbar'*Fig'))*H(:,:,LastPeriod+i));
               end;
            end;
         end;
         MeanPredictY = MeanPredictY+((Weights(ParamIndex)/NumDraws)*PredMean);
         if ComputePITMoment==1;
            PITMoments(ParamIndex).PredictY = PredMean;
         end;
         drawnow;
      end;
   end;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
if StopEstimation==1;
   return;
end;
for i=1:h;
   ParameterCov(:,:,i) = ParameterCov(:,:,i)-(MeanPredictY(:,i)*MeanPredictY(:,i)');
   ParameterCov(:,:,i) = diag(abs(diag(ParameterCov(:,:,i))))+tril(ParameterCov(:,:,i),-1)+tril(ParameterCov(:,:,i),-1)';
end;
if IsPosterior==1;
   MatFile = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearningDSGE-CondPredictiveMoments-WaggonerZha-' DSGEModel.NameOfModel '-' SmpStr '-' int2str(CurrChain) '-' int2str(h) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
else;
   MatFile = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearningDSGE-CondPredictiveMoments-WaggonerZha-Prior-' DSGEModel.NameOfModel '-' SmpStr '-' int2str(h) '.' int2str(NumDraws) '.mat']);
end;
save(MatFile,'MeanPredictY','MeasureCov','ShockCov','StateCov','ParameterCov','PITMoments');
DoneCalc = 1;

%
% end of AdaptiveLearningDSGECondPredictiveMomentsWZ.m
%
