function [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,UniModestyStatLZ,BeliefStatus,TransStatus,status] = AdaptiveLearningDSGECondPredictionPathsThetaWZ(theta,thetaPositions,ModelParameters,BeliefSys,DSGEModel,CurrINI,X,Z,U,h,NumPaths,AnnualizeData,TransData,NameStr)
% AdaptiveLearningDSGECondPredictionPathsThetaWZ: Calculates a certain number of possible paths that the endogenous variables
%                                                 can take out-of-sample conditional on a set of assumptions and for a given
%                                                 value of the parameters in the DSGE model subject to adaptive learning. Uses
%                                                 the Waggoner-Zha (1999) method.
%
% USAGE:
%
%       [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,UniModestyStatLZ,BeliefStatus,...
%             TransStatus,status] = AdaptiveLearningDSGECondPredictionPathsThetaWZ(theta,thetaPositions,ModelParameters,...
%                                                                          BeliefSys,DSGEModel,CurrINI,X,Z,U,h,NumPaths,...
%                                                                          AnnualizeData,TransData,NameStr)
%
% REQUIRED INPUT:  theta (vector) with values of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  BeliefSys (structure) with fields "m" an integer denoting the maximum number of lags
%                                 of the state variables in the belief equations; "Name" a string with the
%                                 name of the belief system; and G a cell array with f matrices, one for each
%                                 forward looking variable of dimension (k+r*m) x g_j where g_j is the number
%                                 of explanatory variables in the belief equation for forward looking variable
%                                 j. If empty, the function will compute the structure itself.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  X (matrix) of dimension kxMaxPH with the observations of the exogenous variables for
%                                 the prediction paths.
%
%                  Z (matrix) of dimension mxg with the out-of-sample data we shall condition the
%                                 predictions on.
%
%                  U (matrix) of dimension mxg with initial values for the mapping between the observed
%                                 data and the conditioning data.
%
%                  h (integer) with the prediction horizon.
%
%                  NumPaths (integer)the number of paths to draw.
%
%                  AnnualizeData (boolean) that is 1 if the prediction paths should be annualized and 0 otherwise.
%
%                  TransData (boolean) that is 1 if the prediction paths should be transformed and 0 otherwise.
%
%                  NameStr (string vector) that is sent to the progress dialog name after the string
%                                 "Predictive Distribution".
%
% REQUIRED OUTPUT: PredPaths (n x h x NumPaths) 3D matrix with the NumPaths possible paths for the n
%                                 endogenous variables.
%
%                  PredData (structure) with fields:
%                                 PredMean   - (nxh) matrix with the mean predictions of the observed
%                                              variables.
%                                 etaShocks  - (qxh) matrix with the values of the shocks at the mean
%                                              predictions.
%                                 xiStates   - (rxh) matrix with the values of the states at the mean
%                                              predictions.
%                                 X          - (kxh) matrix with the deterministic variables for the
%                                              prediction horizon.
%                                 etaMean    - (qxh) matrix with the mean values for the economic shocks over
%                                              the forecast period.
%                                 wMean      - (nxh) matrix with the mean values for the measurement errors over
%                                              the forecast period.
%                                 A          - (kxn) matrix with the parameters on the exogenous variables in
%                                              the measurement equation.
%                                 H          - (rxn) matrix with parameters on the state variabels in the
%                                              measurement equation.
%                                 R          - (nxn) covariance matrix for the measurement errors.
%                                 MT1        - (rxk) or (rxkxh) matrix with parameters on the deterministic
%                                              variables in the state equation over the forecast horizon.
%                                              The 3rd dimension appears when beliefs are updated over the
%                                              forecast horizon.
%                                 FT1        - (rxr) or (rxrxh) matrix with parameters on the state variables
%                                              in the state equation over the forecast horizon.
%                                 BT1        - (rxq) or (rxqxh) matrix with parameters on the structural shocks
%                                              in the state equation over the forecast horizon.
%                                 ShockNames - (qxqstr) string matrix with the names of the economic shocks.
%                                 Shocks     - (qxhxNumPaths) matrix with the economic shocks for the paths.
%                                 MeasError  - (nxhxNumPaths) matrix with the measurement errors for the paths.
%                                 HistState  - (rxNumPaths) matrix with the time T period draw of the state
%                                              vector for the paths.
%                                 KernelX    - (3D matrix) of dimension n x 2^8 x h with the x-axis values
%                                              from kernel density estimation of the marginal predictive
%                                              densities.
%                                 KernelY    - (3D matrix) of dimension n x 2^8 x h with the y-axis values
%                                              from kernel density estimation of the marginal predictive
%                                              densities.
%
%                  PredEventData (n x 8) matrix where the first column holds the number of times
%                                 the PredPaths data satisfies the event, the second holds the number
%                                 of times the PredPaths data is greater than the event, the third
%                                 the number of time the data is above the upper bound, while the
%                                 fourth column holds the number of times the event has been tested.
%                                 The fifth and sixth column measure the average deviation from the
%                                 lower bound when below the lower bound and the square of this, while
%                                 the seventh and eighth columns hold similar data for the upper bound
%                                 of the event when the variables are above the upper bound.
%
%                  YObsEventData (n x 8) matrix where the first column hold the number of times
%                                 the PredPaths (adjusted such that the mean is equal to the realized
%                                 value) satisfies the event, the second holds the number of times
%                                 the adjusted paths data is greater than the event, the third
%                                 the number of time the data is above the upper bound, while the
%                                 fourth column holds the number of times the event has been tested.
%                                 The fifth and sixth column measure the average deviation from the
%                                 lower bound when below the lower bound and the square of this, while
%                                 the seventh and eighth columns hold similar data for the upper bound
%                                 of the event when the variables are above the upper bound.
%
%                  MultiModestyStat (matrix) with NumPaths x 2 elements. The first column is the the multivariate
%                                 modesty statistic based on the conditioning consistent shocks and the second
%                                 is the value of this statistic for randomly drawn shocks.
%
%                  UniModestyStat (matrix) with NumPaths x n elements. Each column gives the univariate modesty
%                                 for a particular observed variable.
%
%                  UniModestyStatLZ (vector) with the values of the univariate modesty statistics suggested
%                                 by Leeper-Zha (other shocks are zero over examined prediction horizon) for
%                                 all observed variables.
%
% OPTIONAL OUTPUT: BeliefStatus (boolean) which is 1 if the Kalman filter of the belief system runs without
%                        error, and 0 otherwise.
%
%                  TransStatus (boolean) which takes the value 1 if the rewrite of the structural form of
%                        the DSGE model was successful and 0 otherwise.
%
%                  status (integer) Indicates whether the solution of the model is unique or not.
%                                 mcode is 1 when unique, 0 if not computed, and various other values when
%                                 there are problems with the solution. Specifically:
%
%                                      mcode = 2:      Roots not correctly computed by real_schur
%                                      mcode = 3:      Too many big roots
%                                      mcode = 35:     Too many big roots, and q(:,right) is singular
%                                      mcode = 4:      Too few big roots
%                                      mcode = 45:     Too few big roots, and q(:,right) is singular
%                                      mcode = 5:      q(:,right) is singular
%                                      mcode = 61:     Too many exact shiftrights
%                                      mcode = 62:     Too many numeric shiftrights
%                                      mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                      cannot be calculated.
%                                      mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                      numbers.
%                                      mcode = -1:     No stable solution.
%                                      mcode = -2:     Too many large generalized eigenvalues.
%                                      mcode = -3:     Too few large generalized eigenvalues.
%                                      else            Return code not properly specified
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 19, 2024.
%                        This version: April 23, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 22-04-2024: Updated the documentation.
%
% * 23-04-2024: Took into account that the function should provide the output variable "status".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
PredPaths = [];
PredData = [];
PredEventData = [];
YObsEventData = [];
MultiModestyStat = [];
UniModestyStat = [];
UniModestyStatLZ = [];
if nargout>7;
   BeliefStatus = 1;
end;
if nargout>8;
   TransStatus = 1;
end;
%
if nargout<=9;
   %
   % Make sure that the ModelParameter structure takes the correct values.
   % This is not necessary to do when initial values are given to the function.
   % In that case, more substantial checking is required!
   %
   ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
   %
   % copy files to the tmp folder
   %
   [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
   [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.SystemPriorFile)==1;
      [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   end;
   %
   % check if we should copy model solver file
   %
   if DSGEModel.ModelSolver==4;
      if FileExist(DSGEModel.ModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
      end;
      if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
      end;
   end;
   %
   % Update any parameteters that need updating
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
   end;
else;
   status = 1;
end;
%
% check if we should also copy the belief system file
%
if DSGEModel.BeliefSystem==2;
   [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if nargout>9;
      status = 0;
   end;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% See if we can solve the model
%
if nargout<=3;
   if DSGEModel.ModelSolver==1;
      [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   elseif DSGEModel.ModelSolver==2;
      [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==3;
      [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==4;
      [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
   end;
else;
   if MatlabNumber>=8.0;
      try;
         if DSGEModel.ModelSolver==1;
            [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
         elseif DSGEModel.ModelSolver==2;
            [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==3;
            [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==4;
            [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      catch ME;
         if nargout>9;
            if exist('mcode')>0;
               status = mcode;
            else;
               status = 0;
            end;
         end;
         if DSGEModel.ModelSolver==1;
            ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==2;
            ErrorStr = ['There was an error running the Klein solver. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==3;
            ErrorStr = ['There was an error running Gensys. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==4;
            ErrorStr = ['There was an error running own DSGE model solver. YADA received the following message: ' YADALastError(ME)];
         end;
         About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
         return;
      end;
   else;
      try;
         if DSGEModel.ModelSolver==1;
            [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
         elseif DSGEModel.ModelSolver==2;
            [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==3;
            [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==4;
            [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      catch;
         if nargout>9;
            if exist('mcode')>0;
               status = mcode;
            else;
               status = 0;
            end;
         end;
         if DSGEModel.ModelSolver==1;
            ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==2;
            ErrorStr = ['There was an error running the Klein solver. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==3;
            ErrorStr = ['There was an error running Gensys. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==4;
            ErrorStr = ['There was an error running own DSGE model solver. YADA received the following message: ' YADALastError];
         end;
         About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
         return;
      end;
   end;
end;
if nargout>9;
   status = mcode;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
   return;
end;
%
% Convert to the state equation matrices
%
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
[B0Alt,KeepVar] = RemoveRedundantColumns(B0);
ModelParameters.YADA.F = F;
ModelParameters.YADA.B0 = B0Alt;
%
% obtain the measurement equation matrices
%
if MatlabNumber>=8.0;
   try;
      [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
   catch ME;
      %
      % display an error message
      %
      ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError(ME)];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
else;
   try;
      [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
   catch;
      %
      % display an error message
      %
      ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
if length(size(H))==3;
   if size(H,3)<size(DSGEModel.Y,2);
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations (T = ' int2str(size(DSGEModel.Y,2)) '). YADA has therefore aborted from the calculations.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
StateVarStructure.nonsingular = 1;
if DSGEModel.n>(rank(R)+size(B0Alt,2));
   StateVarStructure.nonsingular = 0;
end;
%
% get the structural form matrices
%
[Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
%
% Determine the belief system matrices
%
if isempty(BeliefSys)==1;
   if DSGEModel.BeliefSystem==1;
      BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
   else;
      BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
   end;
end;
%
% Compute the transformation of the structural from that we use when solving the model
%
r = length(DSGEModel.StateVariablePositions);
f = length(DSGEModel.ForwardVariablePositions);
SMatrix = eye(r);
Sperp = SMatrix(:,setdiff((1:r),DSGEModel.ForwardVariablePositions));
SMatrix = SMatrix(:,DSGEModel.ForwardVariablePositions);
%
% call the function: 
%
[tstatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
if nargout>5;
   TransStatus = tstatus;
   if TransStatus==0;
      return;
   end;
end;
%
% get estimation sample
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%
% Determine the first observation to use when computing log-likelihood
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
%
% Setup all the basic data matrix for the deterministic variables
% used for initializing Sigmax
%
if DSGEModel.k>0
   x = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
else;
   x = [];
end;
%
% Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
%
[betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,BeliefSys.m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue);
%
% check if we have a training sample or not
%
if KalmanFirstObservation>1;
   StateVarStructure.TrainSample = 1;
else;
   StateVarStructure.TrainSample = 0;
end;
%
% simplify integers
%
m = BeliefSys.m;
k = DSGEModel.k;
q = size(B0,2);
n = DSGEModel.n;
%
% call the joint Kalman filter for state variables, belief coefficients and
% solution matrices
%
ConstPos = 0;
if k>0;
   ConstPos = LocateStr(DSGEModel.XVariableNames,'const');
   if ConstPos==0;
      %
      % we will now check if X has a unit vector
      %
      Xhist = DSGEModel.X(:,FirstPeriod:LastPeriod);
      T = size(Xhist,2);
      for j=1:k;
         ConstTest = (Xhist==ones(1,T));
         if sum(ConstTest)==T;
            ConstPos = j;
            break;
         end;
      end;
   end;
   xbar = zeros(k,1);
   xbar(ConstPos) = 1;
else;
   xbar = [];
end;
%
% compute initial values for the solution matrices
%
if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
   [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
else;
   [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
%
% check if the adaptive learning parameters allows for updating of the beliefs
%
IsALUpdate = (CheckAdaptiveLearningFixedBeliefs(ModelParameters)==0);
%
% compute the log-likelihood, forecast and update estimates of states, belief coefficients
% and solution matrices
%
if MatlabNumber>=8.0;
   try;
      if IsALUpdate==1;
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         else;
            [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      else;
         [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
      end;
   catch ME;
      txt = ['YADA caught the following error message from the Kalman filter routine under adaptive learning: ' YADALastError(ME)];
      About(txt,'error','Error - Kalman Filter',200,500,CurrINI);
      return;
   end;
else;
   try;
      if IsALUpdate==1;
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         else;
            [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      else;
         [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
      end;
   catch;
      txt = ['YADA caught the following error message from the Kalman filter routine under adaptive learning: ' YADALastError];
      About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
      return;
   end;
end;
if nargout>4;
   BeliefStatus = bstatus;
   if BeliefStatus==0;
      return;
   end;
end;
%
% get some parameter values
%
[r,T] = size(Ksitt1);
[qm,g] = size(Z);
%
% create the mapping matrices for conditioning variables
%
K1 = DSGEModel.K1(:,DSGEModel.ConditionalVariables);
if length(size(DSGEModel.K2))==2;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables);
   K2length = 2;
else;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables,:);
   K2length = 3;
end;
SCond = (isequal(U,zeros(size(U)))&isequal(K2,zeros(size(K2))));
if (SCond==1)&(K2length==3);
   K2 = K2(:,:,1);
   K2length = 2;
end;
%
% check if the user wants to take the conditioning infortmation into account
% when determining the mean and covariance matrix for the states at T.
%
% NOTE: to be completed
%
if (DSGEModel.KsiUseCondData==1)&(SCond==1);
   if (k>0);
      XPred = X(:,1:g);
   else;
      XPred = [];
   end;
   if length(size(H))==2;
      [KsiTTz,PTTz] = AdaptiveLearningCondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),Yhat(:,T),DSGEModel.Y(:,LastPeriod),XPred,Z,K1,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
   else;
      [KsiTTz,PTTz] = AdaptiveLearningCondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),Yhat(:,T),DSGEModel.Y(:,LastPeriod),XPred,Z,K1,A,H(:,:,LastPeriod+1:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
   end;
   KsiTTz = Ksitt(:,T);
   PTTz = Ptt(:,:,T);
elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
   %
   % we need to run the Kalman smoother
   %
   [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
   PtT = PtT(:,:,T-g+1:T);
   KsitT = KsitT(:,T-g+1:T);
   if isempty(wtT)==1;
      wtT = zeros(n,g);
   else;
      wtT = wtT(:,T-g+1:T);
   end;
   if length(size(H))==2;
      if k>0;
         if g>1;
            Xtra = DSGEModel.X(:,LastPeriod-g+2:LastPeriod);
         else;
            Xtra = [];
         end;
         [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
      else;
         [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
      end;
   else;
      if k>0;
         if g>1;
            Xtra = DSGEModel.X(:,LastPeriod-g+2:LastPeriod);
         else;
            Xtra = [];
         end;
         [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H(:,:,LastPeriod-g+2:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
      else;
         [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H(:,:,LastPeriod-g+2:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
      end;
   end;
else;
   KsiTTz = Ksitt(:,T);
   PTTz = Ptt(:,:,T);
end;
PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
%
% Now we need to determine KsiTT for just the last period
%
KsiTT = Ksitt(:,T);
%
% use the Schur decomposition for the reduced rank matrix PTTz instead of
% the Choleski decomposition
%
[S,L] = SchurDecomposition(PTTz);
%
% set up the output
%
PredPaths = zeros(n,h,NumPaths);
LZPredPaths = zeros(n,g);
%
% check if we have measurement errors and if so compute a Schur decomposition of the R matrix
%
if rank(R)>0;
   [Rm,Lm] = SchurDecomposition(R);
   Rmeasure = Rm*sqrt(Lm);
   NumMeasure = size(Lm,1);
else;
   Rmeasure = [];
   wTh = zeros(n,h);
   NumMeasure = 0;
end;
%
% check if shocks can be used
%
if length(size(H))==2;
   if rank(K1'*H'*Btilde(:,:,T))<qm;
      txt = ['The conditioning assumptions you have selected cannot be used due to a singularity in the solution algorithm. Estimation of the conditional predictive distribution' lower(NameStr) ' has therfore been aborted.'];
      About(txt,'error','Error - Conditional Shocks',150,600,CurrINI);
      PredPaths = [];
      return;
   end;
else;
   BadEtaMat = 0;
   for i=1:g;
      if rank(K1'*H(:,:,DSGEModel.LastPeriod+i)'*Btilde(:,:,T))<qm;
         txt = ['The conditioning assumptions you have selected cannot be used due to a singularity in the solution algorithm. Estimation of the conditional predictive distribution' lower(NameStr) ' has therfore been aborted.'];
         About(txt,'error','Error - Conditional Shocks',150,600,CurrINI);
         PredPaths = [];
         BadEtaMat = 1;
         break;
      end;
   end;
   if BadEtaMat==1;
      return;
   end;
end;
%
% setup matrices for the solutions
%
if DSGEModel.FixedBeliefsForProj==1;
   MT1 = Mtilde(:,:,T);
   FT1 = Ftilde(:,:,T);
   BT1 = Btilde(:,:,T);
else;
   MT1 = zeros(r,k,h);
   FT1 = zeros(r,r,h);
   BT1 = zeros(r,q,h);
   MT1(:,:,1) = Mtilde(:,:,T);
   FT1(:,:,1) = Ftilde(:,:,T);
   BT1(:,:,1) = Btilde(:,:,T);
end;
%
% stack the model to determine the shock distribution
%
Xbar = zeros(n*g,1);
XTi = zeros(r,1);
G = zeros(n*g,r);
WTg = zeros(n*g,1);
K = zeros(n*g,qm*g);
DT1 = zeros(n*g,q*g);
ZTg = zeros(qm*g,1);
UT = zeros(qm*g,1);
Fexp = eye(r);
if length(size(H))==2;
   if DSGEModel.FixedBeliefsForProj==1;
      for i=1:g;
         Fexp = FT1*Fexp;
         Ind = (1+((g-i)*n):(g-i+1)*n);
         Ind1 = (1+((g-i)*q):(g-i+1)*q);
         Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
         if k>0;
            XTi = (MT1*X(:,i))+(FT1*XTi);
            Xbar(Ind,1) = (A'*X(:,i))+(H'*XTi);
         end;
         G(Ind,:) = H'*Fexp;
         DT1(Ind,Ind1) = H'*BT1;
         K(Ind,Ind2) = K1;
         Fi = FT1;
         for j=1:i-1;
            Ind1 = Ind1+q;
            Ind3 = Ind+(j*n);
            DT1(Ind,Ind1) = H'*Fi*BT1;
            if K2length==2;
               K(Ind3,Ind2) = K2;
            else;
               if j<=size(K2,3);
                  K(Ind3,Ind2) = K2(:,:,j);
               end;
            end;
            Fi = Fi*FT1;
         end;
         ZTg(Ind2,1) = Z(:,i);
         UT(Ind2,1) = U(:,g);
      end;
   else;
      %
      % we have updated beliefs
      %
      for i=1:g;
         Fexp = FT1(:,:,i)*Fexp;
         Ind = (1+((g-i)*n):(g-i+1)*n);
         Ind1 = (1+((g-i)*q):(g-i+1)*q);
         Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
         if k>0;
            XTi = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*XTi);
            Xbar(Ind,1) = (A'*X(:,i))+(H'*XTi);
         end;
         G(Ind,:) = H'*Fexp;
         DT1(Ind,Ind1) = H'*BT1(:,:,i);
         K(Ind,Ind2) = K1;
         Fi = FT1(:,:,i);
         for j=1:i-1;
            Ind1 = Ind1+q;
            Ind3 = Ind+(j*n);
            DT1(Ind,Ind1) = H'*Fi*BT1(:,:,i);
            if K2length==2;
               K(Ind3,Ind2) = K2;
            else;
               if j<=size(K2,3);
                  K(Ind3,Ind2) = K2(:,:,j);
               end;
            end;
            Fi = Fi*FT1(:,:,i-j);
         end;
         ZTg(Ind2,1) = Z(:,i);
         UT(Ind2,1) = U(:,g);
         %
         % require new solution matrices
         %
         if i==1;
            MT1(:,:,i+1) = LastSolution.MTT;
            FT1(:,:,i+1) = LastSolution.FTT;
            BT1(:,:,i+1) = LastSolution.BTT;
            betaTiT = LastSolution.betaTT;
         elseif (i>1)&(i<g);
            %
            % we first need to project betaTiT
            %
            betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
            else;
               [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
            MT1(:,:,i+1) = MTiT;
            FT1(:,:,i+1) = FTiT;
            BT1(:,:,i+1) = BTiT;
         end;
      end;
   end;
else;
   if DSGEModel.FixedBeliefsForProj==1;
      for i=1:g;
         Fexp = FT1*Fexp;
         Ind = (1+((g-i)*n):(g-i+1)*n);
         Ind1 = (1+((g-i)*q):(g-i+1)*q);
         Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
         if k>0;
            XTi = (MT1*X(:,i))+(FT1*XTi);
            Xbar(Ind,1) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*XTi);
         end;
         G(Ind,:) = H(:,:,LastPeriod+i)'*Fexp;
         DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*BT1;
         K(Ind,Ind2) = K1;
         Fi = FT1;
         for j=1:i-1;
            Ind1 = Ind1+q;
            Ind3 = Ind+(j*n);
            DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*Fi*BT1;
            if K2length==2;
               K(Ind3,Ind2) = K2;
            else;
               if j<=size(K2,3);
                  K(Ind3,Ind2) = K2(:,:,j);
               end;
            end;
            Fi = Fi*FT1;
         end;
         ZTg(Ind2,1) = Z(:,i);
         UT(Ind2,1) = U(:,g);
      end;
   else;
      %
      % we have updated beliefs
      %
      for i=1:g;
         Fexp = FT1(:,:,i)*Fexp;
         Ind = (1+((g-i)*n):(g-i+1)*n);
         Ind1 = (1+((g-i)*q):(g-i+1)*q);
         Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
         if k>0;
            XTi = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*XTi);
            Xbar(Ind,1) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*XTi);
         end;
         G(Ind,:) = H(:,:,LastPeriod+i)'*Fexp;
         DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*BT1(:,:,i);
         K(Ind,Ind2) = K1;
         Fi = FT1(:,:,i);
         for j=1:i-1;
            Ind1 = Ind1+q;
            Ind3 = Ind+(j*n);
            DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*Fi*BT1(:,:,i);
            if K2length==2;
               K(Ind3,Ind2) = K2;
            else;
               if j<=size(K2,3);
                  K(Ind3,Ind2) = K2(:,:,j);
               end;
            end;
            Fi = Fi*FT1(:,:,i-j);
         end;
         ZTg(Ind2,1) = Z(:,i);
         UT(Ind2,1) = U(:,g);
         %
         % require new solution matrices
         %
         if i==1;
            MT1(:,:,i+1) = LastSolution.MTT;
            FT1(:,:,i+1) = LastSolution.FTT;
            BT1(:,:,i+1) = LastSolution.BTT;
            betaTiT = LastSolution.betaTT;
         elseif (i>1)&(i<h);
            %
            % we first need to project betaTiT
            %
            betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
            else;
               [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
            MT1(:,:,i+1) = MTiT;
            FT1(:,:,i+1) = FTiT;
            BT1(:,:,i+1) = BTiT;
         end;
      end;
   end;
end;
%
% create a vector with positions for the conditioning assumptions that should be used
%
PosVec = (1:qm*g)' .* ~isnan(ZTg);
PosVec = PosVec(PosVec>0);
%
% adjust for missing restrictions
%
K = K(:,PosVec);
ZTg = ZTg(PosVec,1);
UT = UT(PosVec,1);
%
% speed-up code suggested by Ingvar Strid
%
KpD = K'*DT1;
KpDDpK = KpD*KpD';
DpKInvKpDDpK = KpD'/KpDDpK;
%
% check the rank of K'D
%
if rank(KpD)<length(PosVec);
   txt = ['Some of the conditioning assumptions are linearly dependent. The rank of K''*D is ' int2str(rank(KpD)) ', while the number of conditioning assumptions is ' int2str(length(PosVec)) '. YADA is therefore unable to perform the conditional forecasts for the Waggoner-Zha approach.'];
   About(txt,'error','Error - Conditioning Assumptions',200,500,CurrINI);
   PredPaths = [];
   return;
end;
%
% compute the conditioning relation when state vector and measurement errors are at mean
%
kbarTg = ZTg-UT-(K'*(Xbar+(G*KsiTTz)));
%
% compute the mean of the shocks when state vector and measurement errors are at mean
%
mubarTg = DpKInvKpDDpK*kbarTg;
%
% compute the idempotent covariance matrix for the shocks
%
SigmaTg = eye(q*g)-(DpKInvKpDDpK*KpD);
SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
[SE,LE] = schur(SigmaTg);
SigmaTgStd = SE*sqrt(abs(LE));
if DSGEModel.YNaNs~=0;
   %
   % if there are missing observations over the historical sample,
   % replace these with best predictions.
   %
   YSmooth = DSGEModel.Y(:,FirstPeriod:LastPeriod);
   nanpos = isnan(YSmooth);
   if max(max(nanpos))==1;
      if MatlabNumber>=8.0;
         try;
            [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,1,DSGEModel.YNaNs);
         catch ME;
            txt = ['YADA caught the following error message from the Kalman smoother under adaptive learning: ' YADALastError(ME)];
            About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
            return;
         end;
      else;
         try;
            [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,1,DSGEModel.YNaNs);
         catch;
            txt = ['YADA caught the following error message from the Kalman smoother under adaptive learning: ' YADALastError];
            About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
            return;
         end;
      end;
      if k>0;
         if length(size(H))==2;
            YHistPred = (A'*DSGEModel.X(:,FirstPeriod:LastPeriod))+(H'*KsitT);
         else;
            YHistPred = A'*DSGEModel.X(:,FirstPeriod:LastPeriod);
            for t=1:size(KsitT,2);
               YHistPred(:,t) = YHistPred(:,t)+(H(:,:,FirstPeriod+t-1)'*KsitT(:,t));
            end;
         end;
      else;
         if length(size(H))==2;
            YHistPred = H'*KsitT;
         else;
            YHistPred = zeros(n,size(KsitT,2));
            for t=1:size(KsitT,2);
               YHistPred(:,t) = H(:,:,FirstPeriod+t-1)'*KsitT(:,t);
            end;
         end;
      end;
      YSmooth(nanpos) = YHistPred(nanpos);
   end;
end;
%
% some initializations
%
if (DSGEModel.RunPredictionEvent==1)&(size(DSGEModel.Y,2)>=LastPeriod+h);
   YObserved = DSGEModel.Y(:,LastPeriod+1:LastPeriod+h);
else;
   YObserved = [];
end;
Yinit = [];
if DSGEModel.YNaNs==0;
   if AnnualizeData==1;
      if strcmp(DSGEModel.DataFrequency,'q')==1;
         Yinit = DSGEModel.Y(:,LastPeriod-2:LastPeriod);
      elseif strcmp(DSGEModel.DataFrequency,'m')==1;
         Yinit = DSGEModel.Y(:,LastPeriod-10:LastPeriod);
      else;
         Yinit = [];
      end;
   elseif TransData==1;
      Yinit = DSGEModel.Y(:,1:LastPeriod);
   end;
else;
   if FirstPeriod>1;
      YSmooth = [DSGEModel.Y(:,1:FirstPeriod-1) YSmooth];
   end;
   if AnnualizeData==1;
      if strcmp(DSGEModel.DataFrequency,'q')==1;
         Yinit = YSmooth(:,LastPeriod-2:LastPeriod);
      elseif strcmp(DSGEModel.DataFrequency,'m')==1;
         Yinit = YSmooth(:,LastPeriod-10:LastPeriod);
      else;
         Yinit = [];
      end;
   elseif TransData==1;
      Yinit = YSmooth(:,1:LastPeriod);
   end;
end;
if (AnnualizeData==0)&(TransData==0);
   MultiModestyStat = zeros(NumPaths,2);
   UniModestyStat = zeros(NumPaths,n);
   UniModestyStatLZ = zeros(1,n);
end;
%
% add info to the PredData structure
%
PredData.KernelX = [];
PredData.KernelY = [];
PredData.X = X;
PredData.A = A;
PredData.H = H;
PredData.R = R;
PredData.etaMean = zeros(q,h);
PredData.wMean = zeros(n,h);
PredData.ShockNames = DSGEModel.StateShockNames;
PredData.Shocks = zeros(q,h,NumPaths);
PredData.MeasError = zeros(n,h,NumPaths);
PredData.HistState = zeros(r,NumPaths);
PredData.KsiT = KsiTTz;
PredData.KsiMean = zeros(r,1);
%
PredData.ShockNames = DSGEModel.StateShockNames;
PredData.Shocks = zeros(q,h,NumPaths);
PredData.MeasError = zeros(n,h,NumPaths);
PredData.HistState = zeros(r,NumPaths);
%
% compute the mean prediction
%
PredData.PredMean = zeros(n,h);
PredData.MeanPredPath = zeros(n,h);
KsiT = KsiTTz;
PredData.etaShocks = zeros(q,h);
PredData.etaShocks(:,1:g) = fliplr(reshape(mubarTg,q,g));
PredData.xiStates = zeros(r,h);
ZPos = ~isnan(Z);
if length(size(H))==2;
   if DSGEModel.FixedBeliefsForProj==1;
      for i=1:h;
         %
         % update the states
         %
         if k>0;
            KsiT = (MT1*X(:,i))+(FT1*KsiT)+(BT1*PredData.etaShocks(:,i));
         else;
            KsiT = (FT1*KsiT)+(BT1*PredData.etaShocks(:,i));
         end;
         PredData.xiStates(:,i) = KsiT;
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredData.PredMean(:,i) = (A'*X(:,i))+(H'*KsiT);
         else;
            PredData.PredMean(:,i) = H'*KsiT;
         end;
      end;
   else;
      %
      % we have updated beliefs
      %
      for i=1:h;
         %
         % update the states
         %
         if k>0;
            KsiT = (MT1*X(:,i))+(FT1*KsiT)+(BT1*PredData.etaShocks(:,i));
         else;
            KsiT = (FT1*KsiT)+(BT1*PredData.etaShocks(:,i));
         end;
         PredData.xiStates(:,i) = KsiT;
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredData.PredMean(:,i) = (A'*X(:,i))+(H'*KsiT);
         else;
            PredData.PredMean(:,i) = H'*KsiT;
         end;
         %
         % update the beliesfs and solution if i>g and i<h
         %
         if (i>g)&(i<h);
            betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
            else;
               [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
            MT1(:,:,i+1) = MTiT;
            FT1(:,:,i+1) = FTiT;
            BT1(:,:,i+1) = BTiT;
         end;
      end;
   end;
else;
   if DSGEModel.FixedBeliefsForProj==1;
      for i=1:h;
         %
         % update the states
         %
         if k>0;
            KsiT = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*KsiT)+(BT1(:,:,i)*PredData.etaShocks(:,i));
         else;
            KsiT = (FT1(:,:,i)*KsiT)+(BT1(:,:,i)*PredData.etaShocks(:,i));
         end;
         PredData.xiStates(:,i) = KsiT;
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredData.PredMean(:,i) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*KsiT);
         else;
            PredData.PredMean(:,i) = H(:,:,LastPeriod+i)'*KsiT;
         end;
      end;
   else;
      %
      % we have updated beliefs
      %
      for i=1:h;
         %
         % update the states
         %
         if k>0;
            KsiT = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*KsiT)+(BT1(:,:,i)*PredData.etaShocks(:,i));
         else;
            KsiT = (FT1(:,:,i)*KsiT)+(BT1(:,:,i)*PredData.etaShocks(:,i));
         end;
         PredData.xiStates(:,i) = KsiT;
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredData.PredMean(:,i) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*KsiT);
         else;
            PredData.PredMean(:,i) = H(:,:,LastPeriod+i)'*KsiT;
         end;
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredData.PredMean(:,i) = (A'*X(:,i))+(H'*KsiT);
         else;
            PredData.PredMean(:,i) = H'*KsiT;
         end;
         %
         % update the beliesfs and solution if i>g and i<h
         %
         if (i>g)&(i<h);
            betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
            else;
               [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
            MT1(:,:,i+1) = MTiT;
            FT1(:,:,i+1) = FTiT;
            BT1(:,:,i+1) = BTiT;
         end;
      end;
   end;
end;
PredData.MT1 = MT1;
PredData.FT1 = FT1;
PredData.BT1 = BT1;
%
% annualize the mean predictions when needed
%
Tt = [];
if AnnualizeData==1;
   PredData.PredMean = Annualize([Yinit PredData.PredMean],DSGEModel.annual,DSGEModel.annualscale);
   if isempty(YObserved)==0;
      YObserved = Annualize([Yinit YObserved],DSGEModel.annual,DSGEModel.annualscale);;
   end;
elseif TransData==1;
   YTmp = [Yinit PredData.PredMean];
   [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
   Tt = size(YData,1);
   PredData.PredMean = YData(Tt-h+1:Tt,:)';
   if isempty(YObserved)==0;
      YObserved = [Yinit YObserved];
      [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YObserved',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
      Tt = size(YData,1);
      YObserved = YData(Tt-h+1:Tt,:)';
   end;
end;
%
% check if we should run parallel computations
%
if CurrINI.NumWorkers<2;
   %
   % check if we should setup a wait handle
   %
   if DSGEModel.ShowProgress==1;
      ProgressStructure.title = ['Progress for ' int2str(NumPaths) ' Prediction Paths'];
      ProgressStructure.facecolor = CurrINI.progress_facecolor;
      ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
      ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
      ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
      ProgressStructure.stop = 0;
      ProgressStructure.clock = DSGEModel.ShowProgressClock;
      ProgressStructure.label = 'Path Number:';
      %
      ProgressStructure.name = ['Adaptive Learning: Pred. Dist.' NameStr];
      ProgressStructure.CurrINI = CurrINI;
      WaitHandle = ProgressDLG(0,ProgressStructure);
      set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
      drawnow;
   elseif (DSGEModel.ShowProgress==0)&(TransData==1);
      txt = ['Please wait while YADA computes predictive distributions' lower(NameStr) ' for the transformed data. Computations started at: ' StringTrim(datestr(now,14))];
      WaitHandle = WaitDLG(txt,'information','Adaptive Learning: Pred. Dist. - Transformed Data',600,150,CurrINI,0);
      WaitControls = get(WaitHandle,'UserData');
   end;
   %
   % start the iteration
   %
   abort = '';
   if length(size(H))==2;
      if DSGEModel.FixedBeliefsForProj==1;
         %
         % we have fixed belief coefficients
         %
         Q = BT1*BT1';
         CovMatShocks = Q*H*K1*((K1'*H'*Q*H*K1)\K1')*H'*Q;
         %
         % additional speed-up code suggested by Ingvar Strid
         %
         PTTg = zeros(r,r);
         Fg = eye(r);
         for i=1:g
            Fg = FT1*Fg;
            PTTg = (FT1*PTTg*FT1')+Q;
         end;
         %
         for it=1:NumPaths;
            %
            % check if we should update the progress dialog
            %
            if DSGEModel.ShowProgress==1;
               abort = get(WaitHandle,'UserData');
               if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
                  PredPaths = [];
                  break;
               else;
                  ProgressDLG([it/NumPaths it]);
               end;
            end;
            %
            % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
            %
            KsiT = KsiTTz+(S*sqrt(L)*randn(size(L,2),1));
            PredData.HistState(:,it) = KsiT;
            PredData.KsiMean = PredData.KsiMean+((1/NumPaths)*KsiT);
            KsiCTmp = KsiTT;
            KsiTmp = KsiTT;
            PTTgLZ = zeros(r,r);
            %
            % Draw a sequence of measurement errors
            %
            if isempty(Rmeasure)==0;
               wTh = Rmeasure*randn(NumMeasure,h);
               PredData.MeasError(:,:,it) = wTh;
               PredData.wMean = PredData.wMean+((1/NumPaths)*wTh);
               WTg = vec(fliplr(wTh(:,1:g)));
            end;
            %
            % setut the mean for the shock distribution
            %
            kTg = ZTg-UT-(K'*(Xbar+(G*KsiT)+WTg));
            %
            % use speed-up code
            %
            muTg = DpKInvKpDDpK*kTg;
            kCTmp = ZTg-UT-(K'*(Xbar+(G*KsiCTmp)+WTg));
            %
            % use speed-up code
            %
            muCTmp = DpKInvKpDDpK*kCTmp;
            %
            % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
            % and from N(0,I) for period g+1 to h.
            %
            ShockDraw = randn(q*g,1);
            etaTg = muTg+(SigmaTgStd*ShockDraw);
            etaCTmpg = muCTmp+(SigmaTgStd*ShockDraw);
            etaTh = randn(q,h);
            etaCTmp = etaTh;
            etaTmp = etaTh;
            %
            % overlay with restricted shocks
            %
            etaTh(:,1:g) = fliplr(reshape(etaTg,q,g));
            etaCTmp(:,1:g) = fliplr(reshape(etaCTmpg,q,g));
            PredData.Shocks(:,:,it) = etaTh;
            PredData.etaMean = PredData.etaMean+((1/NumPaths)*etaTh);
            %
            % compute the prediction paths
            %
            for i=1:h;
               %
               % update the state
               %
               if k>0;
                  KsiT = (MT1*X(:,i))+(FT1*KsiT)+(BT1*etaTh(:,i));
                  PredPaths(:,i,it) = (A'*X(:,i))+(H'*KsiT)+wTh(:,i);
               else;
                  KsiT = (FT1*KsiT)+(BT1*etaTh(:,i));
                  PredPaths(:,i,it) = (H'*KsiT)+wTh(:,i);
               end;
               %
               % update modesty related material
               %
               if i<=g;
                  if k>0;
                     KsiCTmp = (MT1*X(:,i))+(FT1*KsiCTmp)+(BT1*etaCTmp(:,i));
                     KsiTmp = (MT1*X(:,i))+(FT1*KsiTmp)+(BT1*etaTmp(:,i));
                  else;
                     KsiCTmp = (FT1*KsiCTmp)+(BT1*etaCTmp(:,i));
                     KsiTmp = (FT1*KsiTmp)+(BT1*etaTmp(:,i));
                  end;
                  if it==1;
                     PTTgLZ = (FT1*PTTgLZ*FT1')+CovMatShocks;
                  end;
               end;
            end;
            PredData.MeanPredPath = PredData.MeanPredPath+((1/NumPaths)*PredPaths(:,:,it));
            if AnnualizeData==1;
               PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
            elseif TransData==1;
               %
               % transform the data
               %
               YTmp = [Yinit PredPaths(:,:,it)];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
            else;
               %
               % deal with modesty statistic
               %
               if k>0;
                  PredictCTmp = (A'*X(:,g))+(H'*KsiCTmp)+wTh(:,g);
                  PredictY = (A'*X(:,g))+(H'*Fg*KsiTT);
               else;
                  PredictCTmp = (H'*KsiCTmp)+wTh(:,g);
                  PredictY = H'*Fg*KsiTT;
               end;
               PredCov = (H'*PTTg*H)+R;
               PredDiff = PredictCTmp-PredictY;
               MultiModestyStat(it,1) = PredDiff'*(PredCov\PredDiff);
               UniModestyStat(it,:) = (PredDiff ./ sqrt(diag(PredCov)))';
               %
               % comparison-path for multivariate statistic
               %
               if k>0;
                  PredictTmp = (A'*X(:,g))+(H'*KsiTmp)+wTh(:,g);
               else;
                  PredictTmp = (H'*KsiTmp)+wTh(:,g);
               end;
               PredDiff = PredictTmp-PredictY;
               MultiModestyStat(it,2) = PredDiff'*(PredCov\PredDiff);
               %
               % Leeper-Zha type statistic
               %
               if it==1;
                  PredDiff = PredData.PredMean(:,g)-PredictY;
                  CovPredDiff = diag(H'*PTTgLZ*H);
                  for i=1:length(CovPredDiff);
                     if CovPredDiff(i)==0;
                        PredDiff(i) = 0;
                        CovPredDiff(i) = 1;
                     end;
                  end;
                  UniModestyStatLZ(1,:) = (PredDiff ./ sqrt(CovPredDiff))';
               end;
            end;
            drawnow;
         end;
      else;
         %
         % we have updated beliefs
         %
         PTTg = zeros(r,r);
         Fg = eye(r);
         for i=1:g;
            Q = BT1(:,:,i)*BT1(:,:,i)';
            Fg = FT1(:,:,i)*Fg;
            PTTg = (FT1(:,:,i)*PTTg*FT1(:,:,i)')+Q;
         end;
         %
         for it=1:NumPaths;
            %
            % check if we should update the progress dialog
            %
            if DSGEModel.ShowProgress==1;
               abort = get(WaitHandle,'UserData');
               if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
                  PredPaths = [];
                  break;
               else;
                  ProgressDLG([it/NumPaths it]);
               end;
            end;
            %
            % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
            %
            KsiT = KsiTTz+(S*sqrt(L)*randn(size(L,2),1));
            PredData.HistState(:,it) = KsiT;
            PredData.KsiMean = PredData.KsiMean+((1/NumPaths)*KsiT);
            KsiCTmp = KsiTT;
            KsiTmp = KsiTT;
            PTTgLZ = zeros(r,r);
            %
            % Draw a sequence of measurement errors
            %
            if isempty(Rmeasure)==0;
               wTh = Rmeasure*randn(NumMeasure,h);
               PredData.MeasError(:,:,it) = wTh;
               PredData.wMean = PredData.wMean+((1/NumPaths)*wTh);
               WTg = vec(fliplr(wTh(:,1:g)));
            end;
            %
            % setut the mean for the shock distribution
            %
            kTg = ZTg-UT-(K'*(Xbar+(G*KsiT)+WTg));
            %
            % use speed-up code
            %
            muTg = DpKInvKpDDpK*kTg;
            kCTmp = ZTg-UT-(K'*(Xbar+(G*KsiCTmp)+WTg));
            %
            % use speed-up code
            %
            muCTmp = DpKInvKpDDpK*kCTmp;
            %
            % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
            % and from N(0,I) for period g+1 to h.
            %
            ShockDraw = randn(q*g,1);
            etaTg = muTg+(SigmaTgStd*ShockDraw);
            etaCTmpg = muCTmp+(SigmaTgStd*ShockDraw);
            etaTh = randn(q,h);
            etaCTmp = etaTh;
            etaTmp = etaTh;
            %
            % overlay with restricted shocks
            %
            etaTh(:,1:g) = fliplr(reshape(etaTg,q,g));
            etaCTmp(:,1:g) = fliplr(reshape(etaCTmpg,q,g));
            PredData.Shocks(:,:,it) = etaTh;
            PredData.etaMean = PredData.etaMean+((1/NumPaths)*etaTh);
            %
            % compute the prediction paths
            %
            for i=1:h;
               Q = BT1(:,:,i)*BT1(:,:,i)';
               CovMatShocks = Q*H*K1*((K1'*H'*Q*H*K1)\K1')*H'*Q;
               %
               % update the state
               %
               if k>0;
                  KsiT = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*KsiT)+(BT1(:,:,i)*etaTh(:,i));
                  PredPaths(:,i,it) = (A'*X(:,i))+(H'*KsiT)+wTh(:,i);
               else;
                  KsiT = (FT1(:,:,i)*KsiT)+(BT1(:,:,i)*etaTh(:,i));
                  PredPaths(:,i,it) = (H'*KsiT)+wTh(:,i);
               end;
               %
               % update modesty related material
               %
               if i<=g;
                  if k>0;
                     KsiCTmp = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*KsiCTmp)+(BT1(:,:,i)*etaCTmp(:,i));
                     KsiTmp = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*KsiTmp)+(BT1(:,:,i)*etaTmp(:,i));
                  else;
                     KsiCTmp = (FT1(:,:,i)*KsiCTmp)+(BT1(:,:,i)*etaCTmp(:,i));
                     KsiTmp = (FT1(:,:,i)*KsiTmp)+(BT1(:,:,i)*etaTmp(:,i));
                  end;
                  if it==1;
                     PTTgLZ = (FT1(:,:,i)*PTTgLZ*FT1(:,:,i)')+CovMatShocks;
                  end;
               end;
            end;
            PredData.MeanPredPath = PredData.MeanPredPath+((1/NumPaths)*PredPaths(:,:,it));
            if AnnualizeData==1;
               PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
            elseif TransData==1;
               %
               % transform the data
               %
               YTmp = [Yinit PredPaths(:,:,it)];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
            else;
               %
               % deal with modesty statistic
               %
               if k>0;
                  PredictCTmp = (A'*X(:,g))+(H'*KsiCTmp)+wTh(:,g);
                  PredictY = (A'*X(:,g))+(H'*Fg*KsiTT);
               else;
                  PredictCTmp = (H'*KsiCTmp)+wTh(:,g);
                  PredictY = H'*Fg*KsiTT;
               end;
               PredCov = (H'*PTTg*H)+R;
               PredDiff = PredictCTmp-PredictY;
               MultiModestyStat(it,1) = PredDiff'*(PredCov\PredDiff);
               UniModestyStat(it,:) = (PredDiff ./ sqrt(diag(PredCov)))';
               %
               % comparison-path for multivariate statistic
               %
               if k>0;
                  PredictTmp = (A'*X(:,g))+(H'*KsiTmp)+wTh(:,g);
               else;
                  PredictTmp = (H'*KsiTmp)+wTh(:,g);
               end;
               PredDiff = PredictTmp-PredictY;
               MultiModestyStat(it,2) = PredDiff'*(PredCov\PredDiff);
               %
               % Leeper-Zha type statistic
               %
               if it==1;
                  PredDiff = PredData.PredMean(:,g)-PredictY;
                  CovPredDiff = diag(H'*PTTgLZ*H);
                  for i=1:length(CovPredDiff);
                     if CovPredDiff(i)==0;
                        PredDiff(i) = 0;
                        CovPredDiff(i) = 1;
                     end;
                  end;
                  UniModestyStatLZ(1,:) = (PredDiff ./ sqrt(CovPredDiff))';
               end;
            end;
            drawnow;
         end;
      end;
   else;
      if DSGEModel.FixedBeliefsForProj==1;
         %
         % we have fixed belief coefficients
         %
         Q = BT1*BT1';
         %
         % additional speed-up code suggested by Ingvar Strid
         %
         PTTg = zeros(r,r);
         Fg = eye(r);
         for i=1:g
            Fg = FT1*Fg;
            PTTg = (FT1*PTTg*FT1')+Q;
         end;
         %
         for it=1:NumPaths;
            %
            % check if we should update the progress dialog
            %
            if DSGEModel.ShowProgress==1;
               abort = get(WaitHandle,'UserData');
               if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
                  PredPaths = [];
                  break;
               else;
                  ProgressDLG([it/NumPaths it]);
               end;
            end;
            %
            % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
            %
            KsiT = KsiTTz+(S*sqrt(L)*randn(size(L,2),1));
            PredData.HistState(:,it) = KsiT;
            PredData.KsiMean = PredData.KsiMean+((1/NumPaths)*KsiT);
            KsiCTmp = KsiTT;
            KsiTmp = KsiTT;
            PTTgLZ = zeros(r,r);
            %
            % Draw a sequence of measurement errors
            %
            if isempty(Rmeasure)==0;
               wTh = Rmeasure*randn(NumMeasure,h);
               PredData.MeasError(:,:,it) = wTh;
               PredData.wMean = PredData.wMean+((1/NumPaths)*wTh);
               WTg = vec(fliplr(wTh(:,1:g)));
            end;
            %
            % setut the mean for the shock distribution
            %
            kTg = ZTg-UT-(K'*(Xbar+(G*KsiT)+WTg));
            %
            % use speed-up code
            %
            muTg = DpKInvKpDDpK*kTg;
            kCTmp = ZTg-UT-(K'*(Xbar+(G*KsiCTmp)+WTg));
            %
            % use speed-up code
            %
            muCTmp = DpKInvKpDDpK*kCTmp;
            %
            % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
            % and from N(0,I) for period g+1 to h.
            %
            ShockDraw = randn(q*g,1);
            etaTg = muTg+(SigmaTgStd*ShockDraw);
            etaCTmpg = muCTmp+(SigmaTgStd*ShockDraw);
            etaTh = randn(q,h);
            etaCTmp = etaTh;
            etaTmp = etaTh;
            %
            % overlay with restricted shocks
            %
            etaTh(:,1:g) = fliplr(reshape(etaTg,q,g));
            etaCTmp(:,1:g) = fliplr(reshape(etaCTmpg,q,g));
            PredData.Shocks(:,:,it) = etaTh;
            PredData.etaMean = PredData.etaMean+((1/NumPaths)*etaTh);
            %
            % compute the prediction paths
            %
            for i=1:h;
               %
               % update the state
               %
               if k>0;
                  KsiT = (MT1*X(:,i))+(FT1*KsiT)+(BT1*etaTh(:,i));
                  PredPaths(:,i,it) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*KsiT)+wTh(:,i);
               else;
                  KsiT = (FT1*KsiT)+(BT1*etaTh(:,i));
                  PredPaths(:,i,it) = (H(:,:,LastPeriod+i)'*KsiT)+wTh(:,i);
               end;
               %
               % update modesty related material
               %
               if i<=g;
                  if k>0;
                     KsiCTmp = (MT1*X(:,i))+(FT1*KsiCTmp)+(BT1*etaCTmp(:,i));
                     KsiTmp = (MT1*X(:,i))+(FT1*KsiTmp)+(BT1*etaTmp(:,i));
                  else;
                     KsiCTmp = (FT1*KsiCTmp)+(BT1*etaCTmp(:,i));
                     KsiTmp = (FT1*KsiTmp)+(BT1*etaTmp(:,i));
                  end;
                  if it==1;
                     PTTgLZ = (FT1*PTTgLZ*FT1')+(Q*H(:,:,LastPeriod+i)*K1*((K1'*H(:,:,LastPeriod+i)'*Q*H(:,:,LastPeriod+i)*K1)\K1')*H(:,:,LastPeriod+i)'*Q);
                  end;
               end;
            end;
            PredData.MeanPredPath = PredData.MeanPredPath+((1/NumPaths)*PredPaths(:,:,it));
            if AnnualizeData==1;
               PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
            elseif TransData==1;
               %
               % transform the data
               %
               YTmp = [Yinit PredPaths(:,:,it)];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
            else;
               %
               % deal with modesty statistic
               %
               if k>0;
                  PredictCTmp = (A'*X(:,g))+(H(:,:,LastPeriod+g)'*KsiCTmp)+wTh(:,g);
                  PredictY = (A'*X(:,g))+(H(:,:,LastPeriod+g)'*Fg*KsiTT);
               else;
                  PredictCTmp = (H(:,:,LastPeriod+g)'*KsiCTmp)+wTh(:,g);
                  PredictY = H(:,:,LastPeriod+g)'*Fg*KsiTT;
               end;
               PredCov = (H(:,:,LastPeriod+g)'*PTTg*H(:,:,LastPeriod+g))+R;
               PredDiff = PredictCTmp-PredictY;
               MultiModestyStat(it,1) = PredDiff'*(PredCov\PredDiff);
               UniModestyStat(it,:) = (PredDiff ./ sqrt(diag(PredCov)))';
               %
               % comparison-path for multivariate statistic
               %
               if k>0;
                  PredictTmp = (A'*X(:,g))+(H(:,:,LastPeriod+g)'*KsiTmp)+wTh(:,g);
               else;
                  PredictTmp = (H(:,:,LastPeriod+g)'*KsiTmp)+wTh(:,g);
               end;
               PredDiff = PredictTmp-PredictY;
               MultiModestyStat(it,2) = PredDiff'*(PredCov\PredDiff);
               %
               % Leeper-Zha type statistic
               %
               if it==1;
                  PredDiff = PredData.PredMean(:,g)-PredictY;
                  CovPredDiff = diag(H'*PTTgLZ*H);
                  for i=1:length(CovPredDiff);
                     if CovPredDiff(i)==0;
                        PredDiff(i) = 0;
                        CovPredDiff(i) = 1;
                     end;
                  end;
                  UniModestyStatLZ(1,:) = (PredDiff ./ sqrt(CovPredDiff))';
               end;
            end;
            drawnow;
         end;
      else;
         %
         % we have updated beliefs
         %
         PTTg = zeros(r,r);
         Fg = eye(r);
         for i=1:g;
            Q = BT1(:,:,i)*BT1(:,:,i)';
            Fg = FT1(:,:,i)*Fg;
            PTTg = (FT1(:,:,i)*PTTg*FT1(:,:,i)')+Q;
         end;
         %
         for it=1:NumPaths;
            %
            % check if we should update the progress dialog
            %
            if DSGEModel.ShowProgress==1;
               abort = get(WaitHandle,'UserData');
               if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
                  PredPaths = [];
                  break;
               else;
                  ProgressDLG([it/NumPaths it]);
               end;
            end;
            %
            % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
            %
            KsiT = KsiTTz+(S*sqrt(L)*randn(size(L,2),1));
            PredData.HistState(:,it) = KsiT;
            PredData.KsiMean = PredData.KsiMean+((1/NumPaths)*KsiT);
            KsiCTmp = KsiTT;
            KsiTmp = KsiTT;
            PTTgLZ = zeros(r,r);
            %
            % Draw a sequence of measurement errors
            %
            if isempty(Rmeasure)==0;
               wTh = Rmeasure*randn(NumMeasure,h);
               PredData.MeasError(:,:,it) = wTh;
               PredData.wMean = PredData.wMean+((1/NumPaths)*wTh);
               WTg = vec(fliplr(wTh(:,1:g)));
            end;
            %
            % setut the mean for the shock distribution
            %
            kTg = ZTg-UT-(K'*(Xbar+(G*KsiT)+WTg));
            %
            % use speed-up code
            %
            muTg = DpKInvKpDDpK*kTg;
            kCTmp = ZTg-UT-(K'*(Xbar+(G*KsiCTmp)+WTg));
            %
            % use speed-up code
            %
            muCTmp = DpKInvKpDDpK*kCTmp;
            %
            % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
            % and from N(0,I) for period g+1 to h.
            %
            ShockDraw = randn(q*g,1);
            etaTg = muTg+(SigmaTgStd*ShockDraw);
            etaCTmpg = muCTmp+(SigmaTgStd*ShockDraw);
            etaTh = randn(q,h);
            etaCTmp = etaTh;
            etaTmp = etaTh;
            %
            % overlay with restricted shocks
            %
            etaTh(:,1:g) = fliplr(reshape(etaTg,q,g));
            etaCTmp(:,1:g) = fliplr(reshape(etaCTmpg,q,g));
            PredData.Shocks(:,:,it) = etaTh;
            PredData.etaMean = PredData.etaMean+((1/NumPaths)*etaTh);
            %
            % compute the prediction paths
            %
            for i=1:h;
               Q = BT1(:,:,i)*BT1(:,:,i)';
               %
               % update the state
               %
               if k>0;
                  KsiT = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*KsiT)+(BT1(:,:,i)*etaTh(:,i));
                  PredPaths(:,i,it) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*KsiT)+wTh(:,i);
               else;
                  KsiT = (FT1(:,:,i)*KsiT)+(BT1(:,:,i)*etaTh(:,i));
                  PredPaths(:,i,it) = (H(:,:,LastPeriod+i)'*KsiT)+wTh(:,i);
               end;
               %
               % update modesty related material
               %
               if i<=g;
                  if k>0;
                     KsiCTmp = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*KsiCTmp)+(BT1(:,:,i)*etaCTmp(:,i));
                     KsiTmp = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*KsiTmp)+(BT1(:,:,i)*etaTmp(:,i));
                  else;
                     KsiCTmp = (FT1(:,:,i)*KsiCTmp)+(BT1(:,:,i)*etaCTmp(:,i));
                     KsiTmp = (FT1(:,:,i)*KsiTmp)+(BT1(:,:,i)*etaTmp(:,i));
                  end;
                  if it==1;
                     PTTgLZ = (FT1(:,:,i)*PTTgLZ*FT1(:,:,i)')+(Q*H(:,:,LastPeriod+i)*K1*((K1'*H(:,:,LastPeriod+i)'*Q*H(:,:,LastPeriod+i)*K1)\K1')*H(:,:,LastPeriod+i)'*Q);
                  end;
               end;
            end;
            PredData.MeanPredPath = PredData.MeanPredPath+((1/NumPaths)*PredPaths(:,:,it));
            if AnnualizeData==1;
               PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
            elseif TransData==1;
               %
               % transform the data
               %
               YTmp = [Yinit PredPaths(:,:,it)];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
            else;
               %
               % deal with modesty statistic
               %
               if k>0;
                  PredictCTmp = (A'*X(:,g))+(H(:,:,LastPeriod+g)'*KsiCTmp)+wTh(:,g);
                  PredictY = (A'*X(:,g))+(H(:,:,LastPeriod+g)'*Fg*KsiTT);
               else;
                  PredictCTmp = (H(:,:,LastPeriod+g)'*KsiCTmp)+wTh(:,g);
                  PredictY = H(:,:,LastPeriod+g)'*Fg*KsiTT;
               end;
               PredCov = (H(:,:,LastPeriod+g)'*PTTg*H(:,:,LastPeriod+g))+R;
               PredDiff = PredictCTmp-PredictY;
               MultiModestyStat(it,1) = PredDiff'*(PredCov\PredDiff);
               UniModestyStat(it,:) = (PredDiff ./ sqrt(diag(PredCov)))';
               %
               % comparison-path for multivariate statistic
               %
               if k>0;
                  PredictTmp = (A'*X(:,g))+(H(:,:,LastPeriod+g)'*KsiTmp)+wTh(:,g);
               else;
                  PredictTmp = (H(:,:,LastPeriod+g)'*KsiTmp)+wTh(:,g);
               end;
               PredDiff = PredictTmp-PredictY;
               MultiModestyStat(it,2) = PredDiff'*(PredCov\PredDiff);
               %
               % Leeper-Zha type statistic
               %
               if it==1;
                  PredDiff = PredData.PredMean(:,g)-PredictY;
                  CovPredDiff = diag(H'*PTTgLZ*H);
                  for i=1:length(CovPredDiff);
                     if CovPredDiff(i)==0;
                        PredDiff(i) = 0;
                        CovPredDiff(i) = 1;
                     end;
                  end;
                  UniModestyStatLZ(1,:) = (PredDiff ./ sqrt(CovPredDiff))';
               end;
            end;
            drawnow;
         end;
      end;
   end;
   %
   % close the wait handle
   %
   if DSGEModel.ShowProgress==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   elseif (DSGEModel.ShowProgress==0)&(TransData==1);
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   %
   % we can do parallel computations over the parameter draws
   %
   TypeStr = 'Original';
   if AnnualizeData==1;
      TypeStr = 'Annualized';
   elseif TransData==1;
      TypeStr = 'Transformed';
   end;
   txt = ['Please wait while YADA computes predictive distributions under adaptive learning for the ' lower(TypeStr) ' data using parallel computations with ' int2str(CurrINI.NumWorkers) ' workers. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Adaptive Learning: Pred. Dist. - ' TypeStr ' Data'],600,250,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   %
   % initialize
   %
   HistState = PredData.HistState;
   Shocks = PredData.Shocks;
   MeasError = PredData.MeasError;
   PredPaths = zeros(n,h,NumPaths);
   %
   MultiModestyStat = zeros(NumPaths,2);
   UniModestyStat = zeros(NumPaths,n);
   %
   if DSGEModel.RandomNumberValue==1;
      %
      % without eval statements the spmd-code gives an error in older matlab systems
      %
      eval('spmd; rng(0,''combRecursive''); end;');
   else;
      eval('spmd; rng(''default''); end;');
   end;
   %
   abort = '';
   if length(size(H))==2;
      if DSGEModel.FixedBeliefsForProj==1;
         %
         % we have fixed belief coefficients
         %
         Q = BT1*BT1';
         CovMatShocks = Q*H*K1*((K1'*H'*Q*H*K1)\K1')*H'*Q;
         %
         % additional speed-up code suggested by Ingvar Strid
         %
         PTTg = zeros(r,r);
         Fg = eye(r);
         for i=1:g
            Fg = FT1*Fg;
            PTTg = (FT1*PTTg*FT1')+Q;
         end;
         %
         parfor it=1:NumPaths;
            %
            % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
            %
            KsiT = KsiTTz+(S*sqrt(L)*randn(size(L,2),1));
            HistState(:,it) = KsiT;
            KsiCTmp = KsiTT;
            KsiTmp = KsiTT;
            PTTgLZ = zeros(r,r);
            %
            % Draw a sequence of measurement errors
            %
            if isempty(Rmeasure)==0;
               wTh = Rmeasure*randn(NumMeasure,h);
               MeasError(:,:,it) = wTh;
               WTg = vec(fliplr(wTh(:,1:g)));
            else;
               wTh = zeros(n,h);
               WTg = 0;
            end;
            %
            % setut the mean for the shock distribution
            %
            kTg = ZTg-UT-(K'*(Xbar+(G*KsiT)+WTg));
            %
            % use speed-up code
            %
            muTg = DpKInvKpDDpK*kTg;
            kCTmp = ZTg-UT-(K'*(Xbar+(G*KsiCTmp)+WTg));
            %
            % use speed-up code
            %
            muCTmp = DpKInvKpDDpK*kCTmp;
            %
            % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
            % and from N(0,I) for period g+1 to h.
            %
            ShockDraw = randn(q*g,1);
            etaTg = muTg+(SigmaTgStd*ShockDraw);
            etaCTmpg = muCTmp+(SigmaTgStd*ShockDraw);
            etaTh = randn(q,h);
            etaCTmp = etaTh;
            etaTmp = etaTh;
            %
            % overlay with restricted shocks
            %
            etaTh(:,1:g) = fliplr(reshape(etaTg,q,g));
            etaCTmp(:,1:g) = fliplr(reshape(etaCTmpg,q,g));
            Shocks(:,:,it) = etaTh;
            %
            % compute the prediction paths
            %
            PredPath = zeros(n,h);
            for i=1:h;
               %
               % update the state
               %
               if k>0;
                  KsiT = (MT1*X(:,i))+(FT1*KsiT)+(BT1*etaTh(:,i));
                  PredPath(:,i) = (A'*X(:,i))+(H'*KsiT)+wTh(:,i);
               else;
                  KsiT = (FT1*KsiT)+(BT1*etaTh(:,i));
                  PredPath(:,i) = (H'*KsiT)+wTh(:,i);
               end;
               %
               % update modesty related material
               %
               if i<=g;
                  if k>0;
                     KsiCTmp = (MT1*X(:,i))+(FT1*KsiCTmp)+(BT1*etaCTmp(:,i));
                     KsiTmp = (MT1*X(:,i))+(FT1*KsiTmp)+(BT1*etaTmp(:,i));
                  else;
                     KsiCTmp = (FT1*KsiCTmp)+(BT1*etaCTmp(:,i));
                     KsiTmp = (FT1*KsiTmp)+(BT1*etaTmp(:,i));
                  end;
                  if it==1;
                     PTTgLZ = (FT1*PTTgLZ*FT1')+CovMatShocks;
                  end;
               end;
            end;
            PredPaths(:,:,it) = PredPath;
            MMStat = zeros(1,2);
            %
            if AnnualizeData==1;
               PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
            elseif TransData==1;
               %
               % transform the data
               %
               YTmp = [Yinit PredPaths(:,:,it)];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
            else;
               %
               % deal with modesty statistic
               %
               if k>0;
                  PredictCTmp = (A'*X(:,g))+(H'*KsiCTmp)+wTh(:,g);
                  PredictY = (A'*X(:,g))+(H'*Fg*KsiTT);
               else;
                  PredictCTmp = (H'*KsiCTmp)+wTh(:,g);
                  PredictY = H'*Fg*KsiTT;
               end;
               PredCov = (H'*PTTg*H)+R;
               PredDiff = PredictCTmp-PredictY;
               MMStat(1,1) = PredDiff'*(PredCov\PredDiff);
               UniModestyStat(it,:) = (PredDiff ./ sqrt(diag(PredCov)))';
               %
               % comparison-path for multivariate statistic
               %
               if k>0;
                  PredictTmp = (A'*X(:,g))+(H'*KsiTmp)+wTh(:,g);
               else;
                  PredictTmp = (H'*KsiTmp)+wTh(:,g);
               end;
               PredDiff = PredictTmp-PredictY;
               MMStat(1,2) = PredDiff'*(PredCov\PredDiff);
               MultiModestyStat(it,:) = MMStat;
               %
               % Leeper-Zha type statistic
               %
               if it==1;
                  PredDiff = PredData.PredMean(:,g)-PredictY;
                  CovPredDiff = diag(H'*PTTgLZ*H);
                  for i=1:length(CovPredDiff);
                     if CovPredDiff(i)==0;
                        PredDiff(i) = 0;
                        CovPredDiff(i) = 1;
                     end;
                  end;
                  UniModestyStatLZ = (PredDiff ./ sqrt(CovPredDiff))';
               end;
            end;
            drawnow;
         end;
      else;
         %
         % we have updated beliefs
         %
         PTTg = zeros(r,r);
         Fg = eye(r);
         for i=1:g;
            Q = BT1(:,:,i)*BT1(:,:,i)';
            Fg = FT1(:,:,i)*Fg;
            PTTg = (FT1(:,:,i)*PTTg*FT1(:,:,i)')+Q;
         end;
         %
         parfor it=1:NumPaths;
            %
            % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
            %
            KsiT = KsiTTz+(S*sqrt(L)*randn(size(L,2),1));
            HistState(:,it) = KsiT;
            KsiCTmp = KsiTT;
            KsiTmp = KsiTT;
            PTTgLZ = zeros(r,r);
            %
            % Draw a sequence of measurement errors
            %
            if isempty(Rmeasure)==0;
               wTh = Rmeasure*randn(NumMeasure,h);
               MeasError(:,:,it) = wTh;
               WTg = vec(fliplr(wTh(:,1:g)));
            else;
               wTh = zeros(n,h);
               WTg = 0;
            end;
            %
            % setut the mean for the shock distribution
            %
            kTg = ZTg-UT-(K'*(Xbar+(G*KsiT)+WTg));
            %
            % use speed-up code
            %
            muTg = DpKInvKpDDpK*kTg;
            kCTmp = ZTg-UT-(K'*(Xbar+(G*KsiCTmp)+WTg));
            %
            % use speed-up code
            %
            muCTmp = DpKInvKpDDpK*kCTmp;
            %
            % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
            % and from N(0,I) for period g+1 to h.
            %
            ShockDraw = randn(q*g,1);
            etaTg = muTg+(SigmaTgStd*ShockDraw);
            etaCTmpg = muCTmp+(SigmaTgStd*ShockDraw);
            etaTh = randn(q,h);
            etaCTmp = etaTh;
            etaTmp = etaTh;
            %
            % overlay with restricted shocks
            %
            etaTh(:,1:g) = fliplr(reshape(etaTg,q,g));
            etaCTmp(:,1:g) = fliplr(reshape(etaCTmpg,q,g));
            Shocks(:,:,it) = etaTh;
            %
            % compute the prediction paths
            %
            PredPath = zeros(n,h);
            for i=1:h;
               Q = BT1(:,:,i)*BT1(:,:,i)';
               CovMatShocks = Q*H*K1*((K1'*H'*Q*H*K1)\K1')*H'*Q;
               %
               % update the state
               %
               if k>0;
                  KsiT = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*KsiT)+(BT1(:,:,i)*etaTh(:,i));
                  PredPath(:,i) = (A'*X(:,i))+(H'*KsiT)+wTh(:,i);
               else;
                  KsiT = (FT1(:,:,i)*KsiT)+(BT1(:,:,i)*etaTh(:,i));
                  PredPath(:,i) = (H'*KsiT)+wTh(:,i);
               end;
               %
               % update modesty related material
               %
               if i<=g;
                  if k>0;
                     KsiCTmp = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*KsiCTmp)+(BT1(:,:,i)*etaCTmp(:,i));
                     KsiTmp = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*KsiTmp)+(BT1(:,:,i)*etaTmp(:,i));
                  else;
                     KsiCTmp = (FT1(:,:,i)*KsiCTmp)+(BT1(:,:,i)*etaCTmp(:,i));
                     KsiTmp = (FT1(:,:,i)*KsiTmp)+(BT1(:,:,i)*etaTmp(:,i));
                  end;
                  if it==1;
                     PTTgLZ = (FT1(:,:,i)*PTTgLZ*FT1(:,:,i)')+CovMatShocks;
                  end;
               end;
            end;
            PredPaths(:,:,it) = PredPath;
            MMStat = zeros(1,2);
            if AnnualizeData==1;
               PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
            elseif TransData==1;
               %
               % transform the data
               %
               YTmp = [Yinit PredPaths(:,:,it)];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
            else;
               %
               % deal with modesty statistic
               %
               if k>0;
                  PredictCTmp = (A'*X(:,g))+(H'*KsiCTmp)+wTh(:,g);
                  PredictY = (A'*X(:,g))+(H'*Fg*KsiTT);
               else;
                  PredictCTmp = (H'*KsiCTmp)+wTh(:,g);
                  PredictY = H'*Fg*KsiTT;
               end;
               PredCov = (H'*PTTg*H)+R;
               PredDiff = PredictCTmp-PredictY;
               MMStat(1,1) = PredDiff'*(PredCov\PredDiff);
               UniModestyStat(it,:) = (PredDiff ./ sqrt(diag(PredCov)))';
               %
               % comparison-path for multivariate statistic
               %
               if k>0;
                  PredictTmp = (A'*X(:,g))+(H'*KsiTmp)+wTh(:,g);
               else;
                  PredictTmp = (H'*KsiTmp)+wTh(:,g);
               end;
               PredDiff = PredictTmp-PredictY;
               MMStat(1,2) = PredDiff'*(PredCov\PredDiff);
               MultiModestyStat(it,:) = MMStat;
               %
               % Leeper-Zha type statistic
               %
               if it==1;
                  PredDiff = PredData.PredMean(:,g)-PredictY;
                  CovPredDiff = diag(H'*PTTgLZ*H);
                  for i=1:length(CovPredDiff);
                     if CovPredDiff(i)==0;
                        PredDiff(i) = 0;
                        CovPredDiff(i) = 1;
                     end;
                  end;
                  UniModestyStatLZ = (PredDiff ./ sqrt(CovPredDiff))';
               end;
            end;
            drawnow;
         end;
      end;
   else;
      if DSGEModel.FixedBeliefsForProj==1;
         %
         % we have fixed belief coefficients
         %
         Q = BT1*BT1';
         %
         % additional speed-up code suggested by Ingvar Strid
         %
         PTTg = zeros(r,r);
         Fg = eye(r);
         for i=1:g
            Fg = FT1*Fg;
            PTTg = (FT1*PTTg*FT1')+Q;
         end;
         %
         parfor it=1:NumPaths;
            %
            % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
            %
            KsiT = KsiTTz+(S*sqrt(L)*randn(size(L,2),1));
            HistState(:,it) = KsiT;
            KsiCTmp = KsiTT;
            KsiTmp = KsiTT;
            PTTgLZ = zeros(r,r);
            %
            % Draw a sequence of measurement errors
            %
            if isempty(Rmeasure)==0;
               wTh = Rmeasure*randn(NumMeasure,h);
               MeasError(:,:,it) = wTh;
               WTg = vec(fliplr(wTh(:,1:g)));
            else;
               wTh = zeros(n,h);
               WTg = 0;
            end;
            %
            % setut the mean for the shock distribution
            %
            kTg = ZTg-UT-(K'*(Xbar+(G*KsiT)+WTg));
            %
            % use speed-up code
            %
            muTg = DpKInvKpDDpK*kTg;
            kCTmp = ZTg-UT-(K'*(Xbar+(G*KsiCTmp)+WTg));
            %
            % use speed-up code
            %
            muCTmp = DpKInvKpDDpK*kCTmp;
            %
            % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
            % and from N(0,I) for period g+1 to h.
            %
            ShockDraw = randn(q*g,1);
            etaTg = muTg+(SigmaTgStd*ShockDraw);
            etaCTmpg = muCTmp+(SigmaTgStd*ShockDraw);
            etaTh = randn(q,h);
            etaCTmp = etaTh;
            etaTmp = etaTh;
            %
            % overlay with restricted shocks
            %
            etaTh(:,1:g) = fliplr(reshape(etaTg,q,g));
            etaCTmp(:,1:g) = fliplr(reshape(etaCTmpg,q,g));
            Shocks(:,:,it) = etaTh;
            %
            % compute the prediction paths
            %
            PredPath = zeros(n,h);
            for i=1:h;
               %
               % update the state
               %
               if k>0;
                  KsiT = (MT1*X(:,i))+(FT1*KsiT)+(BT1*etaTh(:,i));
                  PredPath(:,i) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*KsiT)+wTh(:,i);
               else;
                  KsiT = (FT1*KsiT)+(BT1*etaTh(:,i));
                  PredPath(:,i) = (H(:,:,LastPeriod+i)'*KsiT)+wTh(:,i);
               end;
               %
               % update modesty related material
               %
               if i<=g;
                  if k>0;
                     KsiCTmp = (MT1*X(:,i))+(FT1*KsiCTmp)+(BT1*etaCTmp(:,i));
                     KsiTmp = (MT1*X(:,i))+(FT1*KsiTmp)+(BT1*etaTmp(:,i));
                  else;
                     KsiCTmp = (FT1*KsiCTmp)+(BT1*etaCTmp(:,i));
                     KsiTmp = (FT1*KsiTmp)+(BT1*etaTmp(:,i));
                  end;
                  if it==1;
                     PTTgLZ = (FT1*PTTgLZ*FT1')+(Q*H(:,:,LastPeriod+i)*K1*((K1'*H(:,:,LastPeriod+i)'*Q*H(:,:,LastPeriod+i)*K1)\K1')*H(:,:,LastPeriod+i)'*Q);
                  end;
               end;
            end;
            PredPaths(:,:,it) = PredPath;
            MMStat = zeros(1,2);
            if AnnualizeData==1;
               PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
            elseif TransData==1;
               %
               % transform the data
               %
               YTmp = [Yinit PredPaths(:,:,it)];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
            else;
               %
               % deal with modesty statistic
               %
               if k>0;
                  PredictCTmp = (A'*X(:,g))+(H(:,:,LastPeriod+g)'*KsiCTmp)+wTh(:,g);
                  PredictY = (A'*X(:,g))+(H(:,:,LastPeriod+g)'*Fg*KsiTT);
               else;
                  PredictCTmp = (H(:,:,LastPeriod+g)'*KsiCTmp)+wTh(:,g);
                  PredictY = H(:,:,LastPeriod+g)'*Fg*KsiTT;
               end;
               PredCov = (H(:,:,LastPeriod+g)'*PTTg*H(:,:,LastPeriod+g))+R;
               PredDiff = PredictCTmp-PredictY;
               MMStat(1,1) = PredDiff'*(PredCov\PredDiff);
               UniModestyStat(it,:) = (PredDiff ./ sqrt(diag(PredCov)))';
               %
               % comparison-path for multivariate statistic
               %
               if k>0;
                  PredictTmp = (A'*X(:,g))+(H(:,:,LastPeriod+g)'*KsiTmp)+wTh(:,g);
               else;
                  PredictTmp = (H(:,:,LastPeriod+g)'*KsiTmp)+wTh(:,g);
               end;
               PredDiff = PredictTmp-PredictY;
               MMStat(1,2) = PredDiff'*(PredCov\PredDiff);
               MultiModestyStat(it,:) = MMStat;
               %
               % Leeper-Zha type statistic
               %
               if it==1;
                  PredDiff = PredData.PredMean(:,g)-PredictY;
                  CovPredDiff = diag(H'*PTTgLZ*H);
                  for i=1:length(CovPredDiff);
                     if CovPredDiff(i)==0;
                        PredDiff(i) = 0;
                        CovPredDiff(i) = 1;
                     end;
                  end;
                  UniModestyStatLZ = (PredDiff ./ sqrt(CovPredDiff))';
               end;
            end;
            drawnow;
         end;
      else;
         %
         % we have updated beliefs
         %
         PTTg = zeros(r,r);
         Fg = eye(r);
         for i=1:g;
            Q = BT1(:,:,i)*BT1(:,:,i)';
            Fg = FT1(:,:,i)*Fg;
            PTTg = (FT1(:,:,i)*PTTg*FT1(:,:,i)')+Q;
         end;
         %
         parfor it=1:NumPaths;
            %
            % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
            %
            KsiT = KsiTTz+(S*sqrt(L)*randn(size(L,2),1));
            HistState(:,it) = KsiT;
            KsiCTmp = KsiTT;
            KsiTmp = KsiTT;
            PTTgLZ = zeros(r,r);
            %
            % Draw a sequence of measurement errors
            %
            if isempty(Rmeasure)==0;
               wTh = Rmeasure*randn(NumMeasure,h);
               MeasError(:,:,it) = wTh;
               WTg = vec(fliplr(wTh(:,1:g)));
            else;
               wTh = zeros(n,h);
               WTg = 0;
            end;
            %
            % setut the mean for the shock distribution
            %
            kTg = ZTg-UT-(K'*(Xbar+(G*KsiT)+WTg));
            %
            % use speed-up code
            %
            muTg = DpKInvKpDDpK*kTg;
            kCTmp = ZTg-UT-(K'*(Xbar+(G*KsiCTmp)+WTg));
            %
            % use speed-up code
            %
            muCTmp = DpKInvKpDDpK*kCTmp;
            %
            % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
            % and from N(0,I) for period g+1 to h.
            %
            ShockDraw = randn(q*g,1);
            etaTg = muTg+(SigmaTgStd*ShockDraw);
            etaCTmpg = muCTmp+(SigmaTgStd*ShockDraw);
            etaTh = randn(q,h);
            etaCTmp = etaTh;
            etaTmp = etaTh;
            %
            % overlay with restricted shocks
            %
            etaTh(:,1:g) = fliplr(reshape(etaTg,q,g));
            etaCTmp(:,1:g) = fliplr(reshape(etaCTmpg,q,g));
            Shocks(:,:,it) = etaTh;
            %
            % compute the prediction paths
            %
            PredPath = zeros(n,h);
            for i=1:h;
               Q = BT1(:,:,i)*BT1(:,:,i)';
               %
               % update the state
               %
               if k>0;
                  KsiT = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*KsiT)+(BT1(:,:,i)*etaTh(:,i));
                  PredPath(:,i) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*KsiT)+wTh(:,i);
               else;
                  KsiT = (FT1(:,:,i)*KsiT)+(BT1(:,:,i)*etaTh(:,i));
                  PredPath(:,i) = (H(:,:,LastPeriod+i)'*KsiT)+wTh(:,i);
               end;
               %
               % update modesty related material
               %
               if i<=g;
                  if k>0;
                     KsiCTmp = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*KsiCTmp)+(BT1(:,:,i)*etaCTmp(:,i));
                     KsiTmp = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*KsiTmp)+(BT1(:,:,i)*etaTmp(:,i));
                  else;
                     KsiCTmp = (FT1(:,:,i)*KsiCTmp)+(BT1(:,:,i)*etaCTmp(:,i));
                     KsiTmp = (FT1(:,:,i)*KsiTmp)+(BT1(:,:,i)*etaTmp(:,i));
                  end;
                  if it==1;
                     PTTgLZ = (FT1(:,:,i)*PTTgLZ*FT1(:,:,i)')+(Q*H(:,:,LastPeriod+i)*K1*((K1'*H(:,:,LastPeriod+i)'*Q*H(:,:,LastPeriod+i)*K1)\K1')*H(:,:,LastPeriod+i)'*Q);
                  end;
               end;
            end;
            PredPaths(:,:,it) = PredPath;
            MMStat = zeros(1,2);
            %
            if AnnualizeData==1;
               PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
            elseif TransData==1;
               %
               % transform the data
               %
               YTmp = [Yinit PredPaths(:,:,it)];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
            else;
               %
               % deal with modesty statistic
               %
               if k>0;
                  PredictCTmp = (A'*X(:,g))+(H(:,:,LastPeriod+g)'*KsiCTmp)+wTh(:,g);
                  PredictY = (A'*X(:,g))+(H(:,:,LastPeriod+g)'*Fg*KsiTT);
               else;
                  PredictCTmp = (H(:,:,LastPeriod+g)'*KsiCTmp)+wTh(:,g);
                  PredictY = H(:,:,LastPeriod+g)'*Fg*KsiTT;
               end;
               PredCov = (H(:,:,LastPeriod+g)'*PTTg*H(:,:,LastPeriod+g))+R;
               PredDiff = PredictCTmp-PredictY;
               MMStat(1,1) = PredDiff'*(PredCov\PredDiff);
               UniModestyStat(it,:) = (PredDiff ./ sqrt(diag(PredCov)))';
               %
               % comparison-path for multivariate statistic
               %
               if k>0;
                  PredictTmp = (A'*X(:,g))+(H(:,:,LastPeriod+g)'*KsiTmp)+wTh(:,g);
               else;
                  PredictTmp = (H(:,:,LastPeriod+g)'*KsiTmp)+wTh(:,g);
               end;
               PredDiff = PredictTmp-PredictY;
               MMStat(1,2) = PredDiff'*(PredCov\PredDiff);
               MultiModestyStat(it,:) = MMStat;
               %
               % Leeper-Zha type statistic
               %
               if it==1;
                  PredDiff = PredData.PredMean(:,g)-PredictY;
                  CovPredDiff = diag(H'*PTTgLZ*H);
                  for i=1:length(CovPredDiff);
                     if CovPredDiff(i)==0;
                        PredDiff(i) = 0;
                        CovPredDiff(i) = 1;
                     end;
                  end;
                  UniModestyStatLZ = (PredDiff ./ sqrt(CovPredDiff))';
               end;
            end;
            drawnow;
         end;
      end;
   end;
   %
   % compute mean values of the initial state, the shocks and the measurement errors
   %
   PredData.KsiMean = (1/NumPaths)*sum(HistState,2);
   PredData.etaMean = (1/NumPaths)*sum(Shocks,3);
   if isempty(Rmeasure)==0;
      PredData.wMean = (1/NumPaths)*sum(MeasError,3);
   end;
   PredData.MeanPredPath = (1/NumPaths)*sum(PredPaths,3);
   PredData.HistState = HistState;
   PredData.Shocks = Shocks;
   PredData.MeasError = MeasError;
   %
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
if (strcmp(abort,'cancel')==0)&(DSGEModel.RunPredictionEvent==1);
   MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
   if DSGEModel.AdjustPredictionPaths==1;
      %
      % we need to fix the paths such that the mean is correct
      %
      AdjustMean = PredData.PredMean-MeanPrediction;
      for i=1:NumPaths;
         PEPredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
      end;
   else;
      PEPredPaths = PredPaths;
   end;
   %
   % compute the prediction events results
   %
   PredEventData = CalculatePredictionEvents(PEPredPaths,DSGEModel.PredictionEvent);
   %
   if isempty(YObserved)==0;
      AdjustMean = YObserved-MeanPrediction;
      %
      % make sure that missing observations don't affect adjustment
      %
      AdjustMean(isnan(AdjustMean)) = 0;
      for i=1:NumPaths;
         PEPredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
      end;
      YObsEventData = CalculatePredictionEvents(PEPredPaths,DSGEModel.PredictionEvent);
   end;
end;
%
% check if we should compute kernel density estimates
%
if (strcmp(abort,'cancel')==0)&(NumPaths>=500);
   if DSGEModel.RunPredictionEvent==0;
      if DSGEModel.AdjustPredictionPaths==1;
         MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
         AdjustMean = PredData.PredMean-MeanPrediction;
         for i=1:NumPaths;
            PEPredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
         end;
      else;
         PEPredPaths = PredPaths;
      end;
   end;
   %
   [PredData.KernelX,PredData.KernelY] = PredictionKernelDensity(PEPredPaths);
end;

%
% end of AdaptiveLearningDSGECondPredictionPathsThetaWZ.m
%
