function hessian_mat = YADAHessianMatrix(func,x,gstep,NumWorkers,varargin)
% function hessian_mat = hessian_sparse(func,x,gstep,varargin)
% Computes second order partial derivatives
%
% INPUTS
%    func        [string]   name of the function
%    x           [double]   vector, the Hessian of "func" is evaluated at x.
%    gstep       [double]   scalar, size of epsilon.
%    NumWorkers  [integer]  with the number of workers in the parallel pool
%    varargin    [void]     list of additional arguments for "func".
%
% OUTPUTS
%    hessian_mat [double, sparse]   Hessian matrix
%
% ALGORITHM
%    Uses Abramowitz and Stegun (1965) formulas 25.3.24 and 25.3.27 p. 884
%
% SPECIAL REQUIREMENTS
%    none
%
% code for parallelization and some vectorization added by Anders Warne to allow
% for speedier calculations. Code related to wait dialog also added by A.W.
%

% Copyright (C) 2001-2017 Dynare Team
%
% This file is part of Dynare.
%
% Dynare is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% Dynare is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.

x = x(:);
if NumWorkers>1;
   n = length(x);
   h1 = max(abs(x),sqrt(gstep)*ones(n,1))*eps^(1/6);
   %h_1 = h1;
   %
   f0 = feval(func,x,varargin{:});
   sf = size(f0,1);
   f1 = zeros(sf,n);
   f_1 = f1;
   xh1 = x;
   parfor i=1:n;
      xh1 = x;
      xh1(i) = x(i)+h1(i);
      f1(:,i) = feval(func,xh1,varargin{:});
      %xh1(i) = x(i)-h_1(i);
      xh1(i) = x(i)-h1(i);
      f_1(:,i) = feval(func,xh1,varargin{:});
   end;
   xh_1 = x;
   hessian_mat = sparse(sf,n*n);
   %
   % we move temp outside the for-loop since it's fixed
   %
   temp = f1+f_1-f0*ones(1,n);
   %hessian_mat(:,(1:n+1:n*n)) = (f1+f_1-2*f0*ones(1,n))./(h1.*h_1)';
   hessian_mat(:,(1:n+1:n*n)) = (f1+f_1-2*f0*ones(1,n))./(h1.*h1)';
   hmat = sparse(sf,n-1);
   %
   for i=2:n;
      %     if i > 1
      %         k=[i:n:n*(i-1)];
      %         hessian_mat(:,(i-1)*n+1:(i-1)*n+i-1)=hessian_mat(:,k);
      %         hessian_mat(:,k)=0;
      %     end
      parfor j=1:i-1;
         xh1 = x;
         xh_1 = x;
         xh1(i) = x(i)+h1(i);
         %xh1(j) = x(j)+h_1(j);
         xh1(j) = x(j)+h1(j);
         xh_1(i) = x(i)-h1(i);
         %xh_1(j) = x(j)-h_1(j);
         xh_1(j) = x(j)-h1(j);
         %hmat(:,j) = -(-feval(func,xh1,varargin{:})-feval(func,xh_1,varargin{:})+temp(:,i)+temp(:,j))./(2*h1(i)*h_1(j));
         fh1 = feval(func,xh1,varargin{:});
         fh_1 = feval(func,xh_1,varargin{:});
         %hmat(:,j) = -(-feval(func,xh1,varargin{:})-feval(func,xh_1,varargin{:})+temp(:,i)+temp(:,j))./(2*h1(i)*h1(j));
         hmat(:,j) = -(-fh1-fh_1+temp(:,i)+temp(:,j))./(2*h1(i)*h1(j));
      end;
      hessian_mat(:,(i-1)*n+(1:i-1)) = hmat(:,1:i-1);
   end;
else;
   n = length(x);
   %
   % check if there may be a wait dialog
   %
   WaitDLGExist = 0;
   num = 0;
   StopEstimation = 0;
   if n>=25;
      WaitHandle = findobj('Type','Figure','Tag','WaitDLG');
      if IsHandle(WaitHandle)==1;
         WaitDLGExist = 1;
         WaitControls = get(WaitHandle,'UserData');
         TxtStr = vecstr(char(get(WaitControls.text,'String')),' ');
      end;
   end;
   %
   h1 = max(abs(x),sqrt(gstep)*ones(n,1))*eps^(1/6);
   %h_1 = h1;
   %
   f0 = feval(func,x,varargin{:});
   sf = size(f0,1);
   f1 = zeros(sf,n);
   f_1 = f1;
   xh1 = x;
   for i=1:n;
      if WaitDLGExist==1;
         if IsHandle(WaitControls.cancel)==1;
            if strcmp(get(WaitControls.cancel,'UserData'),'cancel')==1;
               StopEstimation = 1;
               break;
            end;
         end;
         set(WaitControls.text,'String',[TxtStr ' Computing diagonal element number: ' int2str(i) ' (of ' int2str(n) ').']);
      end;
      xh1 = x;
      xh1(i) = x(i)+h1(i);
      f1(:,i) = feval(func,xh1,varargin{:});
      %xh1(i) = x(i)-h_1(i);
      xh1(i) = x(i)-h1(i);
      f_1(:,i) = feval(func,xh1,varargin{:});
   end;
   if StopEstimation==1;
      hessian_mat = [];
      return;
   end;
   xh_1 = x;
   hessian_mat = sparse(sf,n*n);
   %
   % we move temp outside the for-loop since it's fixed
   %
   temp = f1+f_1-f0*ones(1,n);
   %hessian_mat(:,(1:n+1:n*n)) = (f1+f_1-2*f0*ones(1,n))./(h1.*h_1)';
   hessian_mat(:,(1:n+1:n*n)) = (f1+f_1-2*f0*ones(1,n))./(h1.*h1)';
   hmat = sparse(sf,n-1);
   %
   for i=2:n;
      %     if i > 1
      %         k=[i:n:n*(i-1)];
      %         hessian_mat(:,(i-1)*n+1:(i-1)*n+i-1)=hessian_mat(:,k);
      %         hessian_mat(:,k)=0;
      %     end
      for j=1:i-1;
         if WaitDLGExist==1;
            if IsHandle(WaitControls.cancel)==1;
               if strcmp(get(WaitControls.cancel,'UserData'),'cancel')==1;
                  StopEstimation = 1;
                  break;
               end;
            end;
            num = num+1;
            set(WaitControls.text,'String',[TxtStr ' Computing off-diagonal element: ' int2str(num) ' (of ' int2str(n*(n-1)/2) ').']);
         end;
         xh1 = x;
         xh_1 = x;
         xh1(i) = x(i)+h1(i);
         %xh1(j) = x(j)+h_1(j);
         xh1(j) = x(j)+h1(j);
         xh_1(i) = x(i)-h1(i);
         %xh_1(j) = x(j)-h_1(j);
         xh_1(j) = x(j)-h1(j);
         %hmat(:,j) = -(-feval(func,xh1,varargin{:})-feval(func,xh_1,varargin{:})+temp(:,i)+temp(:,j))./(2*h1(i)*h_1(j));
         fh1 = feval(func,xh1,varargin{:});
         fh_1 = feval(func,xh_1,varargin{:});
         %hmat(:,j) = -(-feval(func,xh1,varargin{:})-feval(func,xh_1,varargin{:})+temp(:,i)+temp(:,j))./(2*h1(i)*h1(j));
         hmat(:,j) = -(-fh1-fh_1+temp(:,i)+temp(:,j))./(2*h1(i)*h1(j));
      end;
      if StopEstimation==1;
         hessian_mat = [];
         break;
      end;
      hessian_mat(:,(i-1)*n+(1:i-1)) = hmat(:,1:i-1);
   end;
end;

%
% end of YADAHessianMatrix.m
%
