function [CRPSMatrix,BackCRPS] = CRPSNormalDistribution(YData,PredY,CovPred,BackYData,BNPredictY,BNCov)
% CRPSNormalDistribution: Compute the CRPS for individual variables using the predictive mean and standard deviation
%                         and a normal distribution.
%
% USAGE:
%
%       [CRPSMatrix,BackCRPS] = CRPSNormalDistribution(YData,PredY,CovPred,BackYData,BNPredictY,BNCov)
%
% REQUIRED INPUT:  YData (matrix) of dimension nxh with the actual values of the n observed variables over the h forecast periods.
%
%                  PredY (matrix) of dimension nxh with the mean predictions of the n observed variables.
%
%                  CovPred (matrix) of dimension nxnxh with the predtive covariance matrices for the n observed variables.
%
%                  BackYData (matrix) of dimension nx2 (or empty) of the observed values of the backcast and nowcast of the
%                            n observed variables.
%
%                  BNPredictY (matrix) of dimension nx2 (or empty) of the predicted mean backcast and nowcast of the n observed
%                            variables.
%
%                  BNCov (matrix) of dimension nxnx2 (or empty) of the predicted covariance backcast and nowcast of the n
%                            observed variables.
%
% REQUIRED OUTPUT: CRPSMatrix (matrix) of dimension nxh with the CRPS values for the n observed variables over the
%                            forecast sample.
%
%                  BackCRPS (matrix) of dimension nx2 with the CRPS values for the backcast and nowcast of the n observed
%                            variables.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: June 11, 2024.
%                        This version: Auguust 1, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 12-06-2024: Updated the documentation.
%
% * 01-08-2024: The conditional forecasts are now supported such that sigma may be zero and the
%               corresponding element of CRPSMatrix should then be NaN.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
[n,h] = size(YData);
CRPSMatrix = zeros(n,h);
if isempty(BackYData)==0;
   BackCRPS = zeros(size(BackYData));
else;
   BackCRPS = [];
end;
%
% start with the forecasts
%
for i=1:n;
   for j=1:h;
      y = YData(i,j);
      mu = PredY(i,j);
      sigma = sqrt(CovPred(i,i,j));
      if abs(sigma)>1.0e-06;
         x = (y-mu)/sigma;
         CRPSMatrix(i,j) = -sigma*((x*((2*PhiFunction(x))-1)) + (2*exp(logNormalPDF(x,0,1))) - 1/sqrt(pi));
      else;
         CRPSMatrix(i,j) = NaN;
      end;
   end;
end;
%
% and now the backcast
%
if isempty(BackYData)==0;
   k = size(BackYData,2);
   for i=1:n;
      for j=1:k;
         y = BackYData(i,j);
         mu = BNPredictY(i,j);
         sigma = BNCov(i,i,j);
         if abs(sigma)>1.0e-06;
            x = (y-mu)/sigma;
            BackCRPS(i,j) = -sigma*((x*((2*PhiFunction(x))-1)) + (2*exp(logNormalPDF(x,0,1))) - 1/sqrt(pi));
         else;
            BackCRPS(i,j) = NaN;
         end;
      end;
   end;
end;

%
% end of CRPSNormalDistribution.m
%
