function DoneCalc = DSGECondPredictiveMomentsMixed(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,h,TotalDraws,CurrChain,IsPosterior,Weights,SmpStr,ComputePITMoment)
% DSGECondPredictiveMomentsMixed: Computes the conditional predictive mean and covariance matrix under the mixed/subset approach for
%                                 a DSGE model with rational expectations.
%
% USAGE:
%
%       DoneCalc = DSGECondPredictiveMomentsMixed(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,h,TotalDraws,CurrChain,IsPosterior,Weights,SmpStr,ComputePITMoment)
%
% REQUIRED INPUT:  thetaPostSample (matrix) of dimension NumDrawsxNumParam with draws from the posterior or prior
%                                  distribution of the original DSGE model parameters.
%
%                  thetaMode (vector) with the posterior mode or initial value of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                  needs the former field to know which parameter name corresponds to
%                                  position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                  DSGE model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  X (matrix) of dimension kxh with the observations of the exogenous variables for
%                                  the prediction paths.
%
%                  Z (matrix) of dimension mxg with the out-of-sample data we shall condition the
%                                  predictions on.
%
%                  U (matrix) of dimension mxg with initial values for the mapping between the observed
%                                  data and the conditioning data.
%
%                  h (integer) with the prediction horizon.
%
%                  TotalDraws (integer) with the total number of parameter draws that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                  if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                                 likelihood tempering posterior sampler.
%
%                  SmpStr (string) which reflects the first and last period in the prediction sample.
%
%                  ComputePITMoment (boolean) that takes the value 1 if moments useable for the PITs should
%                                 be computed and 0 otherwise.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: July 31, 2024.
%                        This version: August 1, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 01-08-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DoneCalc = 0;
[NumDraws,NumParam] = size(thetaPostSample);
k = DSGEModel.k;
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   else;
      SetLogText('Missing AIM data file. Calculations aborted.');
   end;
   return;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% try to solve the model for the posterior mode estimates
%
if MatlabNumber>=8.0;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch ME;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
   else;
      SetLogText('Error when trying to solve the DSGE model. Calculations aborted.');
   end;
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   if isempty(CurrINI.Sequence)==1;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
      About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
   else;
      SetLogText(strvcat(['The ' SolverStr ' solver provided the return code:'],mcodeStr,'Calculations aborted.'));
   end;
   return;
end;
if length(size(H))==3;
   if size(H,3)<DSGEModel.LastPeriod+h;
      %
      % too few data points in the 3D H matrix.
      %
      if isempty(CurrINI.Sequence)==1;
         ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations plus forecasts (T+h = ' int2str(DSGEModel.LastPeriod+h) '). YADA has therefore aborted from the marginal predictive moments.'];
         About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      else;
         SetLogText(strvcat('Number of time periods for H(t) is less than the number of observations.','Calculations aborted.'));
      end;
      return;
   end;
end;
[B0Alt,KeepShocks] = RemoveRedundantColumns(B0);
%
% set up the data matrices
%
Y = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
if isempty(DSGEModel.X)==0;
   Xdata = DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
else;
   Xdata = [];
end;
%
% check B0 to make sure that it is not empty
%
if isempty(KeepShocks)==1;
   txt = 'All economic shocks are redundant, i.e., the B0 matrix is a zero matrix. YADA is therefore unable to calculate any conditional forecasts.';
   About(txt,'information','YADA - Shock Selection',500,600,CurrINI);
   return;
end;
%
% determine dimensions
%
[r,q] = size(B0);
q_r = length(DSGEModel.MixedConditionalShocks);
n = DSGEModel.n;
[qm,g] = size(Z);
%
% create the matrices for selecting shocks
%
N = eye(q);
M = N(:,DSGEModel.MixedConditionalShocks);
Morth = N(:,setdiff((1:q),DSGEModel.MixedConditionalShocks));
IgM = kron(eye(g),M);
IgMorth = kron(eye(g),Morth);
IgMMorth = kron(eye(g),Morth*Morth');
%
% create the mapping matrices for conditioning variables
%
K1 = DSGEModel.K1(:,DSGEModel.ConditionalVariables);
if length(size(DSGEModel.K2))==2;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables);
   K2length = 2;
else;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables,:);
   K2length = 3;
end;
SCond = (isequal(U,zeros(size(U)))&isequal(K2,zeros(size(K2))));
if (SCond==1)&(K2length==3);
   K2 = K2(:,:,1);
   K2length = 2;
end;
if (DSGEModel.KsiUseCondData==1)&(SCond==1);
   if (k>0);
      XPred = X(:,1:g);
   else;
      XPred = [];
   end;
else;
   XPred = [];
end;
%
% check if shocks can be used
%
if length(size(H))==2;
   if rank(K1'*H'*B0*M)<qm;
      txt = ['The conditioning assumptions you have selected cannot be used due to a singularity in the solution algorithm. Estimation of the marginal predictive moments has therfore been aborted.'];
      About(txt,'error','Error - Conditional Shocks',150,500,CurrINI);
      PredPaths = [];
      return;
   end;
else;
   BadEtaMat = 0;
   for i=1:g;
      if rank(K1'*H(:,:,DSGEModel.LastPeriod+i)'*B0*M)<qm;
         txt = ['The conditioning assumptions you have selected cannot be used due to a singularity in the solution algorithm. Estimation of the marginal predictive moments has therfore been aborted.'];
         About(txt,'error','Error - Conditional Shocks',150,500,CurrINI);
         PredPaths = [];
         BadEtaMat = 1;
         break;
      end;
   end;
   if BadEtaMat==1;
      return;
   end;
end;
%
% stack certain matrices and vector for the shock distribution
%
K = zeros(n*g,qm*g);
ZTg = zeros(qm*g,1);
UT = zeros(qm*g,1);
WTg = zeros(n*g,1);
for i=1:g;
   Ind = (1+((g-i)*n):(g-i+1)*n);
   Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
   K(Ind,Ind2) = K1;
   for j=1:i-1;
      Ind3 = Ind+(j*n);
      if K2length==2;
         K(Ind3,Ind2) = K2;
      else;
         if j<=size(K2,3);
            K(Ind3,Ind2) = K2(:,:,j);
         end;
      end;
   end;
   ZTg(Ind2,1) = Z(:,i);
   UT(Ind2,1) = U(:,g);
end;
%
% create a vector with positions for the conditioning assumptions that should be used
%
PosVec = (1:qm*g)' .* ~isnan(ZTg);
PosVec = PosVec(PosVec>0);
%
% adjust for missing restrictions
%
K = K(:,PosVec);
ZTg = ZTg(PosVec,1);
UT = UT(PosVec,1);
%
% compute the predictive mean and covariance using the fixed parameters
%
[logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,Xdata,1,size(Y,2),A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
T = size(Ksitt1,2);
if (DSGEModel.KsiUseCondData==1)&(SCond==1);
   if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
      Yerror = Yerror(:,T);
      SigmaSqRoot = SigmaSqRoot(:,:,T);
      InvMSEY = InvMSEY(:,:,T);
      KalmanGain = KalmanGain(:,:,T);
   end;
   [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
   [KsiTTz,PTTz] = CondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),KsitT(:,1),PtT(:,:,1),Yhat(:,T),DSGEModel.Y(:,DSGEModel.LastPeriod),XPred,Z,K1,A,H,R,F,B0,DSGEModel.KalmanAlgorithm);
   KsiTT = KsitT(:,1);
elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
   if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
      Yerror = Yerror(:,T-g+1:T);
      SigmaSqRoot = SigmaSqRoot(:,:,T-g+1:T);
      InvMSEY = InvMSEY(:,:,T-g+1:T);
      KalmanGain = KalmanGain(:,:,T-g+1:T);
   end;
   [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T-g+1:T),1,g,Yhat(:,T-g+1:T),Ksitt1(:,T-g+1:T),Ptt1(:,:,T-g+1:T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
   if rank(R)>0;
      if k>0;
         wtT = Y(:,T-g+1:T)-(A'*Xdata(:,T-g+1:T))-(H'*KsitT);
      else;
         wtT = Y(:,T-g+1:T)-(H'*KsitT);
      end;
   else;
      wtT = zeros(n,g);
   end;
   %
   % Smooth estimation of the states at time T using the conditioning assumptions.
   %
   if k>0;
      if g>1;
         Xtra = Xdata(:,T-g+2:T);
      else;
         Xtra = [];
      end;
      [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H,R,F,B0);
   else;
      [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H,R,F,B0);
   end;
   KsiTT = KsitT(:,g);
else;
   if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
      Yerror = Yerror(:,T);
      SigmaSqRoot = SigmaSqRoot(:,:,T);
      InvMSEY = InvMSEY(:,:,T);
      KalmanGain = KalmanGain(:,:,T);
   end;
   [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
   KsiTTz = KsitT(:,1);
   PTTz = PtT(:,:,1);
   KsiTT = KsitT(:,1);
end;
PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
%
% store fixed parameter values
%
Amode = A;
Hmode = H;
Rmode = R;
Fmode = F;
B0mode = B0;
KsiTTzmode = KsiTTz;
PTTzmode = PTTz;
%
% initialize
%
StateCov = zeros(n,n,h);
ShockCov = StateCov;
MeasureCov = zeros(n,n,h);
ParameterCov = StateCov;
MeanPredictY = zeros(n,h);
PITMoments = [];
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% check if we should use parallel computations or not
%
if CurrINI.NumWorkers<2;
   %
   % check if we should setup a wait handle
   %
   if DSGEModel.ShowProgress==1;
      ProgressStructure.title = ['Progress for ' int2str(NumDraws) ' parameter draws'];
      ProgressStructure.facecolor = CurrINI.progress_facecolor;
      ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
      ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
      ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
      ProgressStructure.stop = 0;
      ProgressStructure.clock = DSGEModel.ShowProgressClock;
      ProgressStructure.label = 'Draw Number:';
      %
      ProgressStructure.name = ['Marginal Predictive Moments' PriorHeader ' - ' int2str(h) ' Periods Ahead'];
      ProgressStructure.CurrINI = CurrINI;
      WaitHandle = ProgressDLG(0,ProgressStructure);
      set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
      drawnow;
   else;
      txt = ['Please wait while YADA computes marginal predictive moments for the original data. Computations started at: ' StringTrim(datestr(now,14))];
      WaitHandle = WaitDLG(txt,'information',['Marginal Predictive Moments ' PriorHeader ' - ' int2str(h) ' Periods Ahead'],600,150,CurrINI,0);
      WaitControls = get(WaitHandle,'UserData');
   end;
   %
   % begin the loop
   %
   abort = '';
   StopEstimation = 0;
   ZPos = ~isnan(Z);
   if length(size(H))==2;
      for ParamIndex=1:NumDraws;
         if DSGEModel.ShowProgress==1;
            abort = get(WaitHandle,'UserData');
            if strcmp(abort,'cancel')==1;
               StopEstimation = 1;
               break;
            else;
               ProgressDLG([ParamIndex/NumDraws ParamIndex]);
            end;
         end;
         %
         % Solve the model for the current parameter vector
         %
         theta = thetaPostSample(ParamIndex,:)';
         [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         if status==1;
            %
            % check invertibility
            %
            etaMat = K1'*H'*B0;
            if rank(etaMat)==qm;
               %
               % We can now run the Kalman filter
               %
               [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,Xdata,1,size(Y,2),A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
               %
               % Now we need to calculate KsiTTz and PTTz for just the last period
               %
               if (DSGEModel.KsiUseCondData==1)&(SCond==1);
                  if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
                     Yerror = Yerror(:,T);
                     SigmaSqRoot = SigmaSqRoot(:,:,T);
                     InvMSEY = InvMSEY(:,:,T);
                     KalmanGain = KalmanGain(:,:,T);
                  end;
                  [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
                  [KsiTTz,PTTz] = CondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),KsitT(:,1),PtT(:,:,1),Yhat(:,T),DSGEModel.Y(:,DSGEModel.LastPeriod),XPred,Z,K1,A,H,R,F,B0,DSGEModel.KalmanAlgorithm);
                  KsiTT = KsitT(:,1);
               elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
                  if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
                     Yerror = Yerror(:,T-g+1:T);
                     SigmaSqRoot = SigmaSqRoot(:,:,T-g+1:T);
                     InvMSEY = InvMSEY(:,:,T-g+1:T);
                     KalmanGain = KalmanGain(:,:,T-g+1:T);
                  end;
                  [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T-g+1:T),1,g,Yhat(:,T-g+1:T),Ksitt1(:,T-g+1:T),Ptt1(:,:,T-g+1:T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
                  if rank(R)>0;
                     if k>0;
                        wtT = Y(:,T-g+1:T)-(A'*Xdata(:,T-g+1:T))-(H'*KsitT);
                     else;
                        wtT = Y(:,T-g+1:T)-(H'*KsitT);
                     end;
                  else;
                     wtT = zeros(n,g);
                  end;
                  %
                  % Smooth estimation of the states at time T using the conditioning assumptions.
                  %
                  if k>0;
                     if g>1;
                        Xtra = Xdata(:,T-g+2:T);
                     else;
                        Xtra = [];
                     end;
                     [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H,R,F,B0);
                  else;
                     [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H,R,F,B0);
                  end;
                  KsiTT = KsitT(:,g);
               else;
                  if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
                     Yerror = Yerror(:,T);
                     SigmaSqRoot = SigmaSqRoot(:,:,T);
                     InvMSEY = InvMSEY(:,:,T);
                     KalmanGain = KalmanGain(:,:,T);
                  end;
                  [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
                  KsiTTz = KsitT(:,1);
                  PTTz = PtT(:,:,1);
                  KsiTT = KsitT(:,1);
               end;
               PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
            else;
               %
               % use posterior mode or initial values as a backup
               %
               A = Amode;
               H = Hmode;
               R = Rmode;
               F = Fmode;
               B0 = B0mode;
               KsiTTz = KsiTTzmode;
               PTTz = PTTzmode;
            end;
         else;
            %
            % use posterior mode or initial values as a backup
            %
            A = Amode;
            H = Hmode;
            R = Rmode;
            F = Fmode;
            B0 = B0mode;
            KsiTTz = KsiTTzmode;
            PTTz = PTTzmode;
         end;
         %
         % stack the model to determine the shock distribution
         %
         Xbar = zeros(n*g,1);
         G = zeros(n*g,r);
         D = zeros(n*g,q*g);
         Fexp = F;
         Q = B0*B0';
         QM = B0*M*M'*B0';
         Fg = eye(r);
         for i=1:g;
            Ind = (1+((g-i)*n):(g-i+1)*n);
            Ind1 = (1+((g-i)*q):(g-i+1)*q);
            if k>0;
               Xbar(Ind,1) = A'*X(:,i);
            end;
            G(Ind,:) = H'*Fexp;
            D(Ind,Ind1) = H'*B0;
            Fi = F;
            for j=1:i-1;
               Ind1 = Ind1+q;
               D(Ind,Ind1) = H'*Fi*B0;
               Fi = F*Fi;
            end;
            Fexp = F*Fexp;
         end;
         %
         % speed-up code suggested by Ingvar Strid
         %
         KpD = K'*D*IgM;
         KpDDpK = KpD*KpD';
         DpKInvKpDDpK = KpD'/KpDDpK;
         %
         % compute the conditioning relation when state vector and measurement errors are at mean
         %
         kbarTg = ZTg-UT-(K'*(Xbar+(G*KsiTTz)));
         %
         % compute the mean of the shocks when state vector and measurement errors are at mean
         %
         mubarTg = DpKInvKpDDpK*kbarTg;
         %
         % compute the idempotent covariance matrix for the shocks
         %
         SigmaTg = eye(q_r*g)-(DpKInvKpDDpK*KpD);
         SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
         Dbar = eye(n*g)-(D*IgM*DpKInvKpDDpK*K');
         CovShock = (D*IgM*SigmaTg*IgM'*D')+(Dbar*D*IgMMorth*D'*Dbar');
         CovState = Dbar*G*PTTz*G'*Dbar';
         if rank(R)>0;
            IgR = kron(eye(g),R);
            CovME = Dbar*IgR*Dbar';
         else;
            CovME = 0;
         end;
         Bbar = zeros(r,q*g);
         %
         % compute the mean prediction at the given parameter values
         %
         PredMean = zeros(n,h);
         KsiTm = KsiTTz;
         etaShocksm = zeros(q,h);
         etaShocksm(DSGEModel.MixedConditionalShocks,1:g) = fliplr(reshape(mubarTg,q_r,g));
         for i=1:g;
            %
            % update the states
            %
            KsiTm = (F*KsiTm)+(B0*etaShocksm(:,i));
            %
            % compute the conditional mean predictions
            %
            if k>0;
               PredMean(:,i) = (A'*X(:,i))+(H'*KsiTm);
            else;
               PredMean(:,i) = H'*KsiTm;
            end;
            Ind1 = (1+((g-i)*n):(g-i+1)*n);
            Ind2 = (1+(i-1)*q):(i*q);
            StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovState(Ind1,Ind1))))+tril(CovState(Ind1,Ind1),-1)+tril(CovState(Ind1,Ind1),-1)'));
            ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovShock(Ind1,Ind1))))+tril(CovShock(Ind1,Ind1),-1)+tril(CovShock(Ind1,Ind1),-1)'));
            if rank(R)>0;
               MeasureCov(:,:,i) = MeasureCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovME(Ind1,Ind1))))+tril(CovME(Ind1,Ind1),-1)+tril(CovME(Ind1,Ind1),-1)'));
            end;
            Bbar(:,Ind2) = Fg*B0;
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)');
            Fg = Fg*F;
            if ComputePITMoment==1;
               if rank(R)>0;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = CovState(Ind1,Ind1)+CovShock(Ind1,Ind1)+CovME(Ind1,Ind1);
               else;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = CovState(Ind1,Ind1)+CovShock(Ind1,Ind1);
               end;
            end;
         end;
         for i=g+1:h;
            %
            % update the states
            %
            KsiTm = (F*KsiTm);
            %
            % compute the conditional mean predictions
            %
            if k>0;
               PredMean(:,i) = (A'*X(:,i))+(H'*KsiTm);
            else;
               PredMean(:,i) = H'*KsiTm;
            end;
            if i==(g+1);
               Kbar = Bbar*IgM*DpKInvKpDDpK*K';
               Gbar = Fg-(Kbar*G);
               Nbar = eye(q*g)-(IgM*DpKInvKpDDpK*K'*D);
               Fig = eye(r);
               FjSum = zeros(r,r);
            end;
            FjSum = FjSum+(Fig*Q*Fig');
            Fig = Fig*F;
            StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*Fig*Gbar*PTTz*Gbar'*Fig'*H);
            %
            % compute the shock covariance
            %
            ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*(FjSum+(Fig*Bbar*((IgM*SigmaTg*IgM')+(Nbar*IgMMorth*Nbar'))*Bbar'*Fig'))*H);
            if rank(R)>0;
               MeasureCov(:,:,i) = MeasureCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*((H'*Fig*Kbar*IgR*Kbar'*Fig'*H)+R));
            end;
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)');
            if ComputePITMoment==1;
               if rank(R)>0;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = (H'*Fig*Gbar*PTTz*Gbar'*Fig'*H)+(H'*(FjSum+(Fig*Bbar*((IgM*SigmaTg*IgM')+(Nbar*IgMMorth*Nbar'))*Bbar'*Fig'))*H)+((H'*Fig*Kbar*IgR*Kbar'*Fig'*H)+R);
               else;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = (H'*Fig*Gbar*PTTz*Gbar'*Fig'*H)+(H'*(FjSum+(Fig*Bbar*((IgM*SigmaTg*IgM')+(Nbar*IgMMorth*Nbar'))*Bbar'*Fig'))*H);
               end;
            end;
         end;
         MeanPredictY = MeanPredictY+((Weights(ParamIndex)/NumDraws)*PredMean);
         if ComputePITMoment==1;
            PITMoments(ParamIndex).PredictY = PredMean;
         end;
         drawnow;
      end;
   else;
      %
      % a time-varying H-matrix
      %
      for ParamIndex=1:NumDraws;
         if DSGEModel.ShowProgress==1;
            abort = get(WaitHandle,'UserData');
            if strcmp(abort,'cancel')==1;
               StopEstimation = 1;
               break;
            else;
               ProgressDLG([ParamIndex/NumDraws ParamIndex]);
            end;
         end;
         %
         % Solve the model for the current parameter vector
         %
         theta = thetaPostSample(ParamIndex,:)';
         [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         if status==1;
            %
            % check invertibility
            %
            BadRank = 0;
            for i=1:g;
               etaMat = K1'*H(:,:,DSGEModel.LastPeriod+i)'*B0*M;
               if rank(etaMat)<qm;
                  BadRank = 1;
                  break;
               end;
            end;
            if BadRank==0;
               %
               % We can now run the Kalman filter
               %
               [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,Xdata,1,size(Y,2),A,H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod),F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
               %
               % Now we need to calculate KsiTTz and PTTz for just the last period
               %
               if (DSGEModel.KsiUseCondData==1)&(SCond==1);
                  if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
                     Yerror = Yerror(:,T);
                     SigmaSqRoot = SigmaSqRoot(:,:,T);
                     InvMSEY = InvMSEY(:,:,T);
                     KalmanGain = KalmanGain(:,:,T);
                  end;
                  [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H(:,:,DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
                  [KsiTTz,PTTz] = CondPredictionSmootherHt(Ksitt1(:,T),Ptt1(:,:,T),KsitT(:,1),PtT(:,:,1),Yhat(:,T),DSGEModel.Y(:,DSGEModel.LastPeriod),XPred,Z,K1,A,H(:,:,DSGEModel.LastPeriod:DSGEModel.LastPeriod+g),R,F,B0,DSGEModel.KalmanAlgorithm);
                  KsiTT = KsitT(:,1);
               elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
                  if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
                     Yerror = Yerror(:,T-g+1:T);
                     SigmaSqRoot = SigmaSqRoot(:,:,T-g+1:T);
                     InvMSEY = InvMSEY(:,:,T-g+1:T);
                     KalmanGain = KalmanGain(:,:,T-g+1:T);
                  end;
                  [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T-g+1:T),1,g,Yhat(:,T-g+1:T),Ksitt1(:,T-g+1:T),Ptt1(:,:,T-g+1:T),H(:,:,DSGEModel.LastPeriod-g+1:DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
                  wtT = zeros(n,g);
                  if rank(R)>0;
                     if k>0;
                        for i=1:g;
                           wtT(:,i) = Y(:,i)-(A'*Xdata(:,i))-(H(:,:,DSGEModel.LastPeriod-g+i)'*KsitT(:,i));
                        end;
                     else;
                        for i=1:g;
                           wtT(:,i) = Y(:,i)-(H(:,:,DSGEModel.LastPeriod-g+i)'*KsitT(:,i));
                        end;
                     end;
                  else;
                     wtT = zeros(n,g);
                  end;
                  %
                  % Smooth estimation of the states at time T using the conditioning assumptions.
                  %
                  if k>0;
                     if g>1;
                        Xtra = Xdata(:,T-g+2:T);
                     else;
                        Xtra = [];
                     end;
                     [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H(:,:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod+g),R,F,B0);
                  else;
                     [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H(:,:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod+g),R,F,B0);
                  end;
                  KsiTT = KsitT(:,g);
               else;
                  if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
                     Yerror = Yerror(:,T);
                     SigmaSqRoot = SigmaSqRoot(:,:,T);
                     InvMSEY = InvMSEY(:,:,T);
                     KalmanGain = KalmanGain(:,:,T);
                  end;
                  [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H(:,:,DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
                  KsiTTz = KsitT(:,1);
                  PTTz = PtT(:,:,1);
                  KsiTT = KsitT(:,1);
               end;
               PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
            else;
               %
               % use posterior mode or initial values as a backup
               %
               A = Amode;
               H = Hmode;
               R = Rmode;
               F = Fmode;
               B0 = B0mode;
               KsiTTz = KsiTTzmode;
               PTTz = PTTzmode;
            end;
         else;
            %
            % use posterior mode or initial values as a backup
            %
            A = Amode;
            H = Hmode;
            R = Rmode;
            F = Fmode;
            B0 = B0mode;
            KsiTTz = KsiTTzmode;
            PTTz = PTTzmode;
         end;
         %
         % stack the model to determine the shock distribution
         %
         Xbar = zeros(n*g,1);
         G = zeros(n*g,r);
         D = zeros(n*g,q*g);
         Fexp = F;
         Q = B0*B0';
         QM = B0*M*M'*B0';
         Fg = eye(r);
         for i=1:g;
            Ind = (1+((g-i)*n):(g-i+1)*n);
            Ind1 = (1+((g-i)*q):(g-i+1)*q);
            if k>0;
               Xbar(Ind,1) = A'*X(:,i);
            end;
            G(Ind,:) = H(:,:,DSGEModel.LastPeriod+i)'*Fexp;
            D(Ind,Ind1) = H(:,:,DSGEModel.LastPeriod+i)'*B0;
            Fi = F;
            for j=1:i-1;
               Ind1 = Ind1+q;
               D(Ind,Ind1) = H(:,:,DSGEModel.LastPeriod+i)'*Fi*B0;
               Fi = F*Fi;
            end;
            Fexp = F*Fexp;
         end;
         %
         % speed-up code suggested by Ingvar Strid
         %
         KpD = K'*D*IgM;
         KpDDpK = KpD*KpD';
         DpKInvKpDDpK = KpD'/KpDDpK;
         %
         % compute the conditioning relation when state vector and measurement errors are at mean
         %
         kbarTg = ZTg-UT-(K'*(Xbar+(G*KsiTTz)));
         %
         % compute the mean of the shocks when state vector and measurement errors are at mean
         %
         mubarTg = DpKInvKpDDpK*kbarTg;
         %
         % compute the idempotent covariance matrix for the shocks
         %
         SigmaTg = eye(q_r*g)-(DpKInvKpDDpK*KpD);
         SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
         Dbar = eye(n*g)-(D*IgM*DpKInvKpDDpK*K');
         CovShock = (D*IgM*SigmaTg*IgM'*D')+(Dbar*D*IgMMorth*D'*Dbar');
         CovState = Dbar*G*PTTz*G'*Dbar';
         if rank(R)>0;
            IgR = kron(eye(g),R);
            CovME = Dbar*IgR*Dbar';
         else;
            CovME = 0;
         end;
         Bbar = zeros(r,q*g);
         %
         % compute the mean prediction at the given parameter values
         %
         PredMean = zeros(n,h);
         KsiTm = KsiTTz;
         etaShocksm = zeros(q,h);
         etaShocksm(DSGEModel.MixedConditionalShocks,1:g) = fliplr(reshape(mubarTg,q_r,g));
         for i=1:g;
            %
            % update the states
            %
            KsiTm = (F*KsiTm)+(B0*etaShocksm(:,i));
            %
            % compute the conditional mean predictions
            %
            if k>0;
               PredMean(:,i) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiTm);
            else;
               PredMean(:,i) = H(:,:,DSGEModel.LastPeriod+i)'*KsiTm;
            end;
            Ind1 = (1+((g-i)*n):(g-i+1)*n);
            Ind2 = (1+(i-1)*q):(i*q);
            StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovState(Ind1,Ind1))))+tril(CovState(Ind1,Ind1),-1)+tril(CovState(Ind1,Ind1),-1)'));
            ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovShock(Ind1,Ind1))))+tril(CovShock(Ind1,Ind1),-1)+tril(CovShock(Ind1,Ind1),-1)'));
            if rank(R)>0;
               MeasureCov(:,:,i) = MeasureCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovME(Ind1,Ind1))))+tril(CovME(Ind1,Ind1),-1)+tril(CovME(Ind1,Ind1),-1)'));
            end;
            Bbar(:,Ind2) = Fg*B0;
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)');
            Fg = Fg*F;
            if ComputePITMoment==1;
               if rank(R)>0;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = CovState(Ind1,Ind1)+CovShock(Ind1,Ind1)+CovME(Ind1,Ind1);
               else;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = CovState(Ind1,Ind1)+CovShock(Ind1,Ind1);
               end;
            end;
         end;
         for i=g+1:h;
            %
            % update the states
            %
            KsiTm = (F*KsiTm);
            %
            % compute the conditional mean predictions
            %
            if k>0;
               PredMean(:,i) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiTm);
            else;
               PredMean(:,i) = H(:,:,DSGEModel.LastPeriod+i)'*KsiTm;
            end;
            if i==(g+1);
               Kbar = Bbar*IgM*DpKInvKpDDpK*K';
               Gbar = Fg-(Kbar*G);
               Nbar = eye(q*g)-(IgM*DpKInvKpDDpK*K'*D);
               Fig = eye(r);
               FjSum = zeros(r,r);
            end;
            FjSum = FjSum+(Fig*Q*Fig');
            Fig = Fig*F;
            StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,DSGEModel.LastPeriod+i)'*Fig*Gbar*PTTz*Gbar'*Fig'*H(:,:,DSGEModel.LastPeriod+i));
            %
            % compute the shock covariance
            %
            ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,DSGEModel.LastPeriod+i)'*(FjSum+(Fig*Bbar*((IgM*SigmaTg*IgM')+(Nbar*IgMMorth*Nbar'))*Bbar'*Fig'))*H(:,:,DSGEModel.LastPeriod+i));
            if rank(R)>0;
               MeasureCov(:,:,i) = MeasureCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*((H(:,:,DSGEModel.LastPeriod+i)'*Fig*Kbar*IgR*Kbar'*Fig'*H(:,:,DSGEModel.LastPeriod+i))+R));
            end;
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)');
            if ComputePITMoment==1;
               if rank(R)>0;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = (H(:,:,DSGEModel.LastPeriod+i)'*Fig*Gbar*PTTz*Gbar'*Fig'*H(:,:,DSGEModel.LastPeriod+i))+(H(:,:,DSGEModel.LastPeriod+i)'*(FjSum+(Fig*Bbar*((IgM*SigmaTg*IgM')+(Nbar*IgMMorth*Nbar'))*Bbar'*Fig'))*H(:,:,DSGEModel.LastPeriod+i))+((H(:,:,DSGEModel.LastPeriod+i)'*Fig*Kbar*IgR*Kbar'*Fig'*H(:,:,DSGEModel.LastPeriod+i))+R);
               else;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = (H(:,:,DSGEModel.LastPeriod+i)'*Fig*Gbar*PTTz*Gbar'*Fig'*H(:,:,DSGEModel.LastPeriod+i))+(H(:,:,DSGEModel.LastPeriod+i)'*(FjSum+(Fig*Bbar*((IgM*SigmaTg*IgM')+(Nbar*IgMMorth*Nbar'))*Bbar'*Fig'))*H(:,:,DSGEModel.LastPeriod+i));
               end;
            end;
         end;
         MeanPredictY = MeanPredictY+((Weights(ParamIndex)/NumDraws)*PredMean);
         if ComputePITMoment==1;
            PITMoments(ParamIndex).PredictY = PredMean;
         end;
         drawnow;
      end;
   end;
   %
   % close the wait dialog
   %
   if DSGEModel.ShowProgress==1;
      if IsHandle(WaitHandle)==1;
         set(WaitHandle,'UserData','done');
         close(WaitHandle);
         drawnow;
         pause(0.02);
      end;
   else;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   %
   % we can do parallel computations over the parameter draws
   %
   txt = ['Please wait while YADA computes marginal predictive moments for the original data. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Marginal Predictive Moments ' PriorHeader ' - ' int2str(h) ' Periods Ahead'],600,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   %
   % initialize
   %
   StopEstimation = 0;
   MP = ModelParameters;
   Moments = [];
   ZPos = ~isnan(Z);
   %
   % begin the loop
   %
   if length(size(H))==2;
      parfor ParamIndex=1:NumDraws;
         %
         % initialization
         %
         ModelParameters = MP;
         Moments(ParamIndex).StateCov = StateCov;
         Moments(ParamIndex).ShockCov = ShockCov;
         Moments(ParamIndex).MeasureCov = MeasureCov;
         Moments(ParamIndex).ParameterCov = ParameterCov;
         Moments(ParamIndex).MeanPredictY = MeanPredictY;
         %
         % Solve the model for the current parameter vector
         %
         theta = thetaPostSample(ParamIndex,:)';
         [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         if status==1;
            %
            % check invertibility
            %
            etaMat = K1'*H'*B0;
            if rank(etaMat)==qm;
               %
               % We can now run the Kalman filter
               %
               [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,Xdata,1,size(Y,2),A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
               %
               % Now we need to calculate KsiTTz and PTTz for just the last period
               %
               if (DSGEModel.KsiUseCondData==1)&(SCond==1);
                  if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
                     Yerror = Yerror(:,T);
                     SigmaSqRoot = SigmaSqRoot(:,:,T);
                     InvMSEY = InvMSEY(:,:,T);
                     KalmanGain = KalmanGain(:,:,T);
                  end;
                  [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
                  [KsiTTz,PTTz] = CondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),KsitT(:,1),PtT(:,:,1),Yhat(:,T),DSGEModel.Y(:,DSGEModel.LastPeriod),XPred,Z,K1,A,H,R,F,B0,DSGEModel.KalmanAlgorithm);
                  KsiTT = KsitT(:,1);
               elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
                  if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
                     Yerror = Yerror(:,T-g+1:T);
                     SigmaSqRoot = SigmaSqRoot(:,:,T-g+1:T);
                     InvMSEY = InvMSEY(:,:,T-g+1:T);
                     KalmanGain = KalmanGain(:,:,T-g+1:T);
                  end;
                  [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T-g+1:T),1,g,Yhat(:,T-g+1:T),Ksitt1(:,T-g+1:T),Ptt1(:,:,T-g+1:T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
                  if rank(R)>0;
                     if k>0;
                        wtT = Y(:,T-g+1:T)-(A'*Xdata(:,T-g+1:T))-(H'*KsitT);
                     else;
                        wtT = Y(:,T-g+1:T)-(H'*KsitT);
                     end;
                  else;
                     wtT = zeros(n,g);
                  end;
                  %
                  % Smooth estimation of the states at time T using the conditioning assumptions.
                  %
                  if k>0;
                     if g>1;
                        Xtra = Xdata(:,T-g+2:T);
                     else;
                        Xtra = [];
                     end;
                     [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H,R,F,B0);
                  else;
                     [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H,R,F,B0);
                  end;
                  KsiTT = KsitT(:,g);
               else;
                  if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
                     Yerror = Yerror(:,T);
                     SigmaSqRoot = SigmaSqRoot(:,:,T);
                     InvMSEY = InvMSEY(:,:,T);
                     KalmanGain = KalmanGain(:,:,T);
                  end;
                  [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
                  KsiTTz = KsitT(:,1);
                  PTTz = PtT(:,:,1);
                  KsiTT = KsitT(:,1);
               end;
               PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
            else;
               %
               % use posterior mode or initial values as a backup
               %
               A = Amode;
               H = Hmode;
               R = Rmode;
               F = Fmode;
               B0 = B0mode;
               KsiTTz = KsiTTzmode;
               PTTz = PTTzmode;
            end;
         else;
            %
            % use posterior mode or initial values as a backup
            %
            A = Amode;
            H = Hmode;
            R = Rmode;
            F = Fmode;
            B0 = B0mode;
            KsiTTz = KsiTTzmode;
            PTTz = PTTzmode;
         end;
         %
         % stack the model to determine the shock distribution
         %
         Xbar = zeros(n*g,1);
         G = zeros(n*g,r);
         D = zeros(n*g,q*g);
         Fexp = F;
         Q = B0*B0';
         QM = B0*M*M'*B0';
         Fg = eye(r);
         for i=1:g;
            Ind = (1+((g-i)*n):(g-i+1)*n);
            Ind1 = (1+((g-i)*q):(g-i+1)*q);
            if k>0;
               Xbar(Ind,1) = A'*X(:,i);
            end;
            G(Ind,:) = H'*Fexp;
            D(Ind,Ind1) = H'*B0;
            Fi = F;
            for j=1:i-1;
               Ind1 = Ind1+q;
               D(Ind,Ind1) = H'*Fi*B0;
               Fi = F*Fi;
            end;
            Fexp = F*Fexp;
         end;
         %
         % speed-up code suggested by Ingvar Strid
         %
         KpD = K'*D*IgM;
         KpDDpK = KpD*KpD';
         DpKInvKpDDpK = KpD'/KpDDpK;
         %
         % compute the conditioning relation when state vector and measurement errors are at mean
         %
         kbarTg = ZTg-UT-(K'*(Xbar+(G*KsiTTz)));
         %
         % compute the mean of the shocks when state vector and measurement errors are at mean
         %
         mubarTg = DpKInvKpDDpK*kbarTg;
         %
         % compute the idempotent covariance matrix for the shocks
         %
         SigmaTg = eye(q_r*g)-(DpKInvKpDDpK*KpD);
         SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
         Dbar = eye(n*g)-(D*IgM*DpKInvKpDDpK*K');
         CovShock = (D*IgM*SigmaTg*IgM'*D')+(Dbar*D*IgMMorth*D'*Dbar');
         CovState = Dbar*G*PTTz*G'*Dbar';
         if rank(R)>0;
            IgR = kron(eye(g),R);
            CovME = Dbar*IgR*Dbar';
         else;
            CovME = 0;
         end;
         Bbar = zeros(r,q*g);
         %
         % compute the mean prediction at the given parameter values
         %
         PredMean = zeros(n,h);
         KsiTm = KsiTTz;
         etaShocksm = zeros(q,h);
         etaShocksm(DSGEModel.MixedConditionalShocks,1:g) = fliplr(reshape(mubarTg,q_r,g));
         for i=1:g;
            %
            % update the states
            %
            KsiTm = (F*KsiTm)+(B0*etaShocksm(:,i));
            %
            % compute the conditional mean predictions
            %
            if k>0;
               PredMean(:,i) = (A'*X(:,i))+(H'*KsiTm);
            else;
               PredMean(:,i) = H'*KsiTm;
            end;
            Ind1 = (1+((g-i)*n):(g-i+1)*n);
            Ind2 = (1+(i-1)*q):(i*q);
            Moments(ParamIndex).StateCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovState(Ind1,Ind1))))+tril(CovState(Ind1,Ind1),-1)+tril(CovState(Ind1,Ind1),-1)');
            Moments(ParamIndex).ShockCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovShock(Ind1,Ind1))))+tril(CovShock(Ind1,Ind1),-1)+tril(CovShock(Ind1,Ind1),-1)');
            if rank(R)>0;
               Moments(ParamIndex).MeasureCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovME(Ind1,Ind1))))+tril(CovME(Ind1,Ind1),-1)+tril(CovME(Ind1,Ind1),-1)');
            end;
            Bbar(:,Ind2) = Fg*B0;
            Moments(ParamIndex).ParameterCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)';
            Fg = Fg*F;
            if ComputePITMoment==1;
               if rank(R)>0;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = CovState(Ind1,Ind1)+CovShock(Ind1,Ind1)+CovME(Ind1,Ind1);
               else;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = CovState(Ind1,Ind1)+CovShock(Ind1,Ind1);
               end;
            end;
         end;
         for i=g+1:h;
            %
            % update the states
            %
            KsiTm = (F*KsiTm);
            %
            % compute the conditional mean predictions
            %
            if k>0;
               PredMean(:,i) = (A'*X(:,i))+(H'*KsiTm);
            else;
               PredMean(:,i) = H'*KsiTm;
            end;
            if i==(g+1);
               Kbar = Bbar*IgM*DpKInvKpDDpK*K';
               Gbar = Fg-(Kbar*G);
               Nbar = eye(q*g)-(IgM*DpKInvKpDDpK*K'*D);
               Fig = eye(r);
               FjSum = zeros(r,r);
            end;
            FjSum = FjSum+(Fig*Q*Fig');
            Fig = Fig*F;
            Moments(ParamIndex).StateCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H'*Fig*Gbar*PTTz*Gbar'*Fig'*H;
            %
            % compute the shock covariance
            %
            Moments(ParamIndex).ShockCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H'*(FjSum+(Fig*Bbar*((IgM*SigmaTg*IgM')+(Nbar*IgMMorth*Nbar'))*Bbar'*Fig'))*H;
            if rank(R)>0;
               Moments(ParamIndex).MeasureCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*((H'*Fig*Kbar*IgR*Kbar'*Fig'*H)+R);
            end;
            Moments(ParamIndex).ParameterCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)';
            if ComputePITMoment==1;
               if rank(R)>0;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = (H'*Fig*Gbar*PTTz*Gbar'*Fig'*H)+(H'*(FjSum+(Fig*Bbar*((IgM*SigmaTg*IgM')+(Nbar*IgMMorth*Nbar'))*Bbar'*Fig'))*H)+((H'*Fig*Kbar*IgR*Kbar'*Fig'*H)+R);
               else;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = (H'*Fig*Gbar*PTTz*Gbar'*Fig'*H)+(H'*(FjSum+(Fig*Bbar*((IgM*SigmaTg*IgM')+(Nbar*IgMMorth*Nbar'))*Bbar'*Fig'))*H);
               end;
            end;
         end;
         Moments(ParamIndex).MeanPredictY = (Weights(ParamIndex)/NumDraws)*PredMean;
         if ComputePITMoment==1;
            PITMoments(ParamIndex).PredictY = PredMean;
         end;
         drawnow;
      end;
   else;
      %
      % a time-varying H-matrix
      %
      parfor ParamIndex=1:NumDraws;
         %
         % initialization
         %
         ModelParameters = MP;
         Moments(ParamIndex).StateCov = StateCov;
         Moments(ParamIndex).ShockCov = ShockCov;
         Moments(ParamIndex).MeasureCov = MeasureCov;
         Moments(ParamIndex).ParameterCov = ParameterCov;
         Moments(ParamIndex).MeanPredictY = MeanPredictY;
         %
         % Solve the model for the current parameter vector
         %
         theta = thetaPostSample(ParamIndex,:)';
         [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         if status==1;
            %
            % check invertibility
            %
            BadRank = 0;
            for i=1:g;
               etaMat = K1'*H(:,:,DSGEModel.LastPeriod+i)'*B0*M;
               if rank(etaMat)<qm;
                  BadRank = 1;
                  break;
               end;
            end;
            if BadRank==0;
               %
               % We can now run the Kalman filter
               %
               [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,Xdata,1,size(Y,2),A,H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod),F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
               %
               % Now we need to calculate KsiTTz and PTTz for just the last period
               %
               if (DSGEModel.KsiUseCondData==1)&(SCond==1);
                  if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
                     Yerror = Yerror(:,T);
                     SigmaSqRoot = SigmaSqRoot(:,:,T);
                     InvMSEY = InvMSEY(:,:,T);
                     KalmanGain = KalmanGain(:,:,T);
                  end;
                  [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H(:,:,DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
                  [KsiTTz,PTTz] = CondPredictionSmootherHt(Ksitt1(:,T),Ptt1(:,:,T),KsitT(:,1),PtT(:,:,1),Yhat(:,T),DSGEModel.Y(:,DSGEModel.LastPeriod),XPred,Z,K1,A,H(:,:,DSGEModel.LastPeriod:DSGEModel.LastPeriod+g),R,F,B0,DSGEModel.KalmanAlgorithm);
                  KsiTT = KsitT(:,1);
               elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
                  if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
                     Yerror = Yerror(:,T-g+1:T);
                     SigmaSqRoot = SigmaSqRoot(:,:,T-g+1:T);
                     InvMSEY = InvMSEY(:,:,T-g+1:T);
                     KalmanGain = KalmanGain(:,:,T-g+1:T);
                  end;
                  [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T-g+1:T),1,g,Yhat(:,T-g+1:T),Ksitt1(:,T-g+1:T),Ptt1(:,:,T-g+1:T),H(:,:,DSGEModel.LastPeriod-g+1:DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
                  wtT = zeros(n,g);
                  if rank(R)>0;
                     if k>0;
                        for i=1:g;
                           wtT(:,i) = Y(:,i)-(A'*Xdata(:,i))-(H(:,:,DSGEModel.LastPeriod-g+i)'*KsitT(:,i));
                        end;
                     else;
                        for i=1:g;
                           wtT(:,i) = Y(:,i)-(H(:,:,DSGEModel.LastPeriod-g+i)'*KsitT(:,i));
                        end;
                     end;
                  else;
                     wtT = zeros(n,g);
                  end;
                  %
                  % Smooth estimation of the states at time T using the conditioning assumptions.
                  %
                  if k>0;
                     if g>1;
                        Xtra = Xdata(:,T-g+2:T);
                     else;
                        Xtra = [];
                     end;
                     [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H(:,:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod+g),R,F,B0);
                  else;
                     [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H(:,:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod+g),R,F,B0);
                  end;
                  KsiTT = KsitT(:,g);
               else;
                  if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
                     Yerror = Yerror(:,T);
                     SigmaSqRoot = SigmaSqRoot(:,:,T);
                     InvMSEY = InvMSEY(:,:,T);
                     KalmanGain = KalmanGain(:,:,T);
                  end;
                  [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H(:,:,DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
                  KsiTTz = KsitT(:,1);
                  PTTz = PtT(:,:,1);
                  KsiTT = KsitT(:,1);
               end;
               PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
            else;
               %
               % use posterior mode or initial values as a backup
               %
               A = Amode;
               H = Hmode;
               R = Rmode;
               F = Fmode;
               B0 = B0mode;
               KsiTTz = KsiTTzmode;
               PTTz = PTTzmode;
            end;
         else;
            %
            % use posterior mode or initial values as a backup
            %
            A = Amode;
            H = Hmode;
            R = Rmode;
            F = Fmode;
            B0 = B0mode;
            KsiTTz = KsiTTzmode;
            PTTz = PTTzmode;
         end;
         %
         % stack the model to determine the shock distribution
         %
         Xbar = zeros(n*g,1);
         G = zeros(n*g,r);
         D = zeros(n*g,q*g);
         Fexp = F;
         Q = B0*B0';
         QM = B0*M*M'*B0';
         Fg = eye(r);
         for i=1:g;
            Ind = (1+((g-i)*n):(g-i+1)*n);
            Ind1 = (1+((g-i)*q):(g-i+1)*q);
            if k>0;
               Xbar(Ind,1) = A'*X(:,i);
            end;
            G(Ind,:) = H(:,:,DSGEModel.LastPeriod+i)'*Fexp;
            D(Ind,Ind1) = H(:,:,DSGEModel.LastPeriod+i)'*B0;
            Fi = F;
            for j=1:i-1;
               Ind1 = Ind1+q;
               D(Ind,Ind1) = H(:,:,DSGEModel.LastPeriod+i)'*Fi*B0;
               Fi = F*Fi;
            end;
            Fexp = F*Fexp;
         end;
         %
         % speed-up code suggested by Ingvar Strid
         %
         KpD = K'*D*IgM;
         KpDDpK = KpD*KpD';
         DpKInvKpDDpK = KpD'/KpDDpK;
         %
         % compute the conditioning relation when state vector and measurement errors are at mean
         %
         kbarTg = ZTg-UT-(K'*(Xbar+(G*KsiTTz)));
         %
         % compute the mean of the shocks when state vector and measurement errors are at mean
         %
         mubarTg = DpKInvKpDDpK*kbarTg;
         %
         % compute the idempotent covariance matrix for the shocks
         %
         SigmaTg = eye(q_r*g)-(DpKInvKpDDpK*KpD);
         SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
         Dbar = eye(n*g)-(D*IgM*DpKInvKpDDpK*K');
         CovShock = (D*IgM*SigmaTg*IgM'*D')+(Dbar*D*IgMMorth*D'*Dbar');
         CovState = Dbar*G*PTTz*G'*Dbar';
         if rank(R)>0;
            IgR = kron(eye(g),R);
            CovME = Dbar*IgR*Dbar';
         else;
            CovME = 0;
         end;
         Bbar = zeros(r,q*g);
         %
         % compute the mean prediction at the given parameter values
         %
         PredMean = zeros(n,h);
         KsiTm = KsiTTz;
         etaShocksm = zeros(q,h);
         etaShocksm(DSGEModel.MixedConditionalShocks,1:g) = fliplr(reshape(mubarTg,q_r,g));
         for i=1:g;
            %
            % update the states
            %
            KsiTm = (F*KsiTm)+(B0*etaShocksm(:,i));
            %
            % compute the conditional mean predictions
            %
            if k>0;
               PredMean(:,i) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiTm);
            else;
               PredMean(:,i) = H(:,:,DSGEModel.LastPeriod+i)'*KsiTm;
            end;
            Ind1 = (1+((g-i)*n):(g-i+1)*n);
            Ind2 = (1+(i-1)*q):(i*q);
            Moments(ParamIndex).StateCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovState(Ind1,Ind1))))+tril(CovState(Ind1,Ind1),-1)+tril(CovState(Ind1,Ind1),-1)');
            Moments(ParamIndex).ShockCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovShock(Ind1,Ind1))))+tril(CovShock(Ind1,Ind1),-1)+tril(CovShock(Ind1,Ind1),-1)');
            if rank(R)>0;
               Moments(ParamIndex).MeasureCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*(diag(abs(diag(CovME(Ind1,Ind1))))+tril(CovME(Ind1,Ind1),-1)+tril(CovME(Ind1,Ind1),-1)');
            end;
            Bbar(:,Ind2) = Fg*B0;
            Moments(ParamIndex).ParameterCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)';
            Fg = Fg*F;
            if ComputePITMoment==1;
               if rank(R)>0;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = CovState(Ind1,Ind1)+CovShock(Ind1,Ind1)+CovME(Ind1,Ind1);
               else;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = CovState(Ind1,Ind1)+CovShock(Ind1,Ind1);
               end;
            end;
         end;
         for i=g+1:h;
            %
            % update the states
            %
            KsiTm = (F*KsiTm);
            %
            % compute the conditional mean predictions
            %
            if k>0;
               PredMean(:,i) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiTm);
            else;
               PredMean(:,i) = H(:,:,DSGEModel.LastPeriod+i)'*KsiTm;
            end;
            if i==(g+1);
               Kbar = Bbar*IgM*DpKInvKpDDpK*K';
               Gbar = Fg-(Kbar*G);
               Nbar = eye(q*g)-(IgM*DpKInvKpDDpK*K'*D);
               Fig = eye(r);
               FjSum = zeros(r,r);
            end;
            FjSum = FjSum+(Fig*Q*Fig');
            Fig = Fig*F;
            Moments(ParamIndex).StateCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H(:,:,DSGEModel.LastPeriod+i)'*Fig*Gbar*PTTz*Gbar'*Fig'*H(:,:,DSGEModel.LastPeriod+i);
            %
            % compute the shock covariance
            %
            Moments(ParamIndex).ShockCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*H(:,:,DSGEModel.LastPeriod+i)'*(FjSum+(Fig*Bbar*((IgM*SigmaTg*IgM')+(Nbar*IgMMorth*Nbar'))*Bbar'*Fig'))*H(:,:,DSGEModel.LastPeriod+i);
            if rank(R)>0;
               Moments(ParamIndex).MeasureCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*((H(:,:,DSGEModel.LastPeriod+i)'*Fig*Kbar*IgR*Kbar'*Fig'*H(:,:,DSGEModel.LastPeriod+i))+R);
            end;
            Moments(ParamIndex).ParameterCov(:,:,i) = (Weights(ParamIndex)/NumDraws)*PredMean(:,i)*PredMean(:,i)';
            if ComputePITMoment==1;
               if rank(R)>0;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = (H(:,:,DSGEModel.LastPeriod+i)'*Fig*Gbar*PTTz*Gbar'*Fig'*H(:,:,DSGEModel.LastPeriod+i))+(H(:,:,DSGEModel.LastPeriod+i)'*(FjSum+(Fig*Bbar*((IgM*SigmaTg*IgM')+(Nbar*IgMMorth*Nbar'))*Bbar'*Fig'))*H(:,:,DSGEModel.LastPeriod+i))+((H(:,:,DSGEModel.LastPeriod+i)'*Fig*Kbar*IgR*Kbar'*Fig'*H(:,:,DSGEModel.LastPeriod+i))+R);
               else;
                  PITMoments(ParamIndex).PredictYCov(:,:,i) = (H(:,:,DSGEModel.LastPeriod+i)'*Fig*Gbar*PTTz*Gbar'*Fig'*H(:,:,DSGEModel.LastPeriod+i))+(H(:,:,DSGEModel.LastPeriod+i)'*(FjSum+(Fig*Bbar*((IgM*SigmaTg*IgM')+(Nbar*IgMMorth*Nbar'))*Bbar'*Fig'))*H(:,:,DSGEModel.LastPeriod+i));
               end;
            end;
         end;
         Moments(ParamIndex).MeanPredictY = (Weights(ParamIndex)/NumDraws)*PredMean;
         if ComputePITMoment==1;
            PITMoments(ParamIndex).PredictY = PredMean;
         end;
         drawnow;
      end;
   end;
   %
   % make sure that the Moments structure data are used to compute moments
   %
   for ParamIndex=1:NumDraws;
      StateCov = StateCov+Moments(ParamIndex).StateCov;
      ShockCov = ShockCov+Moments(ParamIndex).ShockCov;
      MeasureCov = MeasureCov+Moments(ParamIndex).MeasureCov;
      ParameterCov = ParameterCov+Moments(ParamIndex).ParameterCov;
      MeanPredictY = MeanPredictY+Moments(ParamIndex).MeanPredictY;
   end;
   %
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
if StopEstimation==1;
   return;
end;
for i=1:h;
   ParameterCov(:,:,i) = ParameterCov(:,:,i)-(MeanPredictY(:,i)*MeanPredictY(:,i)');
   ParameterCov(:,:,i) = diag(abs(diag(ParameterCov(:,:,i))))+tril(ParameterCov(:,:,i),-1)+tril(ParameterCov(:,:,i),-1)';
end;
if IsPosterior==1;
   MatFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-CondPredictiveMoments-Subset-' DSGEModel.NameOfModel '-' SmpStr '-' int2str(CurrChain) '-' int2str(h) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
else;
   MatFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-CondPredictiveMoments-Subset-Prior-' DSGEModel.NameOfModel '-' SmpStr '-' int2str(h) '.' int2str(NumDraws) '.mat']);
end;
save(MatFile,'MeanPredictY','MeasureCov','ShockCov','StateCov','ParameterCov','PITMoments');
DoneCalc = 1;

%
% end of DSGECondPredictiveMomentsMixed.m
%
